/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          Animation.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'Animation'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _ANIMATION_H
#define _ANIMATION_H

//@START_USER1
#include "MediaEvaluable.h"


#include <osg/Matrixf>
#include <osg/Matrixd>
#include <osg/Quat>
#include <osg/NodeCallback>
#include <osg/AnimationPath>

#include <map>
#include <float.h>


#include "XMLUtil.h"
#include "LoaderGeneric.h"

namespace cooki3d{
//@END_USER1


/*@NOTE_161
classe ...
*/

class AFX_EXT_CLASS Animation
    : public MediaEvaluable
{

//@START_USER2
  public :
    typedef map<string,Animation*,less<string > > mapStrAnim;
    
//@END_USER2

//
// Group: ClassBuilder methods
//

private:
    void ConstructorInclude();
    void DestructorInclude();

//
// Group: Media
//

public:
    virtual void setPosition(double sec);
    virtual bool isPlay();
    virtual bool isPause();
    virtual double eval(std::string methode, double args[10], int nbParam);
    double getDebut();
    virtual double getDuration();
    double getFin();
    virtual void pause();
    virtual void play();
    virtual double getPosition();
    virtual void setDuration(double duree);
    virtual void stop();

//
// Group: Modes
//

public:
    std::string getLoopMode();
    void setLoopMode(std::string mode);
    std::string traduireLoopModeFromEnumToString(osg::AnimationPath::LoopMode mode);
    osg::AnimationPath::LoopMode traduireLoopModeFromStringToEnum(std::string mode);

//
// Group: load
//

public:
    void init();
    static bool linkAnimation(osg::Node* n, const std::string& pickName,
                              LoaderGeneric* loader);
    static Evaluable* staticCreate(BD* pBD);
    bool loadXML(const xmlpp::Node* node, LoaderXML* loaderXML);
    virtual void setApc(osg::AnimationPathCallback* pApc);
    static bool checkAllInstances(LoaderGeneric* loader);
    virtual bool check();
    Animation(BD* pBD);
    Animation(Animation& anim);
    virtual ~Animation();

//
// Group: zdata
//

protected:
    osg::AnimationPathCallback* _apc;
    bool _automaticSet;
    std::string _bufferLoopMode;
    int _debut;
    int _fin;
    int _fps;
    std::string _nodeName;
    osg::AnimationPath* _path;
    osg::AnimationPath* _pathInit;

public:
    osg::AnimationPathCallback* getApc() const;
    bool getAutomaticSet() const;
    void setAutomaticSet(bool automaticSet);
    std::string getBufferLoopMode() const;
    int getFps() const;
    void setFps(int fps);
    std::string getNodeName() const;
    void setNodeName(std::string nodeName);

//
// Non-Grouped Members
//

//
// Non-Grouped Methods
//

public:
    virtual bool isAtomic();
};

#endif


#ifdef CB_INLINES
#ifndef _ANIMATION_H_INLINES
#define _ANIMATION_H_INLINES


//
// Group: zdata
//
/*@NOTE_1029
Returns the value of member '_apc'.
*/
inline osg::AnimationPathCallback* Animation::getApc() const
{//@CODE_1029
    return _apc;
}//@CODE_1029



/*@NOTE_4375
Returns the value of member '_automaticSet'.
*/
inline bool Animation::getAutomaticSet() const
{//@CODE_4375
    return _automaticSet;
}//@CODE_4375



/*@NOTE_4376
Set the value of member '_automaticSet' to 'automaticSet'.
*/
inline void Animation::setAutomaticSet(bool automaticSet)
{//@CODE_4376
    _automaticSet = automaticSet;
}//@CODE_4376



/*@NOTE_3931
Returns the value of member '_bufferLoopMode'.
*/
inline std::string Animation::getBufferLoopMode() const
{//@CODE_3931
    return _bufferLoopMode;
}//@CODE_3931



/*@NOTE_1020
Returns the value of member '_fps'.
*/
inline int Animation::getFps() const
{//@CODE_1020
    return _fps;
}//@CODE_1020



/*@NOTE_1021
Set the value of member '_fps' to 'fps'.
*/
inline void Animation::setFps(int fps)
{//@CODE_1021
    _fps = fps;
}//@CODE_1021



/*@NOTE_1014
Returns the value of member '_nodeName'.
*/
inline std::string Animation::getNodeName() const
{//@CODE_1014
    return _nodeName;
}//@CODE_1014



/*@NOTE_1015
Set the value of member '_nodeName' to 'nodeName'.
*/
inline void Animation::setNodeName(std::string nodeName)
{//@CODE_1015
    _nodeName = nodeName;
}//@CODE_1015



// End of Group: zdata

//@START_USER3
} //end of namespace cooki3d
//@END_USER3

#endif
#endif
