/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          AnimationGroup.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'AnimationGroup'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include "Media.h"
#include "BD.h"
#include "AnimationGroup.h"
#include "XMLUtil.h"
#include "EnvironnementOSG.h"
#include "MapRW.h"

namespace cooki3d {
//@END_USER2



//
// Group: load
//
// Static members
MapRegister<AnimationGroup*> AnimationGroup::_register;
// End of Group: load





//
// Group: load
//
bool AnimationGroup::linkAnimationGroup(osg::Node* n,
                                        const std::string& pickName,
                                        LoaderGeneric* loader)
{//@CODE_3888

  bool res = false;
  
  //parcours de tous les AnimationGroup
  Register<AnimationGroup*> & reg = _register.getRegister(loader->getBD());
  Register<AnimationGroup*>::iterator it  = reg.begin();
  
  AnimationGroup * ag=NULL;
  std::string animName;
  MapRW map;

  while (it != reg.end() ){
    
    ag = it->second;
      
    //si les conditions nécéssaire sont remplies:
    //1. le nom de la zone a une correspondance
    //2. le node a un CallBack
    //3. ce CallBack est bien une animation
    if ( ag->_nodeNameList.findZone( pickName ) 
	 && n->getUpdateCallback()
	 && dynamic_cast<osg::AnimationPathCallback *>(n->getUpdateCallback())
	 )
      {
	
	Animation * anim= new Animation( *( (Animation*)ag) );
	assertC2( anim != ag, loader->getBD() );

	animName = ag->getName()+"#";	
	map.writeToStr ( ag->_animationList.size() , animName);

	anim->setNameWithoutPrefix( animName );
	anim->setNodeName( pickName );

	CINFOs<<"AnimationGroup => new (sub)Animation \""<<anim->getName()
	      <<"\" (path \""<<anim->getNodeName()<<"\",adr "<<anim<<")"<<endl;
	
	//vérification constructeur copie par défaut
	//assertC(   anim->getDebut() == ag->getDebut() ); 
	//assertCMessage (  anim->getFin() == ag->getFin(),
	//		  anim->getFin()<<", "<<ag->getFin()); ?? 
	// BUG? : 3.33332 != 3.33332 !!!
	//assertC(  anim->getFps() == ag->_fps );
	//assertC(  anim->getBufferLoopMode() == ag->_bufferLoopMode );
	//assertC(  anim->getPlayOnStart() == ag->getPlayOnStart() );

	assertC2( anim->getNodeName() == pickName , loader->getBD());
	assertC2( anim->isAtomic() , loader->getBD());

	assertC2( Animation::linkAnimation( n, pickName,loader ) == true , 
		  loader->getBD());

	ag->_animationList.push_back( anim );

	res = true;
      }
    it++;
  }

  return res;
}//@CODE_3888


bool AnimationGroup::loadXML(const xmlpp::Node* node, LoaderXML* loaderXML)
{//@CODE_3891
  
  Animation::loadXML(node , loaderXML );
  
  //recuperation des noeuds actifs
  _nodeNameList.loadXML( node );

  
  
  return load();
}//@CODE_3891


Evaluable* AnimationGroup::staticCreate(BD* pBD)
{//@CODE_3904
  
  return new AnimationGroup(pBD);
  
}//@CODE_3904


/*@NOTE_3872
Constructor method.
*/
AnimationGroup::AnimationGroup(BD* pBD) //@INIT_3872
  :  Animation(pBD)
    ,_animationList()
    , _nodeNameList()
{//@CODE_3872
    ConstructorInclude();
  
    
    // Put in your own code	
}//@CODE_3872


/*@NOTE_3847
Destructor method.
*/
AnimationGroup::~AnimationGroup()
{//@CODE_3847
    DestructorInclude();

    _register.getRegister( getBD() ).remove( getName() );
}//@CODE_3847


/*@NOTE_3922
Check the integritry of the object
*/
bool AnimationGroup::check()
{//@CODE_3922
  //check is OK for Animation check
    return true;
}//@CODE_3922


bool AnimationGroup::load()
{//@CODE_3930
  
  _register.getRegister( getBD() ).add( getName(), this );
  
  return true;
}//@CODE_3930


// End of Group: load


//
// Group: Media
//
void AnimationGroup::setPosition(double sec)
{//@CODE_3873
  
 for (AnimationList::iterator it =  _animationList.begin();
      it != _animationList.end();
      it++){
   
   (*it)->setPosition( sec );
   
 }


}//@CODE_3873


bool AnimationGroup::isPlay()
{//@CODE_3877
  
  CWARN(this,"isPlay")<<" return the play state "
			   <<" of the fist animation in the group"<<endl;
  return (*(_animationList.begin()) )->isPlay();  


}//@CODE_3877


bool AnimationGroup::isPause()
{//@CODE_3878

  CWARN(this,"isPause")<<" return the pause state "
			   <<" of the fist animation in the group"<<endl;
  return (*(_animationList.begin()) )->isPause();  

}//@CODE_3878


double AnimationGroup::eval(std::string methode, double args[10], int nbParam)
{//@CODE_3879
  return Media::eval(methode,args,nbParam);
}//@CODE_3879


double AnimationGroup::getDuration()
{//@CODE_3884
 
  CWARN(this,"getDuration")<<" return the duration "
			   <<" of the fist animation in the group"<<endl;
  
  return (*(_animationList.begin()) )->getDuration();
  
}//@CODE_3884


void AnimationGroup::pause()
{//@CODE_3894
  for (AnimationList::iterator it =  _animationList.begin();
       it != _animationList.end();
       it++){
    
    (*it)->pause();

  }
  
}//@CODE_3894


void AnimationGroup::play()
{//@CODE_3895
  
  for (AnimationList::iterator it =  _animationList.begin();
       it != _animationList.end();
       it++){
    
    (*it)->play();

  }
  
}//@CODE_3895


double AnimationGroup::getPosition()
{//@CODE_3896

  CWARN(this,"getPosition")<<" return the position "
			   <<" of the fist animation in the group"<<endl;
  
  return (*(_animationList.begin()) )->getPosition();
  
}//@CODE_3896


void AnimationGroup::setDuration(double duree)
{//@CODE_3900
  
  for (AnimationList::iterator it =  _animationList.begin();
       it != _animationList.end();
       it++){
    
    (*it)->setDuration(duree);
    
  }
  
}//@CODE_3900


void AnimationGroup::stop()
{//@CODE_3907

  for (AnimationList::iterator it =  _animationList.begin();
       it != _animationList.end();
       it++){
    
    (*it)->stop();

  }

  
}//@CODE_3907


// End of Group: Media

bool AnimationGroup::isAtomic()
{//@CODE_3924
  //this is group, not an atomic animation !
    return false;
}//@CODE_3924


//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_3848
Method which must be called first in a constructor.
*/
void AnimationGroup::ConstructorInclude()
{
}


/*@NOTE_3849
Method which must be called first in a destructor.
*/
void AnimationGroup::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
//attention CB fait parfois sauter ce crochet fermant...
} //end of namespace cooki3d
