/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          AnimationGroup.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'AnimationGroup'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _ANIMATIONGROUP_H
#define _ANIMATIONGROUP_H

//@START_USER1
#include "MediaEvaluable.h"


#include <osg/Matrixf>
#include <osg/Matrixd>
#include <osg/Quat>
#include <osg/NodeCallback>
#include <osg/AnimationPath>

#include <vector>
#include <float.h>


//#include <libxml++/libxml++.h> 
#include "XMLUtil.h"

#include "Animation.h"
#include "NodeNameList.h"
#include "MapRegister.h"

namespace cooki3d{
//@END_USER1


/*@NOTE_3846
classe ...
*/

class AFX_EXT_CLASS AnimationGroup
    : public Animation
{

//@START_USER2
public :
  typedef std::vector<Animation *>  AnimationList;
  
//@END_USER2

//
// Group: ClassBuilder methods
//

private:
    void ConstructorInclude();
    void DestructorInclude();

//
// Group: load
//

private:
    static MapRegister<AnimationGroup*> _register;

public:
    static bool linkAnimationGroup(osg::Node* n, const std::string& pickName,
                                   LoaderGeneric* loader);
    bool loadXML(const xmlpp::Node* node, LoaderXML* loaderXML);
    static Evaluable* staticCreate(BD* pBD);
    AnimationGroup(BD* pBD);
    virtual ~AnimationGroup();
    virtual bool check();
    bool load();

//
// Group: Media
//

public:
    virtual void setPosition(double sec);
    virtual bool isPlay();
    virtual bool isPause();
    virtual double eval(std::string methode, double args[10], int nbParam);
    virtual double getDuration();
    virtual void pause();
    virtual void play();
    virtual double getPosition();
    virtual void setDuration(double duree);
    virtual void stop();

//
// Non-Grouped Members
//

private:
    AnimationGroup::AnimationList _animationList;
    NodeNameList _nodeNameList;

//
// Non-Grouped Methods
//

public:
    virtual bool isAtomic();
    AnimationGroup::AnimationList getAnimationList() const;
    void setAnimationList(AnimationGroup::AnimationList animationList);
    const NodeNameList& getNodeNameList() const;
    void setNodeNameList(const NodeNameList& rNodeNameList);
};

#endif


#ifdef CB_INLINES
#ifndef _ANIMATIONGROUP_H_INLINES
#define _ANIMATIONGROUP_H_INLINES

/*@NOTE_3915
Returns the value of member '_animationList'.
*/
inline AnimationGroup::AnimationList AnimationGroup::getAnimationList() const
{//@CODE_3915
    return _animationList;
}//@CODE_3915



/*@NOTE_3916
Set the value of member '_animationList' to 'animationList'.
*/
inline void AnimationGroup::setAnimationList(AnimationGroup::AnimationList animationList)
{//@CODE_3916
    _animationList = animationList;
}//@CODE_3916



/*@NOTE_3919
Returns the value of member '_nodeNameList'.
*/
inline const NodeNameList& AnimationGroup::getNodeNameList() const
{//@CODE_3919
    return _nodeNameList;
}//@CODE_3919



/*@NOTE_3920
Set the value of member '_nodeNameList' to 'rNodeNameList'.
*/
inline void AnimationGroup::setNodeNameList(const NodeNameList& rNodeNameList)
{//@CODE_3920
    _nodeNameList = rNodeNameList;
}//@CODE_3920



//@START_USER3
} //end of namespace cooki3d
//@END_USER3

#endif
#endif
