/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          BoundingShape.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'BoundingShape'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include <osg/Geode>
#include <osg/MatrixTransform>
#include <osg/NodeVisitor>
#include <osg/PositionAttitudeTransform>
#include <osg/AnimationPath>
#include <osg/io_utils>
//#include <osg/Matrix>

#include "BoundingShape.h"
#include "Evaluable.h"
#include "Methode_1arg.h"
#include "ScanSetGraph.h"
#include "EnvironnementOSG.h"
#include "BD.h"
#include "MapRW.h"


namespace cooki3d{



///Structure callback appelé par osg lors du drawImplementation
///Ce callback sert à personnaliser la phase de dessin
struct BoundingShapeDrawCallback : public virtual osg::Drawable::DrawCallback
{
  
  BoundingShape * _bsha;
  bool _firstDraw;
  
  BoundingShapeDrawCallback(BoundingShape * obj) : 
    _bsha(obj), _firstDraw(true) {        
  }
  
  /** do customized draw code.*/
  virtual void drawImplementation(osg::State& state,
				  const osg::Drawable* d) const {
    
    if (_bsha->isDebug()){
      //si l'on veut voir la bounding shape enfant...
      d->drawImplementation(state);
    }
  
    // GLfloat projection[16];
//   glGetFloatv( GL_PROJECTION_MATRIX, projection);
//   osg::Matrixf matProj;
//   matProj.set( projection);
  

//   GLfloat modelView[16];
//   glGetFloatv(GL_MODELVIEW_MATRIX , modelView);
//   osg::Matrixf matView;
//   matView.set( modelView);
  

//   //cout<<"MODELVIEW="<<matView<<endl;
//   cout<<"BOUNDING SHAPE PROJ="<<matProj<<endl;
//   cout<<"BOUNDING SHAPE PROJ BIS ="<<state.getProjectionMatrix()<<endl;

    assertC2(_bsha-> getListChild() == NULL, _bsha->getBD() );
    
    //on indique que cette Box sera active pour la frame courante
    CTRACEdebug<<"_bsha="<<_bsha<<" "<<_bsha->getName()<<endl;
    CTRACEdebug<<" BoundingShape::_lastFrameNumber="
	<<BoundingShape::_lastFrameNumber<<endl;
    _bsha->setFrameActive(); 
    
    //cerr<<"drawImplementation "<<_bsha->getName()<<endl;
    
    //calcul de la position de la bounding box enfant...
    //osg::Matrix mPoint;
    //osg::Vec3 res;
    osg::Matrix rTransfoShape = 
      state.getModelViewMatrix() * state.getInitialInverseViewMatrix() ;

    
    CTRACEdebug<<"rTransfoShape="<<rTransfoShape<<endl;

        
    //calcul de la nouvelle boite un fois déplacée
    //NOTE:
    //une produit matTransfo*vecteurPoint n'est pas possible
    //car la matrice n'est pas dans le bon sens pour cela...
    //l'astuce est de transformer le point en une translation.     
    //et de faire la multiplication inverse, matPoint* matTransfo    
    /*mPoint.makeTranslate( _bsha->getBoxInit()._min );  
    osg::Vec3 min = ( mPoint * rTransfoShape ).getTrans();        
    mPoint.makeTranslate( _bsha->getBoxInit()._max );  
    osg::Vec3 max = ( mPoint * rTransfoShape ).getTrans(); */       

    //Revient à :
    //osg::Vec3 min = _bsha->getBoxInit()._min * rTransfoShape; //moitié faux
    //osg::Vec3 max = _bsha->getBoxInit()._max * rTransfoShape; //moitié faux  
    //osg::Vec3 center = _bsha->getBoxInit().center() * rTransfoShape; //OK   
    
    //approximation des nouvelles extrémitées, pour une box à plat    
    osg::Vec3 min;
    osg::Vec3 max;
    osg::Vec3 tmp;
    for (int i=0; i<8; i++){
      tmp = _bsha->getBoxInit().corner( i ) * rTransfoShape;
      if ( i == 0 ){
	min = max = tmp;
      } else {
	for (int j=0; j<3; j++){
	  if (tmp[j] < min[j]){
	    min[j] = tmp[j];
	  }
	  if (tmp[j] > max[j]){
	     max[j] = tmp[j];
	  }
	}
// 	if (tmp.y()<min.y() ||tmp.z()<min.z()){
// 	  min = tmp;
// 	}
// 	if (tmp.x()>max.x() || tmp.y()>max.y() || tmp.z()>max.z()){
// 	  max = tmp;
// 	}
      }
    }

    
    //CTRACEdebug<<"mPoint ="<<mPoint<<endl;
    //CTRACEdebug<<"old center ="<<_bsha->getBoxInit().center()<<endl;
    //CTRACEdebug<<"new center ="<<center<<endl;
    //     CTRACEdebug<<"old min ="<<_bsha->getBoxInit()._min<<endl;
    //     CTRACEdebug<<"new min ="<<min<<endl;
    //     CTRACEdebug<<"old max ="<<_bsha->getBoxInit()._max<<endl;
    //     CTRACEdebug<<"new max ="<<max<<endl;
    
    
    osg::Quat q;    
    rTransfoShape.get( q ); 

    //appel virtuel si héritage prévu par une autre api
    //on enregistre les coordonnées de la boite un fois déplacée
    _bsha->setBoxComputed(min, max, q);

    //appel virtuel si héritage prévu par une autre api
    //on enregistre la dernière transforamtion utilisée
    _bsha->setTransfoShape( rTransfoShape );
    	   
  }
};

osg::Matrix mInitIdentityRef;

inline void expandByOptimisedCall(osg::BoundingBox& bthis, 
				  const osg::BoundingBox& bb,
				  BD* pBD)
{
  CTRACEs("expandByOptimisedCall")<<endl;
    if (!bb.valid()) {
      assertC2( 0, pBD );
      return;
    }

    if(bb._min.x()<bthis._min.x()) bthis._min.x() = bb._min.x();
    if(bb._max.x()>bthis._max.x()) bthis._max.x() = bb._max.x();

    if(bb._min.y()<bthis._min.y()) bthis._min.y() = bb._min.y();
    if(bb._max.y()>bthis._max.y()) bthis._max.y() = bb._max.y();

    if(bb._min.z()<bthis._min.z()) bthis._min.z() = bb._min.z();
    if(bb._max.z()>bthis._max.z()) bthis._max.z() = bb._max.z();
} 

//@END_USER2



//
// Group: data
//
// Static members
int BoundingShape::_lastFrameNumber = 0;
// End of Group: data





//
// Group: data
//
void BoundingShape::setBoxComputed(const osg::Vec3& min, const osg::Vec3& max,
                                   const osg::Quat& q)
{//@CODE_3331

  //(max - min).length() / 2;
  
  _boxComputed.set( min, max );
  //_boxComputed.setRotation( q );

  //CTRACEdebug<<"min="<<min.x()<<", "<<min.y()<<", "<<min.z()<<endl;
  CTRACEdebug<<"max="<<max.x()<<", "<<max.y()<<", "<<max.z()<<endl;

}//@CODE_3331


// End of Group: data


//
// Group: load
//
/*@NOTE_3251
Check the integritry of the object 
(if the boudingShape is linked with osg::Node, ...)
*/
bool BoundingShape::check()
{//@CODE_3251
  bool resultOk = _node!=NULL ;

    if (!resultOk){
      CERR(this,"check")<<getName()<<":"<<getNodeName()
			<<" is not linked !!"<<endl;
    }
    
    return resultOk;
}//@CODE_3251


bool BoundingShape::loadXML(const xmlpp::Node* node, LoaderXML* loaderXML)
{//@CODE_3232
    
  XMLUtil xml(node);
  
  setName(xml.getFirstText("nom"));

  //setNodeMask
  sscanf(xml.getFirstText("nodeMask").c_str(), "%x", &_nodeMask);
  
  setNodeName(xml.getFirstText("nodeName"));
  CTRACE(this,"loadXML")<<"nodeName ="<<_nodeName<<endl;
  
  float scale = atof(xml.getFirstText("scaleAdjust").c_str());
  setScaleAdjust(osg::Vec3(scale,scale,scale));
  
  setDebug(xml.getFirstText("debug")=="true");
  
  setType(xml.getFirstText("type"));

  setDebugGlobal(xml.getFirstText("debugGlobal")=="true");
  
  return true;
}//@CODE_3232


bool BoundingShape::checkAllInstances(LoaderGeneric* loader)
{//@CODE_3248

  BD::EvaluableIterator it(loader->getBD());
  
  BoundingShape * bs;

  bool resultOk = true;
   
  while (++it) {
    
    bs = dynamic_cast<BoundingShape *>(it.Get());
    
    if (bs) {
      resultOk = resultOk && bs->check();
    }

  }
   
  return resultOk;
}//@CODE_3248


bool BoundingShape::linkBoundingShape(osg::Node* n, osg::Node* parent,
                                      const std::string& pickName,
                                      LoaderGeneric* loader)
{//@CODE_3249

  BD::EvaluableIterator itBsha( loader->getBD() );
  //std::string pickName = n->getName();
  BoundingShape * bsha;
  bool res=false;

  while (++itBsha) {

    bsha = dynamic_cast<BoundingShape *>(itBsha.Get());
    
    if (bsha && (bsha->getNodeName() == pickName)
	&& !bsha->getNode() // si le noeud n'est pas encore relié
	) {
      CINFOs<<"BoundingShape link \""
	    <<bsha->getNodeName()<<"\"("<<bsha->getName()
	    <<") ..."<<endl;
      //on set le node.
      bsha->setNode(n);
      bsha->linkRecursively(n,parent);
      
      //cas d'un debugGlobal sur un noeud group...
      if ( bsha->isDebugGlobal() /*&& bsha->_listChild != NULL*/ ) {

	//assertC(0);

	bsha->_sd =
	  new osg::ShapeDrawable( new osg::Box(osg::Vec3(0,0,0), 10 ) );
	
	osg::Geode * grp= new osg::Geode();
	grp->setName("boundingShapeDbgGlobal");
	grp->setNodeMask(bsha->_nodeMask);
	bsha->_sd->setUseDisplayList(false);
	bsha->_sd->setColor(osg::Vec4( 1, 255, 1,200));	
	
	grp->addDrawable( bsha->_sd );
  
	loader->getRootNode()->addChild(grp); 
      }
    }
    
  }
  return res;
  
}//@CODE_3249


void BoundingShape::linkRecursively(osg::Node * n, osg::Node* parentNode)
{//@CODE_3257
  
  //on set le node.
  //this->setNode(n);
  
  CTRACE(this,"linkRecursively")<< this->getName() << "(" << this  << ")"
				<<" avec "<< n->getName()<<endl;
  
  
  if (n->asGroup() ) {
    
    if (n->asGroup()->getNumChildren() == 0) {
      return;
    }    
    
    BoundingShape * bshapChild = NULL;
    
    int num=n->asGroup()->getNumChildren();
    std::string sI;
    MapRW mrw;
    
    for (int i=0; i<num;i++) {
      
      //if ( this->_fatherDesac == NULL ) {
      
      //si c'est une geode, autement dit ce n'est pas un group
      if ( ! ( n->asGroup()->getChild(i)->asGroup() ) ) {

	assertC( dynamic_cast<osg::Geode*>( n->asGroup()->getChild(i)) );
	
	//on crée le fils, on set ses propriétés à partir du père
	bshapChild = this->clone();
	sI ="";
	mrw.writeToStr( i, sI );
	bshapChild->setName( "sub"+sI+this->getName() );
	bshapChild->setType( this->getType() );
	bshapChild->setDebug( this->isDebug() );
	bshapChild->setNodeMask( this->getNodeMask() );
	bshapChild->setNode( n->asGroup()->getChild(i) );

	
	//cas : SpecializedOsgSequence, osg::Transform ...
	//dans ces cas, on va créer une BoundingShape par enfant
	//rassemblées dans une liste de fils
	//assertC( this->_listChild == NULL);
	if ( this->_listChild == NULL ){
	  //	 && this->_fatherDesac == NULL){
	  this->_listChild = new std::vector<BoundingShape*>;
	}

	//on stock dans la liste des fils
	this->_listChild->push_back( bshapChild );
	
	//appel récursif
	bshapChild->linkRecursively( n->asGroup()->getChild(i), n );
      }      
      else {
      
	//appel récursif
	this->linkRecursively( n->asGroup()->getChild(i), n );
      }
      
    }   
    return;
  }

  assertC( n == this->getNode() );

  //on sait qu'a ce point du program, "n" doit être forcement une geode.
  osg::Geode * firstGeode = dynamic_cast<osg::Geode*>( n );

  //on créer une liaison avec la geode
  linkSimply( firstGeode, parentNode );
  
}//@CODE_3257


void BoundingShape::linkSimply(osg::Geode* geode, osg::Node* parentNode)
{//@CODE_3286
  
  //la geode ne doit jamais être NULL
  assertC ( geode );
  //le pere ne doit jamais etre NULL pour une geode
  assertC ( parentNode );

  //on set le node.
  this->setNode(geode);
  
  CTRACE(this,"linkSimply")<<this->getName()<<" avec "
			   <<geode->getName()<<endl;
  
  
  //la geode peut avoir plusieurs parents
  //ces pourquoi on ne peut déterminer le père à partir de celle ci!!
  //c'est pour cela qu'on a le paramètre "parentNode" !!
  
  
  //vérifications d'usage pour l'algo
  assertC ( geode->getNumParents() > 0 );
 

  CTRACE(this,"linkSimply")<<" link \""<<this->getNodeName()
	<<"\"("<<this->getName()<<") on osg::Geode \""
	<<geode->getName()<<"\""<<endl;

  //boite initiale
  _boxInit = ((osg::Geode*)geode)->getBoundingBox();

  CTRACEnext<<"###############################"<<endl;
  CTRACEnext<<"this="<<this<<" Imin.x() ="<<this->_boxInit._min.x()
	    <<",Imax.x() ="<<this->_boxInit._max.x()<<endl;
  CTRACEnext<<"this="<<this<<" Imin.y() ="<<this->_boxInit._min.y()
	    <<",Imax.y() ="<<this->_boxInit._max.y()<<endl;
  CTRACEnext<<"this="<<this<<" Imin.z() ="<<this->_boxInit._min.z()
	    <<",Imax.z() ="<<this->_boxInit._max.z()<<endl;
  CTRACEnext<<"###############################"<<endl;
  
  if (this->getType() == "Sphere"){	 

    //TODO Traiter plusieurs niveau de profondeur (intersection)
    osg::Sphere * _dbgShapeSphere=new osg::Sphere(osg::Vec3(0,0,0),30);
    this->_sd=new osg::ShapeDrawable(_dbgShapeSphere);
   
    
    osg::BoundingSphere bsphere = geode->getBound();
    
    //osg::Sphere* sph = dynamic_cast<osg::Sphere*>( this->_sd->getShape() );
    /*sph*/_dbgShapeSphere->setRadius(bsphere.radius());
    /*sph*/_dbgShapeSphere->setCenter(bsphere.center());
    
  }
  
  if (this->getType() == "Box"){   
    
	  
    CTRACE(this,"linkSimply")
      <<"link Box : nodeName="<<this->_nodeName<<endl;
    
    osg::Box * _dbgShapeBox = new osg::Box; //(pas de constructeur adéquate!)

    //on traite les cas inf, -inf, nan...,
    //peut arrive sur des noeuds group complexe ou inapropriés(light,...)
    osg::Vec3 halfLength = (_boxInit._max - _boxInit._min)/2;
    for (int i=0; i<3;i++){
      
      if( isnan(halfLength[i]) || !finite(halfLength[i]) ){
	CWARNs("BoundingShape::linkSimply")
	  <<"cas de nan ou inf !!"<<endl;
	return;
      } 
      if( halfLength[i] <0 ){
	CWARNs("BoundingShape::linkSimply")
	  <<"cas de diagonal négative !!"<<geode->getName()<<" "
	  <<parentNode->getName()<<endl;
	return;
      }      
    }

    //TODO Traiter plusieurs niveau de profondeur ?(pour les intersections)
    _dbgShapeBox->set( _boxInit.center(), //center
		       halfLength ); //halflength
    
    this->_sd=new osg::ShapeDrawable(_dbgShapeBox);
    
  }
  
  
  //BUG NOTOIRE D'UTILISATION (avec CalculBox): 
  //Si on fait plusieurs BoundingShape
  //sur un meme noeud (de different type ou sinon sans interet),
  //alors selon l'ordre de chargement, les shapes déjà rajoutées
  //seront prises en compte dans le calcul de la shape suivante

  //SOLUTION 1 : Adopter la stratégie de définir la box avant la sphere
  //SOLUTION 2 : Reporter cela en Pass3 du chargement 
  //SOLUTION 3 : Bloquer cette possibilité

  //NOTE : possibilité d'utiliser la geode "geode" en lui ajoutant un 
  //drawable, néanmois on pourra alors difficilement utiliser les
  //nodeMask pour que les boundings shapes ne soit pas intersectées
  //lors de lancés de rayon, de même pour quelles ne soient pas dessinée
  //Néanmoins, l'intersection va détecté la même Geode, donc on s'aura
  //quel objet est intersecté sans probleme.
  //=>TODO le nodeMask n'a maintenant plus d'utilité ???

  //DESACTIVATION
// osg::Geode * grp= new osg::Geode();
//   grp->setName("boundingShapeDbg");
//   grp->setNodeMask(_nodeMask);
//   CTRACE(this,"linkSimply")<<"grp->setNodeMask("<<_nodeMask<<");"<<endl;
  osg::Geode * grp = geode;
  
  this->_sd->setUseDisplayList(false);
    
  this->_sd->setColor(osg::Vec4( (rand()%2) * 255,
				 (rand()%2) * 255,
				 (rand()%2) * 255,200));
   
  this->_sd->setDrawCallback( this->getPersonnalOsgDrawCallback() );
  
  grp->addDrawable(this->_sd);
  
  //=> Attention il peut y avoir plusieurs parents pour une geode !!
  //geode->getParent(0)->asGroup()->addChild(grp); //=>FAUX (1x/10)
  
  //DESACTIVATION
  //parentNode->asGroup()->addChild(grp); //OK
  
 
}//@CODE_3286


// End of Group: load

/*@NOTE_3223
Constructor method.
*/
BoundingShape::BoundingShape(BD* pBD) //@INIT_3223
  : Evaluable(pBD)
  ,_debug(false),
    _listChild(NULL),
    _node(NULL),
    _nodeName("null"),
    _sd(NULL),
    _type("Sphere"),
    _transfoShape(mInitIdentityRef), 
    _nodeMask(0xffffffff), 
    _frameActive(false),
    _debugGlobal()
{//@CODE_3223
  ConstructorInclude();

  _scaleAdjust = osg::Vec3(1,1,1);

  // Put in your own code
}//@CODE_3223


/*@NOTE_3174
Destructor method.
*/
BoundingShape::~BoundingShape()
{//@CODE_3174
  DestructorInclude();
 
  // Put in your own code
}//@CODE_3174


const osg::BoundingBox& BoundingShape::calculGlobalBox()
{//@CODE_3342
  
  //return _boxComputed;
  
  if ( _listChild != NULL){
    
    //cerr<<this<<" noeud "<<_listChild->size()<<endl;
    
    std::vector<BoundingShape*>::iterator it = _listChild->begin();
    
    //vérification d'usage
    assertC( it != _listChild->end() );
    //ou bien à remplacer par ça
    //if ( it != _listChild->end() ){
    // delete _listChild;
    // _listChild = NULL;
    // return _boxComputed;
    //}
    
    //on prend la premiere boite 
    //_boxComputed = (*it)->calculGlobalBox();
    //it++;
    
    bool res = false;
    
    while ( it != _listChild->end() ) {
      
      //cerr<<"(*it)->getName()="<<(*it)->getName()<<endl;
      
      //si la shape est active pour la derniere frame
      if ( (*it)->_frameActive == BoundingShape::_lastFrameNumber ){
	
	//on prend la premiere boite au premier passage
	if ( !res ){
	  _boxComputed = (*it)->_boxComputed/*calculGlobalBox()*/;
	  res = true;
	} else {
	  
	  expandByOptimisedCall( _boxComputed, 
				 (*it)->_boxComputed,
				 getBD()); 
	}
	
      }
      it++;
    } 

  } // end if (_listChild...
  
  if ( isDebugGlobal() ) {
    
    //assertC(0);
    
    osg::Box * dbgShapeBox = 
      dynamic_cast<osg::Box *>( this->_sd->getShape() );
    
    
    osg::Vec3 halfLength = (_boxComputed._max - _boxComputed._min) /2;
    
    dbgShapeBox->set( _boxComputed.center(), //center
		      halfLength ); //halflength
    
  }
 
  CTRACEnext<<this<<"______________________________feuille "<<this->getName()<<endl;
  CTRACEnext<<"_boxComputed.min="<<_boxComputed._min.x()<<","<<_boxComputed._min.y()<<","<<_boxComputed._min.z()<<endl;
  CTRACEnext<<"_boxComputed.max="<<_boxComputed._max.x()<<","<<_boxComputed._max.y()<<","<<_boxComputed._max.z()<<endl;

  return _boxComputed;
  
}//@CODE_3342


BoundingShape* BoundingShape::clone()
{//@CODE_3285
  return new BoundingShape(getBD());
}//@CODE_3285


osg::Drawable::DrawCallback* BoundingShape::getPersonnalOsgDrawCallback()
{//@CODE_3292
  return new BoundingShapeDrawCallback( this );
}//@CODE_3292


bool BoundingShape::hasCollisionBetween(BoundingShape* bsh)
{//@CODE_3709

  this->calculGlobalBox();
  bsh->calculGlobalBox();
  
  return this->_boxComputed.intersects( bsh->_boxComputed );
      
}//@CODE_3709


void BoundingShape::setAllDebug(bool b)
{//@CODE_3289

  this->setDebug( b );
  
  if ( this->_listChild ){

    //pour chaque boite fille on lance l'affectation du booléen
    for (std::vector<BoundingShape*>::iterator it = this->_listChild->begin();
	 it != this->_listChild->end();
	 it++){
      
      (*it)->setAllDebug( b );
      
    }
  }  

}//@CODE_3289


Evaluable* BoundingShape::staticCreate(BD* pBD)
{//@CODE_3234
  return new BoundingShape(pBD);
}//@CODE_3234


//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_3175
Method which must be called first in a constructor.
*/
void BoundingShape::ConstructorInclude()
{
}


/*@NOTE_3176
Method which must be called first in a destructor.
*/
void BoundingShape::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
} //end of namespace cooki3d



//POURRAI REMPLACER linkRecursively

// class InsertCallbacksVisitor : public osg::NodeVisitor
// {

//    public:
   
//         InsertCallbacksVisitor():osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN)
//         {
//         }
        
//         virtual void apply(osg::Node& node)
//         {
//              node.setUpdateCallback(new UpdateCallback());
//              node.setCullCallback(new CullCallback());
//              traverse(node);
//         }

//         virtual void apply(osg::Geode& geode)
//         {
//             geode.setUpdateCallback(new UpdateCallback());
            
//             //note, it makes no sense to attach a cull callback to the node
//             //at there are no nodes to traverse below the geode, only
//             //drawables, and as such the Cull node callbacks is ignored.
//             //If you wish to control the culling of drawables
//             //then use a drawable cullback...

//             for(unsigned int i=0;i<geode.getNumDrawables();++i)
//             {
//                 geode.getDrawable(i)->setUpdateCallback(new DrawableUpdateCallback());
//                 geode.getDrawable(i)->setCullCallback(new DrawableCullCallback());
//                 geode.getDrawable(i)->setDrawCallback(new DrawableDrawCallback());
//             }
//         }
        
//         virtual void apply(osg::Transform& node)
//         {
//   



 //  std::ostream& operator<<(std::ostream& out,const osg::Matrix& _mat)
//   {
//     for(int i=0;i<4;i++)
//       {
// 	for(int j=0;j<4;j++)
// 	  {
// 	    out<<_mat(i,j)<<" ";
// 	}
// 	out << std::endl;
//       }
//     return out;
//   }

//   inline std::ostream & operator<<(std::ostream & os, osg::Vec3 v){
  
//     os<<v[0]<<","<<v[1]<<","<<v[2];
//     return os;

//   }
  
  // osg::Vec3 mult( osg::Matrix& mat, osg::Vec3 vec ){
  
  //     osg::Vec3 origin = vec;
  //     //vec.normalize();
  
  //     return osg::Vec3 (
  // 		      (mat(0,0) * vec.x() + 
  // 		      mat(1,0) * vec.y() +
  // 		      mat(2,0) * vec.z() +
  // 		      mat(3,0) * 1) ,
  
  // 		      (mat(0,1) * vec.x() + 
  // 		      mat(1,1) * vec.y() +
  // 		      mat(2,1) * vec.z() +
  // 		      mat(3,1) * 1),
  
  // 		      (mat(0,2) * vec.x() + 
  // 		      mat(1,2) * vec.y() +
  // 		      mat(2,2) * vec.z() +
  // 		      mat(3,2) * 1)) ;
  
  
  //   }
