/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          BoundingShape.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'BoundingShape'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _BOUNDINGSHAPE_H
#define _BOUNDINGSHAPE_H

//@START_USER1

// #ifdef CB_INLINES
// #undef CB_INLINES
// #define MODIF_BD_H
// #endif 
// class BoundingShape;


// #include "Evaluable.h"


// #ifdef MODIF_BD_H
// #define CB_INLINES
// #undef MODIF_BD_H
// #endif

#include <osg/BoundingBox>
#include <osg/BoundingSphere>
#include <osg/Group>
#include <osg/Shape>
#include <osg/ShapeDrawable>

#include "Evaluable.h"
#include "XMLUtil.h"
#include "LoaderGeneric.h"

namespace cooki3d{
//@END_USER1


/*@NOTE_3173
classe ...
*/

class AFX_EXT_CLASS BoundingShape
    : public Evaluable
{

//@START_USER2


//@END_USER2

//
// Group: ClassBuilder methods
//

private:
    void ConstructorInclude();
    void DestructorInclude();

//
// Group: data
//

protected:
    bool _debug;
    std::vector<BoundingShape*>* _listChild;
    osg::Node* _node;
    std::string _nodeName;
    osg::Vec3 _scaleAdjust;
    osg::ShapeDrawable* _sd;
    std::string _type;
    osg::Matrix& _transfoShape;
    osg::Node::NodeMask _nodeMask;
    osg::BoundingBox _boxInit;
    osg::BoundingBox _boxComputed;
    int _frameActive;
    bool _debugGlobal;

public:
    static int _lastFrameNumber;
    bool isDebug() const;
    void setDebug(bool debug);
    std::vector<BoundingShape*>* getListChild() const;
    void setListChild(std::vector<BoundingShape*>* pListChild);
    osg::Node* getNode() const;
    void setNode(osg::Node* pNode);
    std::string getNodeName() const;
    void setNodeName(std::string nodeName);
    osg::Vec3 getScaleAdjust() const;
    void setScaleAdjust(osg::Vec3 scaleAdjust);
    osg::ShapeDrawable* getSd() const;
    void setSd(osg::ShapeDrawable* pSd);
    std::string getType() const;
    void setType(std::string type);
    osg::Matrix& getTransfoShape() const;
    virtual void setTransfoShape(osg::Matrix& rTransfoShape);
    osg::Node::NodeMask getNodeMask() const;
    void setNodeMask(osg::Node::NodeMask nodeMask);
    osg::BoundingBox getBoxInit() const;
    osg::BoundingBox getBoxComputed() const;
    bool isDebugGlobal() const;
    void setDebugGlobal(bool debugGlobal);
    virtual void setBoxComputed(const osg::Vec3& min, const osg::Vec3& max,
                                const osg::Quat& q);
    void setFrameActive();

//
// Group: load
//

protected:
    void linkRecursively(osg::Node * n, osg::Node* parentNode = NULL);
    void linkSimply(osg::Geode* geode, osg::Node* parentNode);

public:
    bool check();
    virtual bool loadXML(const xmlpp::Node* node, LoaderXML* loaderXML);
    static bool checkAllInstances(LoaderGeneric* loader);
    static bool linkBoundingShape(osg::Node* n, osg::Node* parent,
                                  const std::string& pickName,
                                  LoaderGeneric* loader);

//
// Non-Grouped Members
//

//
// Non-Grouped Methods
//

protected:
    virtual osg::Drawable::DrawCallback* getPersonnalOsgDrawCallback();

public:
    BoundingShape(BD* pBD);
    virtual ~BoundingShape();
    const osg::BoundingBox& calculGlobalBox();
    virtual BoundingShape* clone();
    virtual bool hasCollisionBetween(BoundingShape* bsh);
    void setAllDebug(bool b);
    static Evaluable* staticCreate(BD* pBD);
};

#endif


#ifdef CB_INLINES
#ifndef _BOUNDINGSHAPE_H_INLINES
#define _BOUNDINGSHAPE_H_INLINES


//
// Group: data
//
inline void BoundingShape::setFrameActive()
{//@CODE_3722
  _frameActive = BoundingShape::_lastFrameNumber;
}//@CODE_3722



/*@NOTE_3236
Returns the value of member '_debug'.
*/
inline bool BoundingShape::isDebug() const
{//@CODE_3236
    return _debug;
}//@CODE_3236



/*@NOTE_3237
Set the value of member '_debug' to 'debug'.
*/
inline void BoundingShape::setDebug(bool debug)
{//@CODE_3237
    _debug = debug;
}//@CODE_3237



/*@NOTE_3260
Returns the value of member '_listChild'.
*/
inline std::vector<BoundingShape*>* BoundingShape::getListChild() const
{//@CODE_3260
    return _listChild;
}//@CODE_3260



/*@NOTE_3261
Set the value of member '_listChild' to 'pListChild'.
*/
inline void BoundingShape::setListChild(std::vector<BoundingShape*>* pListChild)
{//@CODE_3261
    _listChild = pListChild;
}//@CODE_3261



/*@NOTE_3188
Returns the value of member '_node'.
*/
inline osg::Node* BoundingShape::getNode() const
{//@CODE_3188
    return _node;
}//@CODE_3188



/*@NOTE_3189
Set the value of member '_node' to 'pNode'.
*/
inline void BoundingShape::setNode(osg::Node* pNode)
{//@CODE_3189
    _node = pNode;
}//@CODE_3189



/*@NOTE_3252
Returns the value of member '_nodeName'.
*/
inline std::string BoundingShape::getNodeName() const
{//@CODE_3252
    return _nodeName;
}//@CODE_3252



/*@NOTE_3253
Set the value of member '_nodeName' to 'nodeName'.
*/
inline void BoundingShape::setNodeName(std::string nodeName)
{//@CODE_3253
    _nodeName = nodeName;
}//@CODE_3253



/*@NOTE_3216
Returns the value of member '_scaleAdjust'.
*/
inline osg::Vec3 BoundingShape::getScaleAdjust() const
{//@CODE_3216
    return _scaleAdjust;
}//@CODE_3216



/*@NOTE_3217
Set the value of member '_scaleAdjust' to 'scaleAdjust'.
*/
inline void BoundingShape::setScaleAdjust(osg::Vec3 scaleAdjust)
{//@CODE_3217
    _scaleAdjust = scaleAdjust;
}//@CODE_3217



/*@NOTE_3245
Returns the value of member '_sd'.
*/
inline osg::ShapeDrawable* BoundingShape::getSd() const
{//@CODE_3245
    return _sd;
}//@CODE_3245



/*@NOTE_3246
Set the value of member '_sd' to 'pSd'.
*/
inline void BoundingShape::setSd(osg::ShapeDrawable* pSd)
{//@CODE_3246
    _sd = pSd;
}//@CODE_3246



/*@NOTE_3180
Returns the value of member '_type'.
*/
inline std::string BoundingShape::getType() const
{//@CODE_3180
    return _type;
}//@CODE_3180



/*@NOTE_3181
Set the value of member '_type' to 'type'.
*/
inline void BoundingShape::setType(std::string type)
{//@CODE_3181
    _type = type;
}//@CODE_3181



/*@NOTE_3270
Returns the value of member '_transfoShape'.
*/
inline osg::Matrix& BoundingShape::getTransfoShape() const
{//@CODE_3270
    return _transfoShape;
}//@CODE_3270



/*@NOTE_3271
Set the value of member '_transfoShape' to 'rTransfoShape'.
*/
inline void BoundingShape::setTransfoShape(osg::Matrix& rTransfoShape)
{//@CODE_3271
    _transfoShape = rTransfoShape;
}//@CODE_3271



/*@NOTE_3277
Returns the value of member '_nodeMask'.
*/
inline osg::Node::NodeMask BoundingShape::getNodeMask() const
{//@CODE_3277
    return _nodeMask;
}//@CODE_3277



/*@NOTE_3278
Set the value of member '_nodeMask' to 'nodeMask'.
*/
inline void BoundingShape::setNodeMask(osg::Node::NodeMask nodeMask)
{//@CODE_3278
    _nodeMask = nodeMask;
}//@CODE_3278



/*@NOTE_3341
Returns the value of member '_boxInit'.
*/
inline osg::BoundingBox BoundingShape::getBoxInit() const
{//@CODE_3341
    return _boxInit;
}//@CODE_3341



/*@NOTE_3338
Returns the value of member '_boxComputed'.
*/
inline osg::BoundingBox BoundingShape::getBoxComputed() const
{//@CODE_3338
    return _boxComputed;
}//@CODE_3338



/*@NOTE_3785
Returns the value of member '_debugGlobal'.
*/
inline bool BoundingShape::isDebugGlobal() const
{//@CODE_3785
    return _debugGlobal;
}//@CODE_3785



/*@NOTE_3786
Set the value of member '_debugGlobal' to 'debugGlobal'.
*/
inline void BoundingShape::setDebugGlobal(bool debugGlobal)
{//@CODE_3786
    _debugGlobal = debugGlobal;
}//@CODE_3786



// End of Group: data

//@START_USER3

} //end of namespace cooki3d
//@END_USER3

#endif
#endif
