/** -*-c++-*- *************************************************************\
* File:          CalculBoundingBox.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'CalculBoundingBox'
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef CALCUL_BOUNDING_BOX
#define CALCUL_BOUNDING_BOX

#include "CookiMetier.h"

#include <osg/Geode>
#include "ScanSetGraph.h"
#include <osg/ShapeDrawable>
#include <osg/MatrixTransform>
#include "EnvironnementOSG.h"
#include "BD.h"

#include <osg/NodeVisitor>
#include <osg/PositionAttitudeTransform>
#include <osg/AnimationPath>


class CalculBoundingBox {

  bool _start;
  osg::BoundingBox _boundingBoxResult;
  osg::Matrix _transfo;
  //osg::Node * _firstGeode;

public:  
  
  CalculBoundingBox() : _start(true),_boundingBoxResult(0,0,0,0,0,0) {} 

  void reset(){
    _start = true;
    _boundingBoxResult.set(0,0,0,0,0,0);
    _transfo.makeIdentity();
  }

  osg::BoundingBox getBoundingBoxSimple(osg::Node* n){

    calculRecursifBoundingBox(n);
    return _boundingBoxResult;    
  }


  osg::BoundingBox getBoundingBox(osg::Node* n){
    calculRecursifBoundingBox(n);

    /*for (int i= 0;i<3;i++){
      _boundingBoxResult._min[i] = _boundingBoxResult._min[i]*0.1f;
      _boundingBoxResult._max[i] = _boundingBoxResult._max[i]*0.1f;
      } */
    
    calculParentTransfo(n);
    osg::Vec3d scale= _transfo.getScale();
    
    //cerr<<"scale final = "<<_boundingBoxResult._min.x()<<" | ";
    for (int i= 0;i<3;i++){
      _boundingBoxResult._min[i] = _boundingBoxResult._min[i]*scale[i];
      _boundingBoxResult._max[i] = _boundingBoxResult._max[i]*scale[i]; 
      //cerr<<scale[i]<<",";        
    }    
    //cerr<<"| "<<_boundingBoxResult._min.x()<<endl;
    

    return _boundingBoxResult;    
  }
  
  bool isCalculBoxOK(){ return !_start; }

  osg::Matrix getParentTransfo(osg::Node* n){
    calculParentTransfo(n);
    return _transfo;
  }
  

private:  
  
  void calculRecursifBoundingBox(osg::Node* n){        
    osg::Geode * g=dynamic_cast<osg::Geode *>(n);
    
    if (g){      
      addBoundingBox(g->getBoundingBox());      
      return;
    }     
    
    if (n->asGroup()) {
      int num=n->asGroup()->getNumChildren();
      
      for (int i=0; i<num;i++) {
	calculRecursifBoundingBox(n->asGroup()->getChild(i));		
      }      
    }
  }  
  void addBoundingBox(const osg::BoundingBox &  bbox){   
    
    if (_start){
      _boundingBoxResult = bbox;
      _start = false;
    }
    else {
      _boundingBoxResult.expandBy(bbox);
    }

  }

  void calculParentTransfo(osg::Node * n){

    osg::Transform * trans=dynamic_cast<osg::Transform*>(n);
    
    if (trans){      
      osg::Matrix localTransfo;
      trans->computeLocalToWorldMatrix(localTransfo,NULL);
      cerr<<"calculParentTransfo "<<n->getName()<<" : "
	  <<localTransfo.getScale().x()<<","
	  <<localTransfo.getScale().y()<<","
	  <<localTransfo.getScale().z()<<endl;
      _transfo = localTransfo * _transfo;      
    }

    if ( n->getNumParents()>0){  
      calculParentTransfo(n->getParent(0));
    } 
  }

};


#endif
