/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          Character.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'Character'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include "Character.h"
//#include <osg/MatrixTransform>
#include <osg/PositionAttitudeTransform>
#include "MapRW.h"
#include "EnvironnementOSG.h"
#include "SolGravitation.h"
#include "SolCollisions.h"

#include <OpenThreads/Thread>
#include <osg/AnimationPath>
#include <osg/NodeCallback>
#include <osg/Quat>
#include <osg/Matrixd>
#include "LoaderXML.h"
#include <osgDB/ReadFile>

//u.v = |u|*|v|*cos(u,v)
double AngleRad(osg::Vec3 vec1,osg::Vec3 vec2)
{
  //produit sclaire des vecteur
  double dotProduct = vec1*vec2;

  //produit des normes des 2 vecteurs
  double vectorsMagnitude = vec1.length() * vec2.length();


  //cout<<"dotProduct="<<dotProduct<<endl;
  //d'après la formule
  double angle = acos( dotProduct / vectorsMagnitude );

  //vérifions de l'angle n'est pas indéfinie ( -1.#IND0000000)
#ifdef UNIX
  if(isnan(angle)) return 0;
#else
  if(_isnan(angle)) return 0;
#endif

  //en radian
  return angle;

  //en dégrés
  //return ( angle*360.0 /(2*M_PI)  );
}

namespace cooki3d{
//@END_USER2





//
// Group: deplacement
//
Character::CalculTourner Character::calculTournerVers(float x, float y, float z)
{//@CODE_640
  //!! on est déjà dans un Threadd...


  Character::CalculTourner r; //résultat des calculs

  //calcul du vecteur direction à prendre
  r.arrivee = osg::Vec3(x,y,0);                      //point d'arrivée
  r.direction = r.arrivee  - _rootNode->getPosition(); //vecteur de direction
  // cout<<"CTournV="<<_rootNode->getPosition()<<endl;

  r.direction.z()=0;  //correction des erreurs de floattants


  //calcule de l'angle de rotation actuel
  osg::Vec3 tmp;         //axe de rotation (logiqmt vers le haut ou le bas)
  _rootNode->getAttitude().getRotate(r.angleActu,tmp);

  //cout<<"r.angleActu="<<(r.angleActu)*360.0 /(2*M_PI)<<endl;
  //cout<<"tmp = "<<tmp<<endl;
  //tmp.normalize();assertC(tmp == osg::Vec3(0,0,1.0));  //vérification

  //calcul du vecteur de direction actuel
  osg::Matrix mat;

  if (tmp.z() == -1)
  {
    r.angleActu =
      (r.angleActu/fabs(r.angleActu))*
      (2*M_PI - fabs(r.angleActu));
    //cout<<"r.angleActu 2="<<(r.angleActu)*360.0 /(2*M_PI)<<endl;
  }

  mat.makeRotate(r.angleActu,osg::Vec3(0,0,-1.0/**tmp.z()*/));
  //_rootNode->getAttitude().get(mat);
  r.dirActu = mat * osg::Vec3(0,-1,0);


  r.angle=AngleRad(r.dirActu,r.direction);
  r.tempsRot = (fabs(r.angle)/osg::inDegrees(_vitesseRot));


  //rectifications en fonction de la getPosition du personnage
  mat.makeRotate(M_PI/2,osg::Vec3(0,0,-1.0));
  osg::Vec3 gauchAlfred = mat * r.dirActu;
  //cout<<"gauchAlfred="<<gauchAlfred<<endl;

  if (gauchAlfred * r.direction > 0) r.angleFinal = r.angle+ r.angleActu;
  else r.angleFinal = r.angleActu - r.angle;
  if (r.angleFinal > 2*M_PI) r.angleFinal -=  2*M_PI;
  else if (r.angleFinal < 0) r.angleFinal = 2*M_PI + r.angleFinal;

  //cout<<"angleActu="<<r.angleActu*180/M_PI<<endl;
  //cout<<"dirActu="<<r.dirActu<<endl;
  //r.direction.normalize();
  //cout<<"directionPrise="<<r.direction<<endl;
  //cout<<"angleFinal="<<r.angleFinal*180/M_PI<<endl;

  return r;

}//@CODE_640


Character::CalculTranslater Character::calculTranslaterVers(float x, float y,
                                                            float z)
{//@CODE_644
  //!! on est déjà dans un Threadd...

  if (!_attraction) assertC(z);
  Character::CalculTranslater r;

  //ajustement sur le terrain si besoinx
  if (!_attraction) r.arrivee = osg::Vec3(x,y,z);
  else r.arrivee=osg::Vec3(x,y,SolGravitation::getHeightAbs(x,y,getBD())
			   +_elevatZ);
  //else r.arrivee=osg::Vec3(x,y,SolGravitation::getHeightZone(x,y,5)+_elevatZ);

  //cout<<"CTransV="<<_rootNode->getPosition()<<endl;

  r.depl = r.arrivee  - _rootNode->getPosition();
  r.tempsParc= r.depl.length()/ _vitesseDep; //temps pour parc. la dist (t=d/v)
  //cout<<"tempsParc1="<<tempsParc<<endl;

  return r;
}//@CODE_644


void Character::startTournerVers(Character::CalculTourner c)
{//@CODE_484
  //!! on est déjà dans un Threadd...

  //float deltaImg = 1/ (c.tempsRot* 30); //30fps-> période d'échantillonage

  //création de la "file" d'animation
  osg::AnimationPath * path=new osg::AnimationPath;
  path->setLoopMode(osg::AnimationPath::NO_LOOPING);

  //angle de départ
  osg::Quat q(c.angleActu,osg::Vec3(0.0,0.0,1.0));

  path->insert(0.0, 
	       osg::AnimationPath::ControlPoint(_rootNode->getPosition(),
               q,
               getScale()));
  //angle d'arrivé
  q=osg::Quat(c.angleFinal,osg::Vec3(0.0,0.0,1.0));
  path->insert(c.tempsRot,
               osg::AnimationPath::ControlPoint(_rootNode->getPosition(),
                                                q,
                                                getScale()));


  //lancement de l'animation (sans quelle soit totalement définie)
  // osg::AnimationPathCallback * ncb=
  // static_cast<osg::AnimationPathCallback *>(_rootNode->getUpdateCallback());
  //ncb->reset();
  //ncb->setAnimationPath(path);

  //faire comme ca sinon bugs
  _rootNode->setUpdateCallback(new osg::AnimationPathCallback(path));


}//@CODE_484


void Character::startTranslaterVers(Character::CalculTranslater c)
{//@CODE_483
  //!! on est déjà dans un Threadd...

  //création de la "file" d'animation
  osg::AnimationPath * path=new osg::AnimationPath;
  path->setLoopMode(osg::AnimationPath::NO_LOOPING);


  float t=0;         //instant t
  float ti=0;        //instant t pour la prise en compte de l'interpolation 
  osg::Vec3 transVec;//translation à l'instant t
  int cpt=0;         //compteur de test

  //2fps-> période d'échantillonage des "images"
  //en faite 2 points d'échantillonage par seconde suffisent 
  //et rendent ainsi le mouvement plus humain et moins saccadé
  //car tout ces points seront ensuite interpolés dans l'obj osg::AnimationPath
  //float deltaImg = 1.0f / (c.tempsParc * 2); 
  
  //pourquoi le temps de parcours??
  float deltaImg = 1.0f / 2;


  //période de vérification des collisions
  //elle prend en compte la précision du solcollision
  //et la distance à parcourir
  float deltaVerifCol = c.tempsParc / 
    ( fabs(max(c.depl.x(),c.depl.y())) / SolCollisions::getDeltaCase(getBD())); 

  //concircuitage pour tester...
  //float deltaImg = deltaVerifCol;

  //cerr<<"deltaVerifCol="<<deltaVerifCol<<endl;
  

  osg::Quat q=_rootNode->getAttitude();


  //construction du animationPath pour cas ou attraction>0 (gravité)
  
  //mesure performances
  osg::Timer_t tpsStart = osg::Timer::instance()->tick();


  //float zPrec=0;//hauteur précédente
  
  
  float zSolGrav; //hauteur du solGravitation
  float zSolCol;  //hauteur di solCollision 
                  //=> si (zSolGrav != zSolCol) alors collision.
  bool collision = false;

  //cas ou l'attraction est active (tous les cas pour l'instant)
  if (_attraction)
  {
    
    while (t<c.tempsParc)
    {
      
      //calcule de la getPosition en fonction du temps t 
      //et du précalcul de translation
      transVec = _rootNode->getPosition() + c.depl*(t/c.tempsParc);

      //récupération de la hauteur du solGravitation
      zSolGrav = SolGravitation::getHeightAbs(transVec.x(),transVec.y(),getBD());

      //récupération de la hauteur du solCollision
      zSolCol = SolCollisions::getHeightAbs(transVec.x(),transVec.y(),getBD());
      
      //test si collision, si c'est le cas on arrête la boucle
      if (zSolCol > zSolGrav+0.1f){
      	collision = true;
	
	CTRACE(this,"startTranslaterVers(...)")
	  <<"collision !! ("<<transVec.x()<<","<<transVec.y()
	  <<")="<<zSolCol<<";"<<zSolGrav<<endl;
      	break;
      }

      //mise à jour de la hauteur du point
      transVec.z() = zSolGrav +_elevatZ;
      
      
    
      t += deltaVerifCol;
      if (t > ti) {
	ti += deltaImg;
	cpt++;
	
	//si la hauteur a changée, on insert un nouveau point d'interpolation
	//if (transVec.z()!=zPrec)
	{	
	  path->insert(t,
		       osg::AnimationPath::ControlPoint(transVec,q,getScale()));
	  //zPrec = transVec.z();
	}

      }
      
    }
  }
  else {
    CERR(this,"startTranslaterVers()")
      <<"Ne peut calculer de translation si il n'y a pas d'attraction"
      <<endl;
    assertC(0);
  }

  osg::Timer_t tpsEnd = osg::Timer::instance()->tick();
  //cout<<"cpt="<<cpt<<endl;
  //cout<<"t = "<<t<<endl;
  CTRACE(this,"startTranslaterVers")<<"performance : "<<osg::Timer::instance()->delta_s(tpsStart,tpsEnd)<<endl;

  if (!collision) {
    path->insert(c.tempsParc,
               osg::AnimationPath::ControlPoint(c.arrivee,q,getScale()));
  }else {
    /* int retrait = 0;
       while (retrait<4 && t>0){
      path->getTimeControlPointMap().
	erase(path->getTimeControlPointMap().end());
      retrait++;
      
      }*/
  }			 



  //lancement de l'animation (sans quelle soit totalement définie)
  // osg::AnimationPathCallback * ncb=
  // static_cast<osg::AnimationPathCallback *>(_rootNode->getUpdateCallback());
  //ncb->reset();
  //ncb->setAnimationPath(path);

  //faire comme ca sinon bugs
  _rootNode->setUpdateCallback(new osg::AnimationPathCallback(path));

}//@CODE_483


void Character::stopMouvements()
{//@CODE_665
  
  TimeBar * t= getTimeBar();
  t->stop(); //à améliorer, solution instable et temporaire!!!
  
  t->vider();//risque de segmentation fault sur des commandes détruites sur systeme lent
  

  //osg::AnimationPath * path=new osg::AnimationPath;
  osg::AnimationPathCallback * ncb=
    static_cast<osg::AnimationPathCallback *>(_rootNode->getUpdateCallback());
  //ncb->reset();
  assertC(ncb);
  ncb->setPause(true);

  //cout<<"-------------StopMouv visualisation"<<endl;
  Character::CalculTourner crtest=this->calculTournerVers(0,0);
  //cout<<"-------------StopMouv fin visualisation"<<endl;

  /* _rootNode->setUpdateCallback(new osg::AnimationPathCallback(path));
  path->insert(0.0,
        osg::AnimationPath::ControlPoint(_rootNode->getPosition(),
  				_rootNode->getAttitude(),
  				getScale()));
  */
}//@CODE_665


// End of Group: deplacement


//
// Group: A voir
//
double Character::eval(std::string methode, double args[10], int nbParam)
{//@CODE_389

  CTRACE(this,"eval")
    <<"-----------------------------thisPerso::eval : "<<methode<<endl;

  /*if (methode == "isPause"){
    return (double)isPause();
    }*/
  assertC(0);


  return 0;
}//@CODE_389


// End of Group: A voir


//
// Group: simple
//
void Character::allerA(float x, float y, float z, bool use_idle)
{//@CODE_1377

  stopMouvements();
  
  Character::CalculTourner cr= calculTournerVers(x,y);
  Character::CalculTranslater ct= calculTranslaterVers(x,y); 
  
  float positionStoppage = 0.1+cr.tempsRot+ct.tempsParc-( getBlendTime()/2);

  TimeBar * t= getTimeBar();
  
  t->setTimeSize(ct.tempsParc+cr.tempsRot+0.1);
  CTRACE(this,"AllerA")<<"ct.tempsParc="<<ct.tempsParc<<endl;
  CTRACE(this,"AllerA")<<"cr.tempsRot="<<cr.tempsRot<<endl;
  
  //TODO revoir _idle, _walk ...
  
  if (use_idle){
    Character::ComStartAnimation * stopIdle= getNewComStopAnimation();
    stopIdle->setArg1(getName()+"_idle");
    t->addCommand(0.0,stopIdle);
  }
  
  Character::ComTourner *tourner= getNewComTourner();
  tourner->setArg1(cr);
  t->addCommand(0.0,tourner);

  
  
  Character::ComStartAnimation * marcher= getNewComStartAnimation();
  marcher->setArg1(getName()+"_walk");
  t->addCommand(0.0,marcher);
  
  Character::ComTranslater *trans= getNewComTranslater();
  trans->setArg1(ct);   
  t->addCommand(0.1+cr.tempsRot,trans);
  
  
  if (positionStoppage< cr.tempsRot)
    positionStoppage = cr.tempsRot + 0.1;
    
  
  Character::ComStopAnimation *comStopMarcher= getNewComStopAnimation();
  comStopMarcher->setArg1(getName()+"_walk");
  t->addCommand(positionStoppage,comStopMarcher);
  
  if (use_idle){
    Character::ComStartAnimation * comIdle= getNewComStartAnimation();
    
    comIdle->setArg1(getName()+"_idle");
    t->addCommand(positionStoppage,comIdle);
  }

  
    
  //t->SetLoop(true);
  t->play();
  

}//@CODE_1377


// End of Group: simple


//
// Group: load
//
void Character::buildRootNode(osg::Node* nodePerso)
{//@CODE_1566

  // Have OpenGL automatically normalize your scaled normals
  // (any arbitrary scaling)
  ////glEnable( GL_NORMALIZE );
  /*  OR   */
  // Perform the rescaling assuming a uniform scaling factor
  // (slightly better performance than GL_NORMALIZE)


  //on ajuste la haute qu'une fois ici, le terrain est normalt chargé
  _rootNode->setPosition(osg::Vec3(getPosition().x(),
				   getPosition().y(),
				   SolGravitation::
				   //getHeightZone(_posX,_posY,5)+_elevatZ));
				   getHeightAbs(getPosition().x(),
						getPosition().y(),
						getBD())+_elevatZ )
			 );
  
  //pat->setScale(_scale);





  //_rootNode->setName("groupTransfo");

  /*osg::Matrix transfo;
  transfo.makeScale(_scale);       
  transfo.postMult(transfo.translate
     (_posXBufferLoad,
      _posYBufferLoad,
      SolGravitation::getHeightAbs(_posXBufferLoad,_posYBufferLoad)+_elevatZ));
  _rootNode->setMatrix(transfo);      
  */

  //GL_NORMALIZE
  //GL_RESCALE_NORMAL
  /*glEnable(GL_NORMALIZE); // needed to support scaling trfs
  glEnable(GL_RESCALE_NORMAL);

  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);*/

  osg::StateSet *state= new osg::StateSet();
  /*  state->setMode(GL_NORMALIZE, osg::StateAttribute::ON );
  state->setMode(GL_RESCALE_NORMAL, osg::StateAttribute::ON );
  state->setMode(GL_DEPTH_TEST, osg::StateAttribute::ON );*/
  //state->setMode(GL_LEQUAL, osg::StateAttribute::ON );

  _rootNode->setStateSet(state);

  //modif 27/03/05
  _rootNode->setUpdateCallback(new osg::AnimationPathCallback(new osg::AnimationPath()));
  assertC(nodePerso);
  _rootNode->addChild(nodePerso);

  _rootNode->setName(getName());

  /*EnvironnementOSG::instance()*/
  //pRootNode->addChild( getRootNode() );
  //EnvironnementOSG::instance()->GetRootNode()->addChild(getModel());

}//@CODE_1566


bool Character::loadXML(const xmlpp::Node* node, LoaderXML* loaderXML)
{//@CODE_1567
  //cout<<"LoadPersonnage &&&&&&&&&&&&&&&&&&&&&&&&&"<<endl;


  XMLUtil xml(node);

  setName(xml.getFirstText("nom"));  
  //cerr<<"LoadPersonnage &&&&"<<getName()<<endl;
  
  _attraction = atof(xml.getFirstText("attraction").c_str());

  
  float posX=atof(xml.getFirstText("positionX").c_str());
  float posY=atof(xml.getFirstText("positionY").c_str());

  
  _elevatZ=atof(xml.getFirstText("elevationZ").c_str());
  _blendTime=atof(xml.getFirstText("blendTime").c_str());
  _vitesseDep= atof(xml.getFirstText("vitesseDeplacement").c_str());
  _vitesseRot= atof(xml.getFirstText("vitesseRotation").c_str()); 


  _rootNode = new osg::PositionAttitudeTransform();
  

  _rootNode->setPosition(osg::Vec3(posX,
				   posY,
				   _elevatZ));
  
  _rootNode->setName(getName());
		     
  //pat->setScale(_scale);



  return true;
}//@CODE_1567


/*@NOTE_4087
Check the integritry of the object
*/
bool Character::check()
{//@CODE_4087
       
    bool resultOk = _rootNode != NULL 
      && _rootNode->getUpdateCallback() != NULL;
    
    if (!resultOk){
      CERR(this,"check")<<"\""<<getName()<<"\" is not linked !!"<<endl;
    }
    return resultOk;
}//@CODE_4087


/*@NOTE_248
Constructor method.
*/
Character::Character(BD* pBD) //@INIT_248
    : Animable(pBD)
    ,_attraction(0),
      _elevatZ(0),
      _rootNode(NULL),
      _vitesseDep(10),
      _vitesseRot(200),
      _pTimeBar(NULL)
{//@CODE_248
  ConstructorInclude();
  ////_model = new osgCal::Model();
  //osg::ref_ptr<osgCal::Model> modelReferenced = new osgCal::Model();
  //_model = modelReferenced.get();


  _pTimeBar = new TimeBar(getBD());
  // Put in your own code
}//@CODE_248


/*@NOTE_244
Destructor method.
*/
Character::~Character()
{//@CODE_244
  DestructorInclude();

  CTRACE(this,"~Character")<<"destruction Character"<<endl;

  //_model->unref();
  //delete _model;
  //delete _pTimeBar;

  // Put in your own code
}//@CODE_244


// End of Group: load

//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_245
Method which must be called first in a constructor.
*/
void Character::ConstructorInclude()
{
}


/*@NOTE_246
Method which must be called first in a destructor.
*/
void Character::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
} //end of namespace cooki3d





