/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          Character.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'Character'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _CHARACTER_H
#define _CHARACTER_H

//@START_USER1
//#include "MediaInterface.h"
//#include "Media.h"


//#include <osgCal/Model>
//#include <osgCal/CoreModel>
#include "XMLUtil.h"
#include "MapRW.h"
//#include "HeightMap/HeightMap.hpp"
#include "Animable.h"
#include "TimeBar.h"
#include "Methode_1arg.h"
#include "SolGravitation.h"
#include <osg/PositionAttitudeTransform>
namespace cooki3d{
//@END_USER1


/*@NOTE_243
class ...
*/

class AFX_EXT_CLASS Character
    : public Animable
{

//@START_USER2
 public:

  struct CalculTourner {
    osg::Vec3 direction;   //vecteur de direction à prendre
    osg::Vec3 arrivee;     //point d'arrivée
    osg::Vec3 dirActu;     //direction actuel du perso
    double angleActu;      //angle actuel de rot du perso
    double angleFinal;     //angle final de rot du perso 
    double angle;          //delta (final,actu)
    double tempsRot;       //temps pour réaliser la rotation
    
  };    
  
  
  struct CalculTranslater{
    
    osg::Vec3 depl;   //vecteur de déplacement
    osg::Vec3 arrivee;//point d'arrivée
    double tempsParc; //temps pour parcourir la translation
    
  };

  //Commandes
  typedef Methode_1arg<Character,CalculTourner>    ComTourner;
  typedef Methode_1arg<Character,CalculTranslater> ComTranslater;
  typedef Methode_1arg<Character,std::string>      ComStartAnimation;
  typedef Methode_1arg<Character,std::string>      ComStopAnimation;


  //friend class ChargeurJeu;
//@END_USER2

//
// Group: ClassBuilder methods
//

private:
    void ConstructorInclude();
    void DestructorInclude();

//
// Group: deplacement
//

public:
    Character::CalculTourner calculTournerVers(float x, float y, float z = 0);
    Character::CalculTranslater calculTranslaterVers(float x, float y,
                                                     float z = 0);
    void startTournerVers(Character::CalculTourner c);
    void startTranslaterVers(Character::CalculTranslater c);
    Character::ComTranslater* getNewComTranslater();
    Character::ComStartAnimation* getNewComStartAnimation();
    Character::ComTourner* getNewComTourner();
    Character::ComStopAnimation* getNewComStopAnimation();
    void stopMouvements();

//
// Group: data
//

protected:
    float _attraction;
    float _elevatZ;
    osg::PositionAttitudeTransform* _rootNode;
    float _vitesseDep;
    float _vitesseRot;
    TimeBar* _pTimeBar;
    float _blendTime;

public:
    osg::PositionAttitudeTransform* getRootNode() const;
    void setRootNode(osg::PositionAttitudeTransform* pRootNode);
    float getVitesseDep() const;
    void setVitesseDep(float vitesseDep);
    float getVitesseRot() const;
    void setVitesseRot(float vitesseRot);
    TimeBar* getTimeBar() const;
    void setTimeBar(TimeBar* pTimeBar);
    float getBlendTime() const;
    void setBlendTime(float blendTime);
    osg::Vec3 getScale();
    osg::Vec3 getPosition();
    void setScale(osg::Vec3 scale);
    void setPosition(osg::Vec3 position);
    void setPositionXY(float x, float y);

//
// Group: animation
//

public:
    virtual double getDureeAnim(std::string anim) = 0;
    virtual void startAnimation(std::string nom) = 0;
    virtual void setDureeAnim(std::string anim, double duree) = 0;
    virtual void stopAnimation(std::string nom) = 0;

//
// Group: A voir
//

public:
    virtual double eval(std::string methode, double args[10], int nbParam);
    void allerAOrigin(float x, float y, float z = 0);

//
// Group: simple
//

public:
    virtual void allerA(float x, float y, float z = 0, bool use_idle = true);

//
// Group: load
//

public:
    void buildRootNode(osg::Node* nodePerso);
    virtual bool loadXML(const xmlpp::Node* node, LoaderXML* loaderXML);
    virtual bool check();
    Character(BD* pBD);
    virtual ~Character();

//
// Non-Grouped Members
//

//
// Non-Grouped Methods
//
};

#endif


#ifdef CB_INLINES
#ifndef _CHARACTER_H_INLINES
#define _CHARACTER_H_INLINES


//
// Group: deplacement
//
inline Character::ComTranslater* Character::getNewComTranslater()
{//@CODE_658
  return new Character::ComTranslater(getBD(),"Character::ComTranslater",
				      &Character::startTranslaterVers,this);
}//@CODE_658



inline Character::ComStartAnimation* Character::getNewComStartAnimation()
{//@CODE_659
  return new Character::ComStartAnimation(getBD(),
					  "Character::ComStartAnimation",
					  &Character::startAnimation,this);
}//@CODE_659



inline Character::ComTourner* Character::getNewComTourner()
{//@CODE_660
  return new Character::ComTourner(getBD(),
				   "Character::ComTourner",
				   &Character::startTournerVers,this);
}//@CODE_660



inline Character::ComStopAnimation* Character::getNewComStopAnimation()
{//@CODE_664
  return new Character::ComStopAnimation(getBD(),
					 "Character::ComStopAnimation",
					 &Character::stopAnimation,this);
}//@CODE_664



// End of Group: deplacement


//
// Group: data
//
inline osg::Vec3 Character::getScale()
{//@CODE_1963
 return _rootNode->getScale();
}//@CODE_1963



inline osg::Vec3 Character::getPosition()
{//@CODE_1964
  return _rootNode->getPosition();
}//@CODE_1964



inline void Character::setScale(osg::Vec3 scale)
{//@CODE_1966
  _rootNode->setScale(scale);
}//@CODE_1966



inline void Character::setPosition(osg::Vec3 position)
{//@CODE_1971
  _rootNode->setPosition(position);
}//@CODE_1971



inline void Character::setPositionXY(float x, float y)
{//@CODE_1973
  _rootNode->setPosition(osg::Vec3(x,y,_rootNode->getPosition().z()));
}//@CODE_1973



/*@NOTE_1077
Returns the value of member '_rootNode'.
*/
inline osg::PositionAttitudeTransform* Character::getRootNode() const
{//@CODE_1077
    return _rootNode;
}//@CODE_1077



/*@NOTE_1078
Set the value of member '_rootNode' to 'pRootNode'.
*/
inline void Character::setRootNode(osg::PositionAttitudeTransform* pRootNode)
{//@CODE_1078
    _rootNode = pRootNode;
}//@CODE_1078



/*@NOTE_1071
Returns the value of member '_vitesseDep'.
*/
inline float Character::getVitesseDep() const
{//@CODE_1071
    return _vitesseDep;
}//@CODE_1071



/*@NOTE_1072
Set the value of member '_vitesseDep' to 'vitesseDep'.
*/
inline void Character::setVitesseDep(float vitesseDep)
{//@CODE_1072
    _vitesseDep = vitesseDep;
}//@CODE_1072



/*@NOTE_936
Returns the value of member '_vitesseRot'.
*/
inline float Character::getVitesseRot() const
{//@CODE_936
    return _vitesseRot;
}//@CODE_936



/*@NOTE_937
Set the value of member '_vitesseRot' to 'vitesseRot'.
*/
inline void Character::setVitesseRot(float vitesseRot)
{//@CODE_937
    _vitesseRot = vitesseRot;
}//@CODE_937



/*@NOTE_1080
Returns the value of member '_pTimeBar'.
*/
inline TimeBar* Character::getTimeBar() const
{//@CODE_1080
    return _pTimeBar;
}//@CODE_1080



/*@NOTE_1081
Set the value of member '_pTimeBar' to 'pTimeBar'.
*/
inline void Character::setTimeBar(TimeBar* pTimeBar)
{//@CODE_1081
    _pTimeBar = pTimeBar;
}//@CODE_1081



/*@NOTE_1541
Returns the value of member '_blendTime'.
*/
inline float Character::getBlendTime() const
{//@CODE_1541
    return _blendTime;
}//@CODE_1541



/*@NOTE_1542
Set the value of member '_blendTime' to 'blendTime'.
*/
inline void Character::setBlendTime(float blendTime)
{//@CODE_1542
    _blendTime = blendTime;
}//@CODE_1542



// End of Group: data

//@START_USER3
} //end of namespace cooki3d
//@END_USER3

#endif
#endif
