/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          CharacterOsgCal.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'CharacterOsgCal'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#ifdef OSG_CAL_SUPPORT

#include "CharacterOsgCal.h"
//#include <osg/MatrixTransform>
#include <osg/PositionAttitudeTransform>
#include "MapRW.h"
#include "EnvironnementOSG.h"
#include "SolGravitation.h"
#include <OpenThreads/Thread>
#include <osg/AnimationPath>
#include <osg/NodeCallback>
#include <osg/Quat>
#include <osg/Matrixd>
#include "LoaderXML.h"
#include <osgDB/ReadFile>





#define LINE_BUFFER_SIZE 4096
#define USE_OSG_PLUGIN


bool loadCoreModelFromCFG(	const std::string& dir,
                           const std::string& path,
                           osgCal::CoreModel *coreModel)
{
  // Read file and update model
  CalCoreModel* calCoreModel	= coreModel->getCalCoreModel();
  std::cout << " Loading cal3d character from " << path.c_str() << std::endl;

  float scale = 1.0f;
  bool bScale = false;

  FILE* f = fopen(path.c_str(), "r");
  if(!f)
  {
    std::cerr << "opening failed!" << std::endl;
    return false;
  }

  char buffer[LINE_BUFFER_SIZE];

  while (fgets(buffer, LINE_BUFFER_SIZE,f))
  {
    // Ignore comments or empty lines
    if (*buffer == '#' || *buffer == 0)
      continue;

    char* equal = strchr(buffer, '=');
    if (equal)
    {
      // Terminates first token
      *equal++ = 0;
      // Removes ending newline (CR & LF)
      {
        int last = strlen(equal) - 1;
        if (equal[last] == '\n') equal[last] = 0;
        if (last > 0 && equal[last-1] == '\r') equal[last-1] = 0;
      }
      std::string fullpath = dir + std::string(equal);

      if (!strcmp(buffer, "scale"))
      {
        bScale	= true;
        scale	= atof( equal );
        continue;
      }

      std::cout << "    subloading " << buffer << ": " << fullpath.c_str() << std::endl;

      if (!strcmp(buffer, "skeleton"))
      {
        if(!calCoreModel->loadCoreSkeleton(fullpath))
        {
          std::cout << "LoadCFG error: " << CalError::getLastErrorDescription().c_str() << std::endl;
        }
      }
      else if (!strcmp(buffer, "animation"))
      {
        if(calCoreModel->loadCoreAnimation(fullpath) < 0)
        {
          std::cout << "LoadCFG error: " << CalError::getLastErrorDescription().c_str() << std::endl;
        }
      }
      else if (!strcmp(buffer, "mesh"))
      {
        if(calCoreModel->loadCoreMesh(fullpath) < 0)
        {
          std::cout << "LoadCFG error: " << CalError::getLastErrorDescription().c_str() << std::endl;
        }
      }
      else if (!strcmp(buffer, "material"))
      {
        int materialId = calCoreModel->loadCoreMaterial(fullpath);

        if(materialId < 0)
        {
          std::cout << "LoadCFG error: " << CalError::getLastErrorDescription().c_str() << std::endl;
        }
        else
        {
          calCoreModel->createCoreMaterialThread(materialId);
          calCoreModel->setCoreMaterialId(materialId, 0, materialId);


          CalCoreMaterial* material = calCoreModel->getCoreMaterial(materialId);
          // the texture file path is relative to the CRF data directory
          for(std::vector<CalCoreMaterial::Map>::iterator i = material->getVectorMap().begin();
              i != material->getVectorMap().end();
              i++)
          {
            i->strFilename = dir + i->strFilename;
          }

        }
      }
    }
  }

  // scaling must be done after everything has been created
  if( bScale )
  {
    std::cout << "scaling model by " << scale << " factor" << std::endl;
    calCoreModel->scale( scale );
  }
  fclose(f);

  return true;
}




namespace cooki3d{
//@END_USER2





//
// Group: animation
//
double CharacterOsgCal::getDureeAnim(std::string anim)
{//@CODE_1526


  assertC(_mapAnim.find(anim)!=_mapAnim.end());

  return  _model->getCalModel()->getCoreModel()->getCoreAnimation(_mapAnim[anim])->getDuration();
  //((osgCal::CoreModel *)_model->getCoreModel())->get()->getCoreAnimation(_mapAnim[anim])->getDuration();

}//@CODE_1526


void CharacterOsgCal::setDureeAnim(std::string anim, double duree)
{//@CODE_1532


  assertC(_mapAnim.find(anim)!=_mapAnim.end());

  CalCoreAnimation *walk=_model->getCalModel()->getCoreModel()->getCoreAnimation(_mapAnim[anim]);
  //((osgCal::CoreModel *)_model->getCoreModel())->get()->getCoreAnimation(_mapAnim[anim]);
  double dureeInit = walk->getDuration();
  walk->setDuration(duree);

  //cout<<"walk->getAnimationTime()="<<walk->getAnimationTime()<<endl;

  std::list< CalCoreTrack * > & trackList = walk->getListCoreTrack();
  std::list< CalCoreTrack * >::iterator itt= trackList.begin();

  double tps1=0;
  double tic = _model->getCalModel()->getMixer()->getAnimationTime();
  cout<<"AnimationTime="<<tic<<endl;
  _model->getCalModel()->getMixer()->setAnimationTime(tic*(duree/dureeInit));

  while ( itt != trackList.end())
  {

    if ((*itt)->getCoreKeyframeCount())
    {
      //à priori facultatif ?
      tps1 = (*itt)->getCoreKeyframe(0)->getTime();
    }

    for (int i = 0; i < (*itt)->getCoreKeyframeCount(); i++)
    {
      (*itt)->getCoreKeyframe(i)->
      setTime(((*itt)->getCoreKeyframe(i)->
               getTime()-tps1)*(duree/dureeInit)+tps1);
    }
    itt++; 
  }

}//@CODE_1532


void CharacterOsgCal::startAnimation(std::string nom)
{//@CODE_1535
  //1 == le poids par rapport au autres animations
  //_blendTime == temps mis pour aquérir ce poids
  //MapRW::mapStrInt::iterator it= _mapAnim.find(nom);
  assertC(_mapAnim.find(nom)!=_mapAnim.end());

  //_model->loop(_mapAnim[nom],1,_blendTime);
  _model->getCalModel()->getMixer()->blendCycle(_mapAnim[nom],1,_blendTime);

}//@CODE_1535


void CharacterOsgCal::stopAnimation(std::string nom)
{//@CODE_1537
  //1 == le poids par rapport au autres animations
  //_blendTime == temps mis pour aquérir ce poids
  //MapRW::mapStrInt::iterator it= _mapAnim.find(nom);
  CTRACE(this,"stopAnimation(string nom)")<<"test-test"<<endl;
  CTRACE(this,"stopAnimation(string nom)")<<" nom="<<nom<<endl;
  if (_mapAnim.find(nom)==_mapAnim.end()){
    CERR(this,"stopAnimation")<<"l'animation "<<nom<<" n'a pas été trouvé!!"<<endl;
    return;
  }


  //_model->stop(_mapAnim[nom],_blendTime);
  _model->getCalModel()->getMixer()->clearCycle(_mapAnim[nom],_blendTime);

}//@CODE_1537


// End of Group: animation


//
// Group: A voir
//
double CharacterOsgCal::eval(std::string methode, double args[10], int nbParam)
{//@CODE_1522

  cout<<"-----------------------------thisPerso::eval : "<<methode<<endl;

  /*if (methode == "isPause"){
    return (double)isPause();
    }*/
  assertC(0);


  return 0;
}//@CODE_1522


// End of Group: A voir


//
// Group: load
//
bool CharacterOsgCal::loadCal3dFile(const std::string absFichierConf)
{//@CODE_1528

  MapRW mapp;
  std::string path,file,ext;
  mapp.split(absFichierConf,path,file,ext);

#ifndef USE_OSG_PLUGIN

  // We will load here the coreModel
  // from .cfg file using global function
  // ****************************************
  // Creating a core model, the template
  osg::ref_ptr<osgCal::CoreModel> coreModelReferenced	= new osgCal::CoreModel();
  osgCal::CoreModel *coreModel = coreModelReferenced.get();

  // Load model attributes from file
  bool bSuccess = loadCoreModelFromCFG(	path,
                                        absFichierConf,
                                        coreModel);

  if( !bSuccess )
  {
    CERR(this,"loadCal3dFile")<< "Cannot load correctly model config file." << std::endl;
    return false;
  }
#else

  // Load model using plugin, osgPlugin cfg
  // **************************************
  osg::Object* object	= osgDB::readObjectFile(absFichierConf);
  if (!object) {
    CERR(this,"loadCal3dFile")<<"File "<< absFichierConf<<"incorrect"<<endl;
    return false;
  }
  osg::ref_ptr<osgCal::CoreModel> coreModelRef	= dynamic_cast<osgCal::CoreModel*>( object );
  osgCal::CoreModel* coreModel	= coreModelRef.get();

#endif

  //ajout 18/09/05 pour le pouilleux
  _model = new osgCal::Model(coreModel->getCalCoreModel());

  //retrait 18/09/05 pour le pouilleux
  /*if(!_model->setCoreModel(coreModel))
  {
    CERR(this,"loadCal3dFile") << "setCoreModel: " << CalError::getLastErrorDescription().c_str() << std::endl;
    return false;
    }*/

  for(int coreMeshId = 0; coreMeshId < _model->getCalCoreModel()->getCoreMeshCount(); coreMeshId++)
  {
    _model->getCalModel()->attachMesh(coreMeshId);
  }

  // set the material set of the whole model
  _model->getCalModel()->setMaterialSet(0);

  // Creating a concrete model using the core template
  if(!_model->create())
  {
    CERR(this,"loadCal3dFile") << "create: " << CalError::getLastErrorDescription().c_str() << std::endl;
    return false;
  }

  CTRACE(this,"loadCal3dFile")<<" SUCCES!!!"<<endl;
  
  //post traitement pour mapper les animations
  MapRW::tabStrStr fichiers;
  MapRW::tabStrStr::iterator itFic;
  mapp.loadTabStrStr(absFichierConf,fichiers,'=');
  itFic=fichiers.begin();
  int id=0;
  
  while (itFic != fichiers.end())
  {
    if (itFic->nom == "animation")
    {
      CalCoreAnimation * anim = _model->getCalModel()->getCoreModel()->getCoreAnimation(id);
      assertC(anim->getListCoreTrack().size());
      string ani = (itFic->champs).substr(0,(itFic->champs).length()-4);
      cout<<"animation : "<<ani<<endl;

      _mapAnim[ani]=id;
      id++;

      CalCoreTrack *coreTrack = anim->getCoreTrack(0);
      if (!coreTrack)
      {
        CWARN(this,"loadCal3dFile")<<" taille:"<<anim->getListCoreTrack().size()<<endl;
        CalCoreTrack * track = *(anim->getListCoreTrack().begin());
        track->setCoreBoneId(0);
        CWARN(this,"loadCal3dFile")<<" correction effectué sur l'ID du premier Track"<<endl;
        CWARN(this,"loadCal3dFile")<<" path+itFic->champs="<<path+itFic->champs<<endl;
      }
    }
    itFic++;
   }
  return true;
}//@CODE_1528


bool CharacterOsgCal::loadXML(const xmlpp::Node* node, LoaderXML* loaderXML)
{//@CODE_1530
  cout<<"LoadPersonnageOsgCal &&&&&&&&&&&&&&&&&&&&&&&&&"<<endl;

  this->Character::loadXML(node);

  XMLUtil xml(node);

  /*setName(xml.getFirstText("nom"));  
  _attraction = atof(xml.getFirstText("attraction").c_str());
  _posXBufferLoad=atof(xml.getFirstText("positionX").c_str());
  _posYBufferLoad=atof(xml.getFirstText("positionY").c_str());
  _elevatZ=atof(xml.getFirstText("elevationZ").c_str());
  _blendTime=atof(xml.getFirstText("blendTime").c_str());
  _vitesseDep= atof(xml.getFirstText("vitesseDeplacement").c_str());
  _vitesseRot= atof(xml.getFirstText("vitesseRotation").c_str());*/
  
  
  if (!loadCal3dFile(LoaderXML::getPath()+xml.getFirstText("cal3d"))){
    CERR(this,"loadXML")<<"Chargement Character "<<getName()<<" a échoué ->destruction"<<endl;
    this->~CharacterOsgCal();
    return false;    
  }
  //_model->getCalModel()->getCoreModel()->get

  buildRootNode(getModel());



  //setDureeAnim("cally_walk",1.5);



  //GetModel()->action(_mapAnim["cally_walk"],1.0,20.0);

  //GetModel()->loop(_mapAnim["cally_walk"], 1.0f, 0.0f);
  //GetModel()->stop(_mapAnim["cally_walk"],3);

  /*osgCal::CoreModel * temp = const_cast<osgCal::CoreModel *>(GetModel()->getCoreModel());
    CalCoreModel * core = temp->get();
    CalCoreSkeleton *skel= core->getCoreSkeleton();
    
    std::vector< CalCoreBone * > & listBone= skel->getVectorCoreBone ();
    
    for (GLuint i=0; i<listBone.size();i++) cout<<"nom bone="<<listBone[i]->getName()<<endl;
    
    CalCoreBone * pied= skel->getCoreBone ("Cally R Foot");
    
    pied->calculateBoundingBox(core);
    
    CalBoundingBox & boxPied = pied->getBoundingBox();
    cout<<"boxPied.plane[0].a="<<boxPied.plane[0].a<<endl;
    CalVector points[8];
    boxPied.computePoints(points);
    for (GLuint i =0; i<8 ;i++) cout<<"x,y,z ="<<points[i].x<<","<<points[i].y<<","<<points[i].z<<endl;
  */

  return true;
}//@CODE_1530


Evaluable* CharacterOsgCal::staticCreate(BD* pBD)
{//@CODE_3048
    return new CharacterOsgCal(pBD);
}//@CODE_3048


/*@NOTE_4088
Check the integritry of the object
*/
bool CharacterOsgCal::check()
{//@CODE_4088
       
    return true;
}//@CODE_4088


/*@NOTE_1517
Constructor method.
*/
CharacterOsgCal::CharacterOsgCal(BD* pBD) //@INIT_1517
    : 
    Character(pBD),
    _model(NULL),
    _osgCalMap(),
    _mapAnim()
{//@CODE_1517
  ConstructorInclude();
  
  //osg::ref_ptr<osgCal::Model> modelReferenced = new osgCal::Model();
  //_model = modelReferenced.get();

  //_pTimeBar = new TimeBar();
  // Put in your own code
}//@CODE_1517


/*@NOTE_1506
Destructor method.
*/
CharacterOsgCal::~CharacterOsgCal()
{//@CODE_1506
  DestructorInclude();

  cout<<"destruction Perso"<<endl;

  //_model->unref();
  //delete _model;
  //delete _pTimeBar;

  // Put in your own code
}//@CODE_1506


// End of Group: load

//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_1507
Method which must be called first in a constructor.
*/
void CharacterOsgCal::ConstructorInclude()
{
}


/*@NOTE_1508
Method which must be called first in a destructor.
*/
void CharacterOsgCal::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
} //end of namespace cooki3d

#endif



