/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          CharacterSeqAnim.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'CharacterSeqAnim'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include "CharacterSeqAnim.h"
//#include <osg/MatrixTransform>
#include <osg/PositionAttitudeTransform>
#include "MapRW.h"
#include "EnvironnementOSG.h"
#include "SolGravitation.h"
#include <OpenThreads/Thread>
#include <osg/AnimationPath>
#include <osg/NodeCallback>
#include <osg/Quat>
#include <osg/Matrixd>
#include "LoaderXML.h"
#include <osgDB/ReadFile>

#include "Animation.h"
#include "Sequence.h"
#include "MediaEvaluable.h"
#include "AnimationGroup.h"

namespace cooki3d{
//@END_USER2





//
// Group: animation
//
double CharacterSeqAnim::getDureeAnim(std::string anim)
{//@CODE_1555

  assertC(_mapMediaEval.find(anim)!=_mapMediaEval.end());

  return  _mapMediaEval[anim]->getDuration();
  

}//@CODE_1555


void CharacterSeqAnim::setDureeAnim(std::string anim, double duree)
{//@CODE_1559


  assertC(_mapMediaEval.find(anim)!=_mapMediaEval.end());

  _mapMediaEval[anim]->setDuration(duree);

}//@CODE_1559


void CharacterSeqAnim::startAnimation(std::string nom)
{//@CODE_1562

  assertC(_mapMediaEval.find(nom)!=_mapMediaEval.end());

  _mapMediaEval[nom]->play();

}//@CODE_1562


void CharacterSeqAnim::stopAnimation(std::string nom)
{//@CODE_1564
  assertCMessage(_mapMediaEval.find(nom)!=_mapMediaEval.end(),
		 "(nom=\""<<nom<<"\")");

  _mapMediaEval[nom]->stop();
}//@CODE_1564


// End of Group: animation


//
// Group: load
//
bool CharacterSeqAnim::loadXML(const xmlpp::Node* node, LoaderXML* loaderXML)
{//@CODE_1557
    
  //load character base properties...
  this->Character::loadXML(node,loaderXML);  

  XMLUtil xml(node);

  float scale = atof(xml.getFirstText("scale").c_str());
  setScale(osg::Vec3(scale,scale,scale));
  _osgNodeName= xml.getFirstText("osgNodeName"); 

  
  
  int checkSum = 0;
  

  Evaluable * eval = NULL;
  {
    vector<const xmlpp::Element*> animNodes= xml.getAllElement("animation");
    checkSum = checkSum + animNodes.size();
    Animation *anim;
    for (unsigned int i=0; i<animNodes.size();i++) {
      
      eval = loaderXML->loadEvaluableXML( animNodes[i], "animation" );
      if (eval){
	
	anim = static_cast<Animation*>(eval);
	_mapMediaEval[anim->getName()] = anim;
      }
      /*anim = new Animation();
	anim->loadXML(animNodes[i],loaderXML);
	_mapMediaEval[anim->getName()] = anim;  */
    }
  }

  {
    vector<const xmlpp::Element*> animGroupNodes= 
      xml.getAllElement("animationGroup");
    checkSum = checkSum + animGroupNodes.size();
    AnimationGroup *animGroup;
    for (unsigned int i=0; i<animGroupNodes.size();i++) {
    
      /* animGroup = new AnimationGroup();
	 animGroup->loadXML(animGroupNodes[i],loaderXML);*/

      eval = loaderXML->loadEvaluableXML( animGroupNodes[i], 
					  "animationGroup" );
      if (eval){
	
	animGroup = static_cast<AnimationGroup*>(eval);
	_mapMediaEval[animGroup->getName()] = animGroup;   
      }

      
    }
  }

  {
    vector<const xmlpp::Element*> seqNodes= xml.getAllElement("sequence");
    checkSum = checkSum + seqNodes.size();
    Sequence *seq;
    for (unsigned int i=0; i<seqNodes.size();i++) {    
      /*seq = new Sequence();
	seq->loadXML(seqNodes[i],loaderXML);*/

       eval = loaderXML->loadEvaluableXML( seqNodes[i], 
					  "sequence" );
       if (eval){
	 
	 seq = static_cast<Sequence*>(eval);
	 _mapMediaEval[seq->getName()] = seq;
       }
      
    }   
  } 

  //verifs
  if (checkSum==0){
    //on a un perso qui possède aucun noeud animés
    CWARNs("CharacterSeqAnim::loadXML")
      <<" Le personnage séquencé "<<getName()
      <<" ne possède aucun noeud animés!!!!!"
      <<endl;
  }
      
  return true;
}//@CODE_1557


bool CharacterSeqAnim::linkCharacterSeqAnim(osg::Node* n,
                                            const std::string& pickName,
                                            LoaderGeneric* loader)
{//@CODE_1888
  
  //cout<<" passage linkCharacterSeqAnim sur le Node "<<typeid(*n).name()<<endl;
  assertC2(n, loader->getBD());

  //CTRACEs("CharacterSeqAnim::linkCharacterSeqAnim ")<<n->getName()<<endl;

  //std::string pickName = n->getName();
  
  BD::EvaluableIterator itMed(loader->getBD());
  
  CharacterSeqAnim * psa;
  
  while (++itMed){
    psa=dynamic_cast<CharacterSeqAnim *>(itMed.Get());
    if (psa && (psa->getOsgNodeName() == pickName) 
	&& !psa->getRootNode()->asGroup()->getNumChildren()) {
      
      psa->buildRootNode(n/*, loaderXML->getRootNode()*/ );
      
      //loaderXML->getRootNode()->addChild( psa->getRootNode() );
       
      n->getParent(0)->asGroup()
	->insertChild(  n->getParent(0)->asGroup()->getChildIndex( n ),
			psa->getRootNode() );

      n->getParent(0)->asGroup()->removeChild(n);
      
      CINFOs<<"CharacterSeqAnim::linkCharacterSeqAnim "<<psa->getName()<<" sur pickName="<<pickName<<endl;
      return true;
    }
  }
  
  return false;
  
}//@CODE_1888


Evaluable* CharacterSeqAnim::staticCreate(BD* pBD)
{//@CODE_3047
    return new CharacterSeqAnim(pBD);
}//@CODE_3047


/*@NOTE_4089
Check the integritry of the object
*/
bool CharacterSeqAnim::check()
{//@CODE_4089
       
    return Character::check();
}//@CODE_4089


/*@NOTE_1554
Constructor method.
*/
CharacterSeqAnim::CharacterSeqAnim(BD* pBD) //@INIT_1554
    : 
    Character(pBD),
     _mapMediaEval()
    , _osgNodeName("null")
{//@CODE_1554
  ConstructorInclude();
  

  //_pTimeBar = new TimeBar();
  // Put in your own code
}//@CODE_1554


/*@NOTE_1545
Destructor method.
*/
CharacterSeqAnim::~CharacterSeqAnim()
{//@CODE_1545
  DestructorInclude();

  CTRACE(this,"~CharacterSeqAnim()")<<"destruction CharacterSeqAnim"<<endl;

  //_model->unref();
  //delete _model;
  //delete _pTimeBar;

  // Put in your own code
}//@CODE_1545


// End of Group: load

//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_1546
Method which must be called first in a constructor.
*/
void CharacterSeqAnim::ConstructorInclude()
{
}


/*@NOTE_1547
Method which must be called first in a destructor.
*/
void CharacterSeqAnim::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
} //end of namespace cooki3d
