/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          EnvironnementOSG.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'EnvironnementOSG'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _ENVIRONNEMENTOSG_H
#define _ENVIRONNEMENTOSG_H

//@START_USER1


#include <osg/Group>
#include <osg/ArgumentParser>
#include <osgGA/GUIEventHandler>
#include <osgGA/GUIActionAdapter>
#include <osgUtil/IntersectVisitor>


#include <SDL/SDL.h>
#include <SDL/SDL_mutex.h>
#include <list>

#include "Command.h"
#include "CookiMessage.h"


namespace cooki3d{

  class SimpleHit {
  public:
    osg::Drawable * _drawable;
    osg::Geode * _geode;
    
    osg::Drawable * getDrawable(){
      return _drawable;
    }
    
    osg::Geode * getGeode(){
      return _geode;
    }
    
    SimpleHit() : _drawable(NULL),_geode(NULL) {}
    
  };
  
  class EnvironnementInstanceException {};
   
  class LoaderXML;
  class LoaderGeneric;
//@END_USER1


/*@NOTE_803
class ...
*/

class AFX_EXT_CLASS EnvironnementOSG
    : public virtual osgGA::GUIActionAdapter
{

//@START_USER2
//@END_USER2

//
// Group: ClassBuilder methods
//

private:
    void ConstructorInclude();
    void DestructorInclude();

//
// Group: Media
//

//
// Non-Grouped Members
//

protected:
    osg::ArgumentParser _parser;
    osg::ref_ptr<osg::Group> _rootNode;
    std::string _activeCamera;
    bool _haveBlockingRunMethod;
    LoaderGeneric* _loader;

//
// Non-Grouped Methods
//

public:
    EnvironnementOSG(int argc, char** argv, LoaderGeneric* loader);
    virtual ~EnvironnementOSG();
    virtual bool computeIntersections(float x, float y,
                                      osgUtil::IntersectVisitor::HitList& hitlist,
                                      osg::Node::NodeMask mask = 0xffffffff) = 0;
    virtual bool computeIntersectionsRectangle(int x1, int y1, int x2, int y2,
                                               std::vector<SimpleHit>& hitlist);
    virtual bool computeIntersectionsVertical(float x, float y,
                                              osgUtil::IntersectVisitor::HitList& hitlist,
                                              osg::Node* data);
    BD* getBD();
    virtual list<osg::ref_ptr<osgGA::GUIEventHandler> >& getEHList() = 0;
    virtual osg::FrameStamp* getFrameStamp() = 0;
    virtual osg::Matrix getViewMatrix() = 0;
    virtual bool isStopped() = 0;
    virtual void pushBack(osgGA::GUIEventHandler* handler);
    virtual void pushFront(osgGA::GUIEventHandler* handler);
    void readAndRun();
    virtual void requestContinuousUpdate(bool ) = 0;
    virtual void requestRedraw() = 0;
    virtual void requestWarpPointer(float x, float y) = 0;
    virtual void run(osg::Node* node) = 0;
    virtual void stop() = 0;
    std::string getActiveCamera() const;
    void setActiveCamera(std::string activeCamera);
    bool isHaveBlockingRunMethod() const;
    void setHaveBlockingRunMethod(bool haveBlockingRunMethod);
    LoaderGeneric* getLoader() const;
    void setLoader(LoaderGeneric* pLoader);
    osg::ArgumentParser getParser() const;
    void setParser(osg::ArgumentParser parser);
    osg::ref_ptr<osg::Group> getRootNode() const;
    void setRootNode(osg::ref_ptr<osg::Group> rootNode);
};

#endif


#ifdef CB_INLINES
#ifndef _ENVIRONNEMENTOSG_H_INLINES
#define _ENVIRONNEMENTOSG_H_INLINES

/*@NOTE_1233
Returns the value of member '_activeCamera'.
*/
inline std::string EnvironnementOSG::getActiveCamera() const
{//@CODE_1233
    return _activeCamera;
}//@CODE_1233



/*@NOTE_1234
Set the value of member '_activeCamera' to 'activeCamera'.
*/
inline void EnvironnementOSG::setActiveCamera(std::string activeCamera)
{//@CODE_1234
    _activeCamera = activeCamera;
}//@CODE_1234



/*@NOTE_4390
Returns the value of member '_haveBlockingRunMethod'.
*/
inline bool EnvironnementOSG::isHaveBlockingRunMethod() const
{//@CODE_4390
    return _haveBlockingRunMethod;
}//@CODE_4390



/*@NOTE_4391
Set the value of member '_haveBlockingRunMethod' to 'haveBlockingRunMethod'.
*/
inline void EnvironnementOSG::setHaveBlockingRunMethod(bool haveBlockingRunMethod)
{//@CODE_4391
    _haveBlockingRunMethod = haveBlockingRunMethod;
}//@CODE_4391



/*@NOTE_4833
Returns the value of member '_loader'.
*/
inline LoaderGeneric* EnvironnementOSG::getLoader() const
{//@CODE_4833
    return _loader;
}//@CODE_4833



/*@NOTE_945
Returns the value of member '_parser'.
*/
inline osg::ArgumentParser EnvironnementOSG::getParser() const
{//@CODE_945
    return _parser;
}//@CODE_945



/*@NOTE_946
Set the value of member '_parser' to 'parser'.
*/
inline void EnvironnementOSG::setParser(osg::ArgumentParser parser)
{//@CODE_946
    _parser = parser;
}//@CODE_946



/*@NOTE_4780
Returns the value of member '_rootNode'.
*/
inline osg::ref_ptr<osg::Group> EnvironnementOSG::getRootNode() const
{//@CODE_4780
    return _rootNode;
}//@CODE_4780



/*@NOTE_4781
Set the value of member '_rootNode' to 'rootNode'.
*/
inline void EnvironnementOSG::setRootNode(osg::ref_ptr<osg::Group> rootNode)
{//@CODE_4781
    _rootNode = rootNode;
}//@CODE_4781



//@START_USER3
} //end of namespace cooki3d
//@END_USER3

#endif
#endif
