/** -*-c++-*- *************************************************************\
* File:          EvtAdapterSDL.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'EvtAdapterSDL'
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
// -*- c++ -*-

#ifndef _COOKI_SDLEVENTADAPTER_
#define _COOKI_SDLEVENTADAPTER_

namespace cooki3d {

enum SDLUserEvent {
  USER_FRAME_EVENT,
  USER_UPDATE_EVENT,
  USER_INIT_EVENT
};

/**
 * class EvtAdapterSDL
 */
class EvtAdapterSDL : public osgGA::GUIEventAdapter {
 public:
  inline EvtAdapterSDL(const SDL_Event &event, 
			 int x1, int y1, int x2, int y2, 
			 double t, int btnstate_);
  
 
  inline virtual EventType getEventTypeCompute() const;
  inline virtual int getKeyCompute() const;
  inline virtual int getButtonCompute() const;
  inline virtual unsigned int getButtonMaskCompute() const;
  inline virtual float getXCompute() const;
  inline virtual float getYCompute() const;
  
 
  // inline virtual unsigned int getModKeyMask() const;  
  // inline virtual float getXmin() const;
  // inline virtual float getYmin() const;
  // inline virtual float getXmax() const;
  // inline virtual float getYmax() const;  
  // inline virtual double time() const;

  //specific SDL
  inline const SDL_Event &getSDLEvent() const;
  inline virtual int getUserEventID() const;
  inline virtual void *getUserEventData() const;
  inline virtual void *getUserEventData2() const;
  inline virtual SDLKey getSDLKey() const;		
  
 protected:
  virtual ~EvtAdapterSDL() {}
  //EvtAdapterSDL &operator=(const EvtAdapterSDL &) { return *this; }
  
 private:
  SDL_Event event_;
  //double t_;
  //int x1_;
  //int x2_;
  //int y1_;
  //int y2_;
  int btnstate_;
};

// INLINE FUNCTIONS

inline EvtAdapterSDL::EvtAdapterSDL(const SDL_Event &event, 
					int x1, int y1, int x2, int y2, 
					double t, int btnstate)
  : event_(event), 	  
    btnstate_(btnstate)
{
  setTime( t );
  setEventType( getEventTypeCompute() );
  setKey( getKeyCompute() );
  setButton( getButtonCompute() );
  setButtonMask( getButtonMaskCompute() );

  // setXmin( x1 );
//   setYmin( y1 );
//   setXmax( x2 );
//   setYmax( y2 );

  setX( float(getXCompute()) / float(x2) );
  setY( float(getYCompute()) / float(y2) );
}

inline EvtAdapterSDL::EventType EvtAdapterSDL::getEventTypeCompute() const
{
  switch (event_.type) {
    
  case SDL_USEREVENT:
    if (event_.user.code == USER_FRAME_EVENT){
      return FRAME;
    }
    return static_cast<EventType>(0xffff);
    
  case SDL_KEYDOWN:
    return KEYDOWN;
    
  case SDL_KEYUP:
    return KEYUP;
    
  case SDL_MOUSEMOTION:
    if (getButtonMaskCompute() == 0) {
      return MOVE;
    }
    else {
      return DRAG;
    }
    
  case SDL_MOUSEBUTTONDOWN:				
    return PUSH;
    
  case SDL_MOUSEBUTTONUP:				
    return RELEASE;
    
  case SDL_VIDEORESIZE:
    return RESIZE;
    
  default: ;
  }
  return NONE;
}



inline int EvtAdapterSDL::getKeyCompute() const
{
  if (event_.type == SDL_KEYDOWN || event_.type == SDL_KEYUP) {
    if ( event_.key.keysym.unicode == 0){
      switch ( event_.key.keysym.sym ) {
	
      case SDLK_UP : 
	return KEY_Up;
	//break;
      case SDLK_DOWN :
	return KEY_Down;
	//break;
      case SDLK_RIGHT:
	return KEY_Right;
	//break;
      case SDLK_LEFT:
	return KEY_Left;
	//break;...
       	
      case SDLK_INSERT: 
	return KEY_Insert;
      case SDLK_HOME: 
	return KEY_Home;
      case SDLK_END:
	return KEY_End; 	       
      case SDLK_PAGEUP: 		
	return KEY_Page_Up;
      case SDLK_PAGEDOWN: 		
	return KEY_Page_Down;
      case SDLK_F1: 		
	return KEY_F1;
      case SDLK_F2: 		
	return KEY_F2;
      case SDLK_F3: 		
	return KEY_F3;
      case SDLK_F4: 		
	return KEY_F4;
      case SDLK_F5: 		
	return KEY_F5;
      case SDLK_F6: 		
	return KEY_F6;
      case SDLK_F7: 		
	return KEY_F7;
      case SDLK_F8: 		
	return KEY_F8;
      case SDLK_F9: 		
	return KEY_F9;
      case SDLK_F10: 		
	return KEY_F10;
      case SDLK_F11: 		
	return KEY_F11;
      case SDLK_F12: 		
	return KEY_F12;
      case SDLK_F13: 		
	return KEY_F13;
      case SDLK_F14: 		
	return KEY_F14;
      case SDLK_F15:
	return KEY_F15;
		
      default:
	return 0;
	break;
      }
      
    }

    return static_cast<int>(event_.key.keysym.unicode);
  }
  return -1;
}

// inline unsigned int EvtAdapterSDL::getModKeyMask() const
// {
//   return 0;
// }


	
inline int EvtAdapterSDL::getButtonCompute() const
{
  if (event_.type == SDL_MOUSEBUTTONDOWN 
      || event_.type == SDL_MOUSEBUTTONUP) {
    switch (event_.button.button) {
    case SDL_BUTTON_LEFT:
      return LEFT_MOUSE_BUTTON;
    case SDL_BUTTON_MIDDLE:
      return MIDDLE_MOUSE_BUTTON;
    case SDL_BUTTON_RIGHT:
      return RIGHT_MOUSE_BUTTON;
    default: ;
    }
  }
  return -1;
}

// inline float EvtAdapterSDL::getXmin() const
// {
//   return x1_;
// }

// inline float EvtAdapterSDL::getYmin() const
// {
//   return y1_;
// }

// inline float EvtAdapterSDL::getXmax() const
// {
//   return x2_;
// }

// inline float EvtAdapterSDL::getYmax() const
// {
//   return y2_;
// }

inline float EvtAdapterSDL::getXCompute() const
{
  switch (event_.type) {
  case SDL_MOUSEMOTION:
    return event_.motion.x;
  case SDL_MOUSEBUTTONDOWN:
  case SDL_MOUSEBUTTONUP:
    return event_.button.x;
  default: ;
  };
  return 0;
}

inline float EvtAdapterSDL::getYCompute() const
{
  switch (event_.type) {
  case SDL_MOUSEMOTION:
    //cout<<"EvtAdapterSDL::getY() motion="<<event_.motion.y<<endl;
    return event_.motion.y;
  case SDL_MOUSEBUTTONDOWN:
  case SDL_MOUSEBUTTONUP:
    //cout<<"EvtAdapterSDL::getY() button="<<event_.motion.y<<endl;
    return event_.button.y;
    

  default: ;
  };
  return 0;
}

inline unsigned int EvtAdapterSDL::getButtonMaskCompute() const
{
  unsigned int mask = 0;
  if (btnstate_ & SDL_BUTTON(1)) mask |= LEFT_MOUSE_BUTTON;
  if (btnstate_ & SDL_BUTTON(2)) mask |= MIDDLE_MOUSE_BUTTON;
  if (btnstate_ & SDL_BUTTON(3)) mask |= RIGHT_MOUSE_BUTTON;
  return mask;
}

// inline double EvtAdapterSDL::time() const
// {
//   //cerr<<"appel EvtAdapterSDL::time()="<<t_<<endl;
//   return t_;
// }


inline SDLKey EvtAdapterSDL::getSDLKey() const
{
  if (event_.type == SDL_KEYDOWN || event_.type == SDL_KEYUP) {
    return event_.key.keysym.sym;
  }
  return static_cast<SDLKey>(-1);
}

inline const SDL_Event &EvtAdapterSDL::getSDLEvent() const
{
  return event_;
}
inline int EvtAdapterSDL::getUserEventID() const
{
  if (event_.type == SDL_USEREVENT) {
    return event_.user.code;
  }
  return -1;
}

inline void *EvtAdapterSDL::getUserEventData() const
{
  if (event_.type == SDL_USEREVENT) {
    return event_.user.data1;
  }
  return 0;
}

inline void *EvtAdapterSDL::getUserEventData2() const
{
  if (event_.type == SDL_USEREVENT) {
    return event_.user.data2;
  }
  return 0;
}

}

#endif
