/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          EvtBoundingShapeCollision.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'EvtBoundingShapeCollision'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include <math.h>

#include "Event.h"
#include "EvtBoundingShapeCollision.h"
#include "XMLUtil.h"
#include "BD.h"
#include "TimeBar.h"
#include "Methode_0arg.h"
#include "Function_1arg.h"
#include "LoaderXML.h"

namespace cooki3d{
//@END_USER2


// Static members
MapRegister<EvtBoundingShapeCollision*> EvtBoundingShapeCollision::_register;



//
// Group: load
//
bool EvtBoundingShapeCollision::loadXML(const xmlpp::Node* node,
                                        LoaderXML* loaderXML)
{//@CODE_3701

  //cout<<"LoadEvtCollision &&&&&&&&&&&&&&&&&&&&&&&&&"<<endl;
  
  XMLUtil xmlEvt(node);

  setName(xmlEvt.getFirstText("nom"));

  _refreshPeriod =atoi(xmlEvt.getFirstText("refreshPeriod").c_str());

  _standbyDelay = atoi(xmlEvt.getFirstText("standbyDelay").c_str());


  _collision1 = dynamic_cast< BoundingShape * >
    (loaderXML->getBD()->FindEvaluable(xmlEvt.getFirstText("collision1")));
  if (!_collision1) {
    CERR(this,"loadXML")<<"l'objet Collision :"
			<<xmlEvt.getFirstText("collision1")
			<<" demande n'existe pas ->destruction"<<endl;
    this->~EvtBoundingShapeCollision();
    return false;
  }


  _collision2 = dynamic_cast< BoundingShape * >
    (loaderXML->getBD()->FindEvaluable(xmlEvt.getFirstText("collision2")));
  if (!_collision1) {
    CERR(this,"loadXML")<<"l'objet Collision :"
			<<xmlEvt.getFirstText("collision2")
			<<" demande n'existe pas ->destruction"<<endl;
    this->~EvtBoundingShapeCollision();
    return false;
  }

  
  //"code"+getName())
  //string codeName = getName();
  //codeName[0] = (char((int('A')-int('a'))+int(codeName[0])));
  //codeName = "code"+ codeName;
  setCode( xmlEvt.getFirstText("code") );
  CTRACE(this,"loadXML")<<getCode()<<endl;
  
  Evaluable * evTmp = loaderXML->getBD()->FindEvaluable(getCode());
  //BUG_C++_JVM sur dynamic_cast quand load fonction interne depuis Editeur
  //_codeExec =dynamic_cast< FctEvtBoundingShapeCollision * >(evTmp);
  _codeExec =static_cast< FctEvtBoundingShapeCollision * >(evTmp);    
  
  if (!_codeExec){
    CERR(this,"loadXML")<<"la commande d'action sur evt "
			<</*codeName*/getCode()
			<<" n'existe pas ->destruction"<<endl;
    this->~EvtBoundingShapeCollision();
    return false;
  }
  //_codeExec->setArg1(this);
  EvtBoundingShapeCollision * p = this;
  _codeExec->pSetArg1(&p);

  return load();
}//@CODE_3701


Evaluable* EvtBoundingShapeCollision::staticCreate(BD* pBD)
{//@CODE_3705
    return new EvtBoundingShapeCollision(pBD);
}//@CODE_3705


/*@NOTE_3699
Constructor method.
*/
EvtBoundingShapeCollision::EvtBoundingShapeCollision(BD* pBD) //@INIT_3699
   
  : Evaluable(pBD) //heritage virtual
    ,Event(pBD)
    , _X()
    , _Y()
    , _codeExec(NULL)
    , _collision1(NULL)
    , _collision2(NULL)
    , _standbyDelay( 1.0f )
    , _refreshPeriod( 1 )
   
{//@CODE_3699
  
  ConstructorInclude();    
  // Put in your own code
  
  //cf. load()
  //_tick = osg::Timer::instance()->tick();
  
}//@CODE_3699


/*@NOTE_3669
Destructor method.
*/
EvtBoundingShapeCollision::~EvtBoundingShapeCollision()
{//@CODE_3669
    DestructorInclude();
    // Put in your own code
        
    _register.getRegister( getBD() ).remove( getName() );
    
}//@CODE_3669


bool EvtBoundingShapeCollision::load()
{//@CODE_3929
  
  //on initialise afin que l'evt soit tout de suite actif
  _tick = osg::Timer::instance()->tick() - osg::Timer_t
    (_standbyDelay / osg::Timer::instance()->getSecondsPerTick() ) ;
  
  _register.getRegister( getBD() ).add( getName(), this );
  
  return true;
}//@CODE_3929


// End of Group: load

void EvtBoundingShapeCollision::executerActions()
{//@CODE_3700
  assertC(_codeExec);
  //_codeExec->setArg1(this);  
  EvtBoundingShapeCollision * p = this;
  _codeExec->pSetArg1(&p);  
  _codeExec->exec();
}//@CODE_3700


void EvtBoundingShapeCollision::test()
{//@CODE_3706
    
  osg::Timer_t testTick = osg::Timer::instance()->tick();
  double delta = osg::Timer::instance()->delta_s(_tick,testTick);
  
  //CERR(this,"test")<<"delta="<<delta<<" _standbyDelay="<<_standbyDelay<<endl;
  //CERR(this,"test")<<BoundingShape::_lastFrameNumber<<" % "
  //	   <<_refreshPeriod<<" = "
  //	   <<BoundingShape::_lastFrameNumber % _refreshPeriod<<endl;
  
  if (BoundingShape::_lastFrameNumber > 0 
      && delta > _standbyDelay 
      && (BoundingShape::_lastFrameNumber % _refreshPeriod) == 0
      && _collision1->hasCollisionBetween(_collision2)  ) {
    
    executerActions();
    _tick = testTick;
  
  }
  
}//@CODE_3706


//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_3670
Method which must be called first in a constructor.
*/
void EvtBoundingShapeCollision::ConstructorInclude()
{
}


/*@NOTE_3671
Method which must be called first in a destructor.
*/
void EvtBoundingShapeCollision::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
} //end of namespace cooki3d

//poubelle bug JVM-C++
/*
try {     
    _codeExec =dynamic_cast< FctEvtBoundingShapeCollision * >(evTmp);
    // _codeExec =static_cast< FctEvtBoundingShapeCollision * >(evTmp);    
  }
  catch(const bad_cast& ex) {
    std::cout << "Exception caught: " << ex.what() << std::endl;
  }
  
  if (!_codeExec){
    CERR(this,"loadXML")<<"la commande d'action sur evt "
		      <<"codeEvtCollision0"<<" n'existe pas ->destruction"<<endl;
    if (evTmp){
      CERR(this,"loadXML")<<"la classe trouvÃ© est "<<typeid(*evTmp).name()<<endl;
      CERR(this,"loadXML")<<"la classe de base trouvÃ© est "<<typeid(evTmp).name()<<endl;

      CERR(this,"loadXML")<<"la classe attendu est "<<typeid(FctEvtBoundingShapeCollision).name()<<endl;
      
      CERR(this,"loadXML")<<"la classe de base attendu est "<<typeid(FctEvtBoundingShapeCollision *).name()<<endl;
      
    }


    this->~EvtBoundingShapeCollision();
    return false;
  }
*/
