/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          File.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'File'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _FILE_H
#define _FILE_H

//@START_USER1
#include <set>

#include "Evaluable.h"
#include "XMLUtil.h"

namespace cooki3d{
//@END_USER1


/*@NOTE_1389
class ...
*/

class AFX_EXT_CLASS File
    : public Evaluable
{

//@START_USER2
protected:
  osg::PositionAttitudeTransform * _pat;
  osg::ref_ptr<osg::Node> _levelNode;
public:
  osg::PositionAttitudeTransform * getPat(){ return _pat;}
  osg::Node * getFileRootNode() { return (osg::Node*)_pat; }
  osg::ref_ptr<osg::Node> & getLevelNode(){ return _levelNode;}

  typedef map<string,int,less<string > > mapStrInt;
              
//@END_USER2

//
// Group: ClassBuilder methods
//

private:
    void ConstructorInclude();
    void DestructorInclude();

//
// Non-Grouped Members
//

private:
    std::set<std::string> _pathSet;
    mapStrInt _pathCount;

protected:
    std::string _fichier;
    std::vector<osg::Geode*> _listGeode;
    std::vector<osg::Group*> _listGroup;
    bool _pathUnicity;

//
// Non-Grouped Methods
//

public:
    File(BD* pBD);
    virtual ~File();
    bool load(double x = 0, double y = 0, double altitude = 0, double scale = 1,
              osg::Group* rootNode = NULL);
    virtual bool loadXML(const xmlpp::Node* node, LoaderXML* loaderXML);
    bool save(std::string fileName);
    virtual void scanRenameCommand(osg::Node* n, osg::Node* parent,
                                   std::string path);
    static Evaluable* staticCreate(BD* pBD);
    std::string getFichier() const;
    void setFichier(std::string fichier);
    const std::vector<osg::Geode*>& getListGeode() const;
    std::vector<osg::Group*> getListGroup() const;
    bool isPathUnicity() const;
    void setPathUnicity(bool pathUnicity);
};

#endif


#ifdef CB_INLINES
#ifndef _FILE_H_INLINES
#define _FILE_H_INLINES

/*@NOTE_1399
Returns the value of member '_fichier'.
*/
inline std::string File::getFichier() const
{//@CODE_1399
    return _fichier;
}//@CODE_1399



/*@NOTE_1400
Set the value of member '_fichier' to 'fichier'.
*/
inline void File::setFichier(std::string fichier)
{//@CODE_1400
    _fichier = fichier;
}//@CODE_1400



/*@NOTE_3535
Returns the value of member '_listGeode'.
*/
inline const std::vector<osg::Geode*>& File::getListGeode() const
{//@CODE_3535
    return _listGeode;
}//@CODE_3535



/*@NOTE_3598
Returns the value of member '_listGroup'.
*/
inline std::vector<osg::Group*> File::getListGroup() const
{//@CODE_3598
    return _listGroup;
}//@CODE_3598



/*@NOTE_3962
Returns the value of member '_pathUnicity'.
*/
inline bool File::isPathUnicity() const
{//@CODE_3962
    return _pathUnicity;
}//@CODE_3962



/*@NOTE_3963
Set the value of member '_pathUnicity' to 'pathUnicity'.
*/
inline void File::setPathUnicity(bool pathUnicity)
{//@CODE_3963
    _pathUnicity = pathUnicity;
}//@CODE_3963



//@START_USER3
} //end of namespace cooki3d
//@END_USER3

#endif
#endif
