/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          HeightMap.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'HeightMap'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include "HeightMap.h"


#include <fstream>
#include <sstream>
#include <iostream>

#include <osg/Geode>
#include "ScanSetGraph.h"
#include "CalculBoundingBox.h"
#include <osg/Group>

#include <osg/StateSet>
#include <osg/Material>

#include <osg/Version>

#include "EnvironnementGeneric.h"
//#ifndef CPP_VERSION
//#error "CPP_VERSION ???"
//#endif

//#if CPP_VERSION >= 4
inline std::ostream & operator<<(std::ostream & os, osg::Vec3 v){
  
  os<<v[0]<<","<<v[1]<<","<<v[2]<<endl;
  return os;
  
}
//#endif

namespace cooki3d{
//@END_USER2




/*@NOTE_2180
Constructor method.
*/
HeightMap::HeightMap(osg::Node * terrain, EnvironnementOSG* env) //@INIT_2180

    : _tail_impo(false)
    , _map_x(_columns)
    , _map_y(_rows)
    , _box()
    , _visual_box()
    , _range()
    , _terrain(terrain)
    , _env(env)
{//@CODE_2180
    ConstructorInclude();

    // Put in your own code
}//@CODE_2180


/*@NOTE_2163
Destructor method.
*/
HeightMap::~HeightMap()
{//@CODE_2163
    DestructorInclude();

    // Put in your own code
}//@CODE_2163


bool HeightMap::contains(float x, float y)
{//@CODE_5280
    return _visual_box.contains(osg::Vec3(x,y,_visual_box.center().z())); 
    
}//@CODE_5280


bool HeightMap::generer()
{//@CODE_2218
  CTRACE(this,"loadRawFile()")<<" Génération d'une heightMap ..."<<endl;
  
  CalculBoundingBox cbb;
  
  _box = cbb.getBoundingBoxSimple(_terrain); //g->getBoundingBox();
  
  if (cbb.isCalculBoxOK()){
    
    osg::Vec3 dep=_terrain->getBound().center()-_box.center();
    
    _visual_box.set(_box._min+dep,_box._max+dep);
    
    
    cout<<"boundingBox min = "<<_box._min<<endl;
    cout<<"boundingBox max = "<<_box._max<<endl;
    cout<<"visual box min = "<<_visual_box._min<<endl;
    cout<<"visual box max = "<<_visual_box._max<<endl;
    
    setOrigin(_visual_box._min); //osg::HeightField (osg::Grid) appel
    
    //////////////////////////////
    _range.set(_box.xMax()-_box.xMin(),
	      _box.yMax()-_box.yMin(),
	      _box.zMax()-_box.zMin());

    cout<<"range = "<<_range.x()<<","<<_range.y()<<endl;


    //@TODO modifier ca pour SolCollision
    float finesse=2.0f; //finesse de la grille
    allocate((int)fabs(_range.x()*finesse),(int)fabs(_range.y()*finesse));
    
    cout<<"Taille de la map = "<<_map_x<<" * "<<_map_y<<endl;
    cout<<"Finesse de la maille = "<<finesse<<endl; 

    osgUtil::IntersectVisitor::HitList hits;



    float fx,fy;
    float fmap_x= (float)_map_x;
    float fmap_y= (float)_map_y;
    
    EnvironnementGeneric instance( 0, NULL, NULL);
	  
    for (GLuint y=0 ; y<_map_y ; y++){
      for (GLuint x=0 ; x<_map_x ; x++){

	fx=((float(x)+0.5f)*_range.x()/fmap_x)+getOrigin().x();
	fy=((float(y)+0.5f)*_range.y()/fmap_y)+getOrigin().y();

	//TODO Read the Z-BUFFER to increase performance of génération.
		
	/*EnvironnementOSG::*/instance/*()->*/.computeIntersectionsVertical(fx,fy,hits,_terrain);
	if (hits.size()>0) {
	  setHeight(x,y,hits[0].getWorldIntersectPoint()[2]);
	  //cout<<"hits[0].getWorldIntersectPoint()[2]="<<hits[0].getWorldIntersectPoint()[2]<<endl;
	  //cout<<'.';
	}else {
	  CERR(this,"HeightMap::generer()")
	    <<"erreur echec point :"<<endl
	    <<fx<<","<<fy<<"|"<<endl
	    <<" Abandon de la génération "<<endl;

	  /*cout<<endl<<_range.x()<<endl;
	    cout<<_range.y()<<endl;
	    cout<<getOrigin().x()<<endl;
	    cout<<getOrigin().y()<<endl;
	    cout<<fmap_x<<endl;
	    cout<<fmap_y<<endl;
	    cout<<float(x)<<endl;
	    cout<<float(y)<<endl;*/
	  
	  
	  return false;
	}
	hits.clear();	
      }
      if(  ( ((y*100)/_map_y) % 10 )==0 ){
	cerr<< ((y*100)/_map_y) <<"%;"<<flush;
      }      
    }
    return true;
 }
 CERR(this,"HeightMap::generer()")<<"erreur genereration"<<endl;
 return false;


 

}//@CODE_2218


float HeightMap::getAllHeightsBetween(float x1, float y1, float x2, float y2)
{//@CODE_2298
  cerr<<"ATTENTION CET ALGO EST FAUX"<<endl;
  assert(0);
    
    //vector<float>
    
    float signeX=1, signeY=1;
  
  if (x1 > x2){ 
      signeX =-1;
  }
  if (y1 > y2){
      signeY =-1;       
  }
  
      
    
  float dX = _range.x() / (float)_map_x;
  float dY = _range.y() / (float)_map_y;
    
  
  float x = x1, y=y1;
  
  //cout<<"dX="<<dX<<endl;    cout<<"dY="<<dY<<endl;

  if (y1 < y2){
  
    while (x<x2 && y<y2){
      
      getHeightAbs(x,y);      
      x = x +  signeX*dX;
      y = y +  signeY*dY;
      
    }
  }
    
  return 0;
    
}//@CODE_2298


osg::Vec3 HeightMap::getCentre()
{//@CODE_2201
    return _visual_box.center();
}//@CODE_2201


float HeightMap::getDeltaCase()
{//@CODE_2303
        
    float dX = _range.x() / (float)_map_x;
    float dY = _range.y() / (float)_map_y;
    
    assertC2( (dX > dY - 0.001f) &&  (dX < dY + 0.001f), _env->getBD() );
    
    return dX;

}//@CODE_2303


osg::Node* HeightMap::getGeode()
{//@CODE_2219
   //color array OK=green (WARNING=blue ERROR=red)
  
#ifdef UNIX
	enum couleur { OK=0 , WARNING=1, ERROR=2 ,NOIR=3};
#endif
  
  static int coloration = 0;
  coloration = (coloration+1)%4;
    
 //vertex array
  osg::Vec3Array* coords = new osg::Vec3Array;

  for(GLuint iy=0; iy<_map_y; ++iy) 
    for(GLuint ix=0; ix<_map_x; ++ix) {

      osg::Vec3 v(((float(ix)+0.5f)*_range.x()/float(_map_x))+_visual_box.xMin(),//getOrigin().x(),
		  ((float(iy)+0.5f)*_range.y()/float(_map_y))+_visual_box.yMin(),//getOrigin().y(),
		  getHeight(ix,iy)/*+_box.zMin()*/);
      /*if (v.z() < _box.zMin()) v.z() = _box.zMin();*/
      coords->push_back(v);
    }
  
  
 
  osg::Vec4Array* colors = new osg::Vec4Array;
  colors->push_back(osg::Vec4(0.0f,1.0f,0.0f,1.0f)); // green
  colors->push_back(osg::Vec4(0.0f,0.0f,1.0f,1.0f)); // blue
  colors->push_back(osg::Vec4(1.0f,0.0f,0.0f,1.0f)); // red
  colors->push_back(osg::Vec4(1.0f,1.0f,1.0f,1.0f)); // white
  colors->push_back(osg::Vec4(0.0f,0.0f,0.0f,1.0f)); // black
  //int numColors=colors->size();
  
  
  osg::UIntArray* coordIndices = new osg::UIntArray;
  osg::UIntArray* colorIndices = new osg::UIntArray;
  
  for(GLuint iy=0; iy<_map_y-1; ++iy)
    for(GLuint ix=0; ix<_map_x-1; ++ix) {
      
      // four vertices per quad.
      coordIndices->push_back(iy * _map_x + ix       ); //1     4    3    
      
      //if (A_des_voisins_egaux(ix,iy)) colorIndices->push_back(2);
      //else colorIndices->push_back(2);
      coordIndices->push_back(iy * _map_x + ix +1    ); //2      .  .
      //if (A_des_voisins_egaux(ix+1,iy)) colorIndices->push_back(2);
      //else colorIndices->push_back(2);
      coordIndices->push_back((iy+1) * _map_x + ix +1); //3      .  .
      //if (A_des_voisins_egaux(ix+1,iy+1)) colorIndices->push_back(2);
      //else colorIndices->push_back(2);
      coordIndices->push_back((iy+1) * _map_x + ix   ); //4     1    2
      /*if (A_des_voisins_egaux(ix,iy+1)) colorIndices->push_back(2);
	else*/ 
      colorIndices->push_back(coloration);
      
    }
  
  // set up normals
  osg::Vec3Array* normals = new osg::Vec3Array;
  for(GLuint iy=0; iy<_map_y-1; ++iy)
    for(GLuint ix=0; ix<_map_x-1; ++ix)
      normals->push_back(getNormal(ix,iy));
  
  osg::Geometry* geom = new osg::Geometry;
  geom->setVertexArray(coords);
  geom->setVertexIndices(coordIndices);
  
  geom->setColorArray(colors);
  geom->setColorIndices(colorIndices);
  //geom->setColorBinding(osg::Geometry::BIND_PER_VERTEX);
  geom->setColorBinding(osg::Geometry::BIND_PER_PRIMITIVE);
  
  geom->setNormalArray(normals);
  geom->setNormalBinding(osg::Geometry::BIND_PER_VERTEX);
  geom->setNormalIndices(coordIndices);
  
  geom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::QUADS,0,coordIndices->size()));
  //geom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::POINTS,0,coords->size()));
  
  osg::Geode* geode = new osg::Geode;
  geode->addDrawable(geom);    
  geode->setName("HeightMapMesh");

  osg::StateSet * ss= new osg::StateSet();
  osg::Material * om= new osg::Material();
  om->setColorMode(osg::Material::AMBIENT_AND_DIFFUSE);
  ss->setAttribute(om);
  
  geode->setStateSet(ss);
  
  return geode;
}//@CODE_2219


float HeightMap::getHeightAbs(float x, float y)
{//@CODE_2181
  
  if (_visual_box.contains(osg::Vec3(x,y,_visual_box.center().z()))) {
    
    return osg::Grid::getHeight(int(((x-getOrig().x())/_range.x())*_map_x),
				int(((y-getOrig().y())/_range.y())*_map_y));
  }
  else throw height_out();
 
}//@CODE_2181


float HeightMap::getHeightAbsNoException(float x, float y)
{//@CODE_5277
  return osg::Grid::getHeight(int(((x-getOrig().x())/_range.x())*_map_x),
			      int(((y-getOrig().y())/_range.y())*_map_y));
  
}//@CODE_5277


float HeightMap::getHeightMoy(float x1, float y1, float x2, float y2)
{//@CODE_2204
  float x=x1,y=y1;
  //int cpt=0;
    //float tot=0;
    float dX = _range.x() / (float)_map_x;
    float dY = _range.y() / (float)_map_y;
    
    float h=0;
    //cout<<"dX="<<dX<<endl;    cout<<"dY="<<dY<<endl;

    float tmp; 
    while (x<x2 && y<y2){
      //tot += getHeightAbs(x,y);
      tmp = getHeightAbs(x,y);
      if (tmp>h)
	h=tmp;
      x += dX;
      y += dY;
      //cpt++;
    }
    
    return h;
    //return tot/cpt;
}//@CODE_2204


float HeightMap::getHeightZone(float x, float y, float r)
{//@CODE_2184
//std::cout<<" getHeightMoy(x-r,y-r,x+r,y+r)="<<getHeightMoy(x-r,y-r,x+r,y+r)<<endl;    
    //std::cout<<"getHeightAbs(x,y)="<<getHeightAbs(x,y)<<endl;
    //return getHeightAbs(x,y);
    return getHeightMoy(x-r,y-r,x+r,y+r);
  
}//@CODE_2184


osg::Vec3 HeightMap::getOrig()
{//@CODE_2202
    return _visual_box._min;
}//@CODE_2202


bool HeightMap::loadRawFile(const char * strName)
{//@CODE_2216
  FILE *pFile = NULL;
  
  // Let's open the file in Read/Binary mode.
  pFile = fopen( strName, "rb" );
  // Check to see if we found the file and could open it
  if ( pFile == NULL ) {
    CWARN(this,"loadRawFile()")<<" Could not open File " <<strName<< endl;
    if (generer()){
      saveAs(strName);
      pFile = fopen( strName, "rb" );
      if ( pFile == NULL){
	CERR(this,"loadRawFile")<<"loadRawFile failed (1)"<<endl;
	return false;
      }
    }else {
      CERR(this,"loadRawFile")<<"loadRawFile failed (2)"<<endl;
      return false;
    }
  }
  
  //read size
  unsigned int xy[2];
  fread(xy, 2, sizeof(unsigned int), pFile ); 
  setImposedSize(xy[0],xy[1]);
  
  //read bounding box
  fread(&_box, 1, sizeof(osg::BoundingBox), pFile ); 
  
  setOrigin(_box._min);
  _range.set(_box.xMax()-_box.xMin(),
	    _box.yMax()-_box.yMin(),
	    _box.zMax()-_box.zMin());
  
  //read visual bounding box
  fread(&_visual_box, 1, sizeof(osg::BoundingBox), pFile ); 
  
  
  // Here we load the .raw file into our pHeightMap data array.
  // We are only isPlay in '1', and the size is the (width * height)
  fread(&_heights[0] , 1,sizeof(float)*_map_x*_map_y, pFile );
  
  // Check if we received an error.
  if (ferror( pFile )) {
    //cerr << "Cannot get Data" << endl;
    CERR(this,"loadRawFile")<<"loadRawFile failed (3)"<<endl;
    return false;
  }
  // Close the file.
  fclose(pFile);
  CTRACE(this,"loadRawFile")<<strName<<" loaded with "
			    <<xy[0]<<"*"<<xy[1]<<" size"<<endl;
  return true;
}//@CODE_2216


void HeightMap::saveAs(const char * rawFile)
{//@CODE_2214
  FILE * filePtr = fopen(rawFile,"wb");
  if (!filePtr) { 
    cerr<<"Impossible de crée le fichier "<<rawFile<<endl; 
    filePtr = fopen("/tmp/tmp.raw","wb");
    cerr<<" => write in /tmp/tmp.raw"<<endl;
  }
  fwrite(&_map_x,1,sizeof(unsigned int),filePtr);
  fwrite(&_map_y,1,sizeof(unsigned int),filePtr);
  fwrite(&_box,1,sizeof(osg::BoundingBox),filePtr);
  fwrite(&_visual_box,1,sizeof(osg::BoundingBox),filePtr);
  fwrite(&_heights[0],1,sizeof(float)*_map_x*_map_y,filePtr);
  fclose(filePtr);
  CTRACE(this,"saveAs")<<" HeightMap saved in "<<rawFile<<" de taille "<<sizeof(unsigned int)*2+sizeof(float)*_map_x*_map_y+sizeof(osg::BoundingBox)<<endl;


}//@CODE_2214


bool HeightMap::setImposedSize(GLuint x, GLuint y)
{//@CODE_2209
     if (x==0 || y==0){
    cerr<<"taille incorrecte"<<endl;
    return false;
  }
  _tail_impo = true;
  allocate/*Grid*/(x,y);
  return true;
}//@CODE_2209


//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_2164
Method which must be called first in a constructor.
*/
void HeightMap::ConstructorInclude()
{
}


/*@NOTE_2165
Method which must be called first in a destructor.
*/
void HeightMap::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
} //end of namespace cooki3d
