/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          HeightMap.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'HeightMap'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _HEIGHTMAP_H
#define _HEIGHTMAP_H

//@START_USER1


#include <osgUtil/Optimizer>
#include <osg/Node>
#include <osg/Geode>
#include <fstream>
#include <iostream>

#include "XMLUtil.h"

#ifdef _HEIGHTMAP_H_INLINES
#error HEIGHTMAP_H_INLINES
#endif

namespace cooki3d{


  class HeightMapAbstract  {
    
  public:
    class height_out{};
    
    HeightMapAbstract() {}
    
    //can throw height_out exception
    virtual float getHeightAbs(float x, float y) = 0;
       
    virtual float getHeightZone(float x, float y,float d) = 0;
    
    virtual float getDeltaCase() = 0;

    virtual bool contains(float x, float y) = 0;
    virtual float getHeightAbsNoException(float x, float y) = 0;
    
    virtual ~HeightMapAbstract(){}
    
  };

//@END_USER1


/*@NOTE_2162
class ...
*/

class AFX_EXT_CLASS HeightMap
    : public HeightMapAbstract
    , public osg::Grid
{

//@START_USER2
public :
//@END_USER2

//
// Group: ClassBuilder methods
//

private:
    void ConstructorInclude();
    void DestructorInclude();

//
// Non-Grouped Members
//

protected:
    bool _tail_impo;
    GLuint& _map_x;
    GLuint& _map_y;
    osg::BoundingBox _box;
    osg::BoundingBox _visual_box;
    osg::Vec3 _range;
    osg::Node * _terrain;
    EnvironnementOSG* _env;

//
// Non-Grouped Methods
//

public:
    HeightMap(osg::Node * terrain, EnvironnementOSG* env);
    virtual ~HeightMap();
    virtual bool contains(float x, float y);
    virtual bool generer();
    virtual float getAllHeightsBetween(float x1, float y1, float x2, float y2);
    virtual osg::Vec3 getCentre();
    virtual float getDeltaCase();
    virtual osg::Node* getGeode();
    virtual float getHeightAbs(float x, float y);
    virtual float getHeightAbsNoException(float x, float y);
    virtual float getHeightMoy(float x1, float y1, float x2, float y2);
    virtual float getHeightZone(float x, float y, float r);
    virtual osg::Vec3 getOrig();
    virtual bool loadRawFile(const char * strName);
    virtual void saveAs(const char * rawFile);
    virtual bool setImposedSize(GLuint x, GLuint y);
    EnvironnementOSG* getEnv() const;
};

#endif


#ifdef CB_INLINES
#ifndef _HEIGHTMAP_H_INLINES
#define _HEIGHTMAP_H_INLINES

/*@NOTE_4842
Returns the value of member '_env'.
*/
inline EnvironnementOSG* HeightMap::getEnv() const
{//@CODE_4842
    return _env;
}//@CODE_4842



//@START_USER3

class HeightMapDynamic : public HeightMap {


public:
  
  HeightMapDynamic(osg::Node *terrain,EnvironnementOSG *env) : 
    HeightMap(terrain, env){}

  virtual bool contains(float x, float y){
    return true;
  }

  virtual float getHeightAbsNoException(float x, float y){
    return getHeightAbs( x, y);
  }
  
  virtual float getHeightAbs(float x, float y){
    
    osgUtil::IntersectVisitor::HitList hits;
    _env->computeIntersectionsVertical(x,y,hits,_terrain);

    if (hits.size()>0) {
      //cout<<"hits[0].getWorldIntersectPoint()[2]="<<hits[0].getWorldIntersectPoint()[2]<<endl;
      return hits[0].getWorldIntersectPoint()[2];

    }
    cout<<"HeightMapDynamic echec"<<endl;
    
    //cout<<"HeightMapDynamic->getHeightAbs("<<x<<","<<y<<") "<<_terrain-><<endl;    
    return 0.1;

  } 

};



} //end of namespace cooki3d
//@END_USER3

#endif
#endif
