/** -*-c++-*- *************************************************************\
* File:          IntersectionRectangleUtil.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'SpecialCullVisitor'
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _INTERSECTION_RECTANGLE_UTIL_H_
#define _INTERSECTION_RECTANGLE_UTIL_H_

#include "ScanSetGraph.h"

namespace cooki3d {

  inline osgUtil::CullVisitor::value_type distance(const osg::Vec3& coord,
						   const osg::Matrix& matrix)
  {
    
    //std::cout << "distance("<<coord<<", "<<matrix<<")"<<std::endl;
    
    return -((osgUtil::CullVisitor::value_type)coord[0]*(osgUtil::CullVisitor::value_type)matrix(0,2)+(osgUtil::CullVisitor::value_type)coord[1]*(osgUtil::CullVisitor::value_type)matrix(1,2)+(osgUtil::CullVisitor::value_type)coord[2]*(osgUtil::CullVisitor::value_type)matrix(2,2)+matrix(3,2));
  }
  
  //modification if osgUtil/CullVisitor.cpp
  class SpecialCullVisitor : public osgUtil::CullVisitor {
    
  public: 
    
    SpecialCullVisitor(){}
    
    bool isCulledSpecial(const osg::BoundingBox & bb){
      
      int count = 8;
      for (int i = 0; i<8; i++){
	
	std::vector<osg::Vec3> uniqList;
	uniqList.push_back( bb.corner( i ) );
	if ( isCulled ( uniqList ) ){
	  count--;
	  if (count < 3){
	    // cout<<"elimination de ..."<<endl;
	    return true;
	  }
	}
	
      }
      
      return false;    

    }
    
    virtual void apply(osg::Geode& node)
    {
      if (isCulled(node)) return;
      
      // push the node's state.
      osg::StateSet* node_state = node.getStateSet();
      if (node_state) pushStateSet(node_state);
      
      // traverse any call callbacks and traverse any children.
      handle_cull_callbacks_and_traverse(node);

      osg::RefMatrix& matrix = getModelViewMatrix();
      for(unsigned int i=0;i<node.getNumDrawables();++i)
	{
	  osg::Drawable* drawable = node.getDrawable(i);
	  const osg::BoundingBox &bb =drawable->getBound();

	  if( drawable->getCullCallback() )
	    {
#ifdef OSG_CVS
	      if( drawable->getCullCallback()
		  ->cull( this, drawable, &_renderInfo ) == true ) {
		continue;
	      }
#else
	     if( drawable->getCullCallback()
		 ->cull( this, 
			 drawable, 
			 _state.valid()?_state.get():NULL ) == true ){
		continue;
	      }
#endif
		
	    }
        
	  //else
	  {
            if (node.isCullingActive() && isCulled(bb)) continue;
	  }

	  //ici on affine la recherche, on demande que au moins trois points
	  //de la BoundingBox soit dans la selection rectangle
	  {
            if (node.isCullingActive() && isCulledSpecial(bb)) continue;
	  }


	  if (_computeNearFar && bb.valid()) 
	    {
	      if (!updateCalculatedNearFar(matrix,*drawable,false)) continue;
	    }

	  // need to track how push/pops there are, so we can unravel the stack correctly.
	  unsigned int numPopStateSetRequired = 0;

	  // push the geoset's state on the geostate stack.    
	  osg::StateSet* stateset = drawable->getStateSet();
	  if (stateset)
	    {
	      ++numPopStateSetRequired;
	      pushStateSet(stateset);
	    }

	  osg::CullingSet& cs = getCurrentCullingSet();
	  if (!cs.getStateFrustumList().empty())
	    {
	      osg::CullingSet::StateFrustumList& sfl = cs.getStateFrustumList();
	      for(osg::CullingSet::StateFrustumList::iterator itr = sfl.begin();
		  itr != sfl.end();
		  ++itr)
		{
		  if (itr->second.contains(bb))
		    {
		      ++numPopStateSetRequired;
		      pushStateSet(itr->first.get());
		    }
		}
	    }

	  if (bb.valid()) 
	    addDrawableAndDepth(drawable,&matrix,
				distance(bb.center(),matrix));

	  else addDrawableAndDepth(drawable,&matrix,0.0f);

	  cout<<"addDrawableAndDepth.."<<node.isCullingActive()<<".."
	      <<(node.isCullingActive() && isCulled(bb))<<".."<<bb.valid()<<endl;

	  for(unsigned int i=0;i< numPopStateSetRequired; ++i)
	    {
	      popStateSet();
	    }

	}

      // pop the node's state off the geostate stack.    
      if (node_state) popStateSet();
    
    }
  

  };

  void getHits(osgUtil::StateGraph * sg,
	       std::vector<SimpleHit> & hitlist){
  
  
    //cout<<"sg->_leaves.size()="<<sg->_leaves.size()<<endl;
    //cout<<"sg->_children.size()="<<sg->_children.size()<<endl;
  
  
    for (osgUtil::StateGraph::LeafList::iterator it =
	   sg->_leaves.begin();
	 it !=  sg->_leaves.end();
	 it++){

      SimpleHit sh;
    
      sh._drawable = (*it)->_drawable;
      sh._geode = (osg::Geode*) (*it)->_drawable->getParent(0);
      hitlist.push_back( sh);

      cout<<ScanSetGraph::getFirstNamePath( sh._geode )<<endl;;
      cout<<"[ ("<<(*it)->_depth<<") "<<sh._geode->getName()<<"]"<<endl;
    
    
    }

    for (osgUtil::StateGraph::ChildList::iterator it = 
	   sg->_children.begin();
	 it !=  sg->_children.end();
	 it++){
      getHits( it->second.get(),hitlist );
    }
  }
}


#endif
