# -*-makefile-*-
 
CPP=g++

OPTIM=-O2
CC_RELEASE=$(CPP) $(OPTIM) -Wall -pipe
CC_DEBUG=$(CPP) $(OPTIM) -Wall -pg -pipe

#Ce parametre d'entree REP_SYS sera écrasé
#REP_SYS= linux, mingw ...
CIBLE=Metier
LIB_NAME_BASE=Cooki3d

ifeq ($(DEBUG),1)
 CC:=$(CC_DEBUG)
 DBG_SUF:=d
 REP_OBJ:=$(REP_SYS)/debug
else 
  ifeq ($(DEBUG),2)
    CC:=$(CC_RELEASE)
    DBG_SUF:=cvs
    REP_OBJ:=$(REP_SYS)cvs/release
    CPPFLAGS+=-DOSG_CVS
  else
    CC:=$(CC_RELEASE)
    REP_OBJ:=$(REP_SYS)/release
  endif
endif


ifeq ($(REP_SYS),linux)

 EXE_FILE:=$(CIBLE).bin
 EXE:=$(REP_OBJ)/$(EXE_FILE)
 LIB_NAME:=$(LIB_NAME_BASE)
 LIB_NAME_TGT:=$(REP_OBJ)/lib$(LIB_NAME).so


 #QT4 
 QGL_SUPPORT:=-DQGL_SUPPORT

 ifneq (QGL_SUPPORT,)
  CPPFLAGS_QT:= -I/usr/include/qt4 -I/usr/include/qt4/Qt
  LIBS_QT:= -lQtCore -lQtGui -lQtOpenGL
 endif

 CPPFLAGS+= -DUNIX -I. -I/usr/X11R6/include -I../includeCB -I/usr/include/SDL -I/usr/include -I/usr/include/libxml++-1.0 $(CPPFLAGS_QT)
 CPPFLAGS+= -DXML_SUPPORT -DSMPEG_SUPPORT $(QGL_SUPPORT) 

 LIBS_UNIX = -L/usr/X11R6/lib/ -lm -lOpenThreads   -L./java 
 LIBS_UNIX +=`sdl-config --libs` -ldl -lsmpeg $(LIBS_QT) #UNIX
 LIBS +=$(LIBS_UNIX) -losgProducer -lProducer -losgText -losgGA -losgDB -losgUtil -losgFX -losg -losgSim -losgParticle  #(osgedit) 
 LIBS += -lSDL -lxml++-1.0 


else 
ifeq ($(REP_SYS),mingw)

 EXE_FILE:=$(CIBLE).exe
 EXE:=$(REP_OBJ)/$(EXE_FILE)
 LIB_NAME:=$(LIB_NAME_BASE)
 LIB_NAME_TGT:=$(REP_OBJ)/lib$(LIB_NAME).dll

 LIBS += -lOpenThreads -lopengl32 -lglu32
 LIBS += -lProducer -losgProducer -losgText 
 LIBS += -losgGA -losgDB -losgUtil -losgFX -losg
 LIBS += -lSDL -lxml++-1.0 -lsmpeg


 #QT4 :-DQGL_SUPPORT
 QGL_SUPPORT=

 ifneq ($(QGL_SUPPORT),)
  LIBS += -lQtCore$(DBG_SUF) -lQtGui$(DBG_SUF) 
  LIBS += -lqtmain$(DBG_SUF) -lQtOpenGL$(DBG_SUF) 
  #LIB_NAME=$(LIB_NAME_BASE)$(DBG_SUF)Qt4
 else
  #LIB_NAME=$(LIB_NAME_BASE)$(DBG_SUF)
 endif

  CPPFLAGS+=$(QGL_SUPPORT) -DXML_SUPPORT -DSMPEG_SUPPORT 

endif
endif


SOURCES:=$(wildcard *.cpp) 

OBJETS:=$(addprefix $(REP_OBJ)/,$(addsuffix .o,$(basename $(SOURCES))))

#fichiers de dépendance autogen
DEPS:=$(addprefix $(REP_OBJ)/,$(addsuffix .d,$(basename $(SOURCES))))


all: $(EXE)


$(REP_OBJ)/%.d: %.cpp
	@echo $@ 
	@$(CC) -o $@.tmp -MM $< $(CPPFLAGS) 
	@echo -n "$(REP_OBJ)/"> $@
	@cat $@.tmp >> $@
	@rm $@.tmp


$(REP_OBJ)/%.o: %.cpp
	@echo $@
	@$(CC) -c $<  $(CPPFLAGS) -o $@ 



$(EXE): $(DEPS) $(OBJETS) 

#######	le moteur C++
	@echo "LD_LIBRARY_PATH="$(LD_LIBRARY_PATH)	
	@echo "LIBRARY_PATH="$(LIBRARY_PATH)	
	@echo "make $(LIB_NAME_TGT)"

ifeq ($(REP_SYS),linux)

	$(CC) -o $(LIB_NAME_TGT) -shared \
		  $(filter-out BuilderDechet yacc $(DEPS),$^) $(LIBS)
else 
ifeq ($(REP_SYS),mingw)

	$(CC) -o $(LIB_NAME_TGT) -shared \
		-Wl,--export-all-symbols \
		-Wl,--output-def,$(REP_OBJ)/lib$(LIB_NAME).def \
		-Wl,--out-implib,$(LIB_NAME_TGT).a \
		$(filter-out BuilderDechet yacc $(DEPS),$^) $(LIBS) 
endif
endif

####### l'executable C++		
	@echo make $(EXE)
	@$(CC) -o $@ -L./$(REP_OBJ)/ -l$(LIB_NAME)

ifdef COOKI3D_HOT_DEV

####### les modules java
	@make -C java DEBUG=$(DEBUG) REP_SYS=$(REP_SYS) \
	              QGL_SUPPORT=$(QGL_SUPPORT)

####### uniquement pour le dévpt à chaud
	@mkdir -p $(HOME)/.cooki3d/api/libCooki3d

 ifeq ($(REP_OBJ),linux/release)
	cp $(LIB_NAME_TGT) $(HOME)/.cooki3d/api/libCooki3d
	cp $(EXE) $(HOME)/.cooki3d/api/libCooki3d
 else
 ifeq ($(REP_OBJ),mingw/release)
	cp $(LIB_NAME_TGT) $(HOME)/.cooki3d/api/libCooki3d
	cp $(LIB_NAME_TGT).a $(HOME)/.cooki3d/api/libCooki3d
	cp $(EXE) $(HOME)/.cooki3d/api/libCooki3d
 endif
 endif

####### maj des ressources pour les packages du framework
        #=>les libraries
	cp $(LIB_NAME_TGT) ../java/ressources/run/$(REP_OBJ)/libCooki3d
	cp $(EXE) ../java/ressources/run/$(REP_OBJ)/libCooki3d/$(EXE_FILE)
 ifeq ($(REP_SYS),mingw)
	cp $(LIB_NAME_TGT).a ../java/ressources/dev/$(REP_OBJ)/libCooki3d
 endif

        #=>l'api cooki3d
	ln -sf $(PWD)/*.h ../java/ressources/dev/$(REP_OBJ)/includeCooki3d

	@touch CookiMetier.h #force to be keep in the jar...

        #=>construction des packages
        #(build of packages)
 ifeq ($(REP_SYS),linux)
	ant -DsysDir=$(REP_OBJ) -DSysName=Linux -DCateg=Cooki3d -buildfile ../java/build.xml ressourcesLinuxCooki3d
 else
 ifeq ($(REP_SYS),mingw)
	ant -Dsys=mingw -DSys=Mingw -DCateg=Cooki3d -buildfile ../java/build.xml ressourcesMingwCooki3d
 endif
 endif

endif

#ifneq ($(QGL_SUPPORT),)
# le plugin par défaut (mode dépannage)
#	@echo make PluginExample-FFD-Qt
#	@make -C PluginExample-FFD-Qt
#else
#	@echo "no support QGL example MoteurJeuDefaut"
#endif

#	TODO à revoir
#	déplacement des fichiers sauvegardes emacs et autres vers bak...
#	@echo "déplacement des fichiers sauvegardes emacs "
#	@echo "et autres vers le répertoire bak/date..."
#	@echo "`date`" >bak/LASTBACKUP
#	@mkdir "bak/`cat bak/LASTBACKUP`" 
#	@echo -n "" || mv *.*~ *.*.bak \#* *.old "bak/`cat bak/LASTBACKUP`" &>/dev/null #pour éviter les erreurs type *.*~ n'existe pas et une sortie violente du makefile...


dist:
	make $(EXE) -j2 CC="$(CC_DIST)"

dist_debug:
	make $(EXE) -j2 CC="$(CC_DIST_DEBUG)"

debug:
	make -f Makefile.linux $(EXE) CC="$(CC_DEBUG)"

release:
	make -f Makefile.linux $(EXE) CC="$(CC_RELEASE)"


-include $(DEPS)


cleand:
	@rm -f $(DEPS)
ifdef COOKI3D_HOT_DEV
	make cleand -C MoteurJeuDefaut
endif

distclean: clean


clean:
	rm -f $(REP_OBJ)/* core a.out $(EXE)
ifdef COOKI3D_HOT_DEV
	make clean -C MoteurJeuDefaut
endif
	rm -f $(LIB_NAME_TGT)


# AIDE 
# $@ la cible exemple.d
# $< le premier prérequis
# $? Le nom de tous les pré-requis plus récent que la cible
# $* le basename de la cible