/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          MoteurJeuC.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'MoteurJeuC'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include "MoteurJeuC.h"
#include "Function_0arg.h"
#include "Function_1arg.h"

#include <sys/fcntl.h>

#ifdef UNIX
#    include <dlfcn.h>
#else
#    include <windows.h>
#endif

#include "LoaderXML.h"


void * MoteurJeuC::loadFunction(char * funcName){


  CTRACE(this,"load")<<" loadFunction "<<funcName<<endl;
  
  
#ifdef UNIX  
  void * getFunc = dlsym(_libsys, funcName);
  if (!getFunc) {
    CERR(this,"load")<<"lib="<<_lib<<" Cannot load symbol "<<funcName 
		     <<" ("<< dlerror() <<")\n"<<endl;
    if (_libsys) {
      dlclose(_libsys);
      _libsys = NULL;
    }
  } 
#else
  // Get the adr of the function to call
  
  void * getFunc = (void*)GetProcAddress((HMODULE)_libsys, funcName);
  if (!getFunc) {
    //pour mingw si pas de __stdcall + lib.def file (EXPORTS ...)
    //on résout le probleme en essayant d'appeler directement l'alias
    
    CERR(this,"load")<<"lib="<<_lib<<" Cannot load symbol ("<<funcName 
		     <<endl;

    getFunc = (void*)GetProcAddress((HMODULE)_libsys, 
				    ("_"+std::string(funcName)).c_str());
    
    if (!getFunc) {
      char MsgBuf[100];
      FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM,
		     NULL,
		     GetLastError(),
		     // Default language
		     MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
		     MsgBuf,
		     99,
		     NULL
		     );
      
      CERR(this,"load")<<"lib="<<_lib<<" Cannot load symbol ("
		       <<("_"+std::string(funcName)).c_str()
		       <<" ("<< MsgBuf <<")"<<endl;
      
      
      if (_libsys) {
	FreeLibrary((HMODULE)_libsys);
	_libsys = NULL;
      }
    }

  }   
#endif

  return getFunc;

}

bool MoteurJeuC::load(){
  
  bool resultOk = true;

  //////////////////
  // load library //
  //////////////////
#ifdef UNIX
  // load the .so
  if (!((_libsys = dlopen( getLib().c_str(), RTLD_NOW | RTLD_GLOBAL)))) {
    CERR(this,"loadXML")<<" lib="<<getLib()<<" erreur dlopen :"
			<<dlerror()<<endl;
    resultOk = false;
  }
#else
  //_libsys = LoadLibrary( (const WCHAR*) getLib().c_str());
  _libsys = LoadLibrary(  getLib().c_str());
  
  // load the .dll
  if ( !_libsys) {
    char MsgBuf[100];
    FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM,
		   NULL,
		   GetLastError(),
		   // Default language
		   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
		   MsgBuf,
		   99,
		   NULL
		   );

    CERR(this,"loadXML")<<" lib="<<getLib()<<" erreur open dll :"
			<<MsgBuf<<endl;
    resultOk = false;
  }
#endif

  // si le load a échoué, on sort maintenant
  if (!resultOk){
    return false;
  }
  
  ////////////////
  //Test version//
  ////////////////
  CTRACE(this,"load")<<" test version "<<endl;
  typedef int (*fonc)();
  
  fonc getVersion = (fonc) loadFunction("getVersion");
  resultOk = getVersion != NULL;

  if (resultOk){
    resultOk = ( getVersion() == COOKIMETIER_VERSION );
    if (!resultOk){
      CERR(this,"load")<<"COOKIMETIER_VERSION("
		       <<COOKIMETIER_VERSION<<") is different from "
		       <<"plugin version("<<getVersion()<<")"<<endl;
    }
  }
  
  if (resultOk){
    CTRACE(this,"load")<<getLib()<<" OK"<<endl;
  }
  return resultOk;
}


namespace cooki3d{
//@END_USER2




/*@NOTE_935
Constructor method.
*/
MoteurJeuC::MoteurJeuC(BD* pBD) //@INIT_935
    : Evaluable(pBD)
    , _lib()
    , _libsys(NULL)
    , _rep()
{//@CODE_935
    ConstructorInclude();

    // Put in your own code
}//@CODE_935


/*@NOTE_931
Destructor method.
*/
MoteurJeuC::~MoteurJeuC()
{//@CODE_931
  
  if (_libsys){

    cleanUserData(getBD());

#ifdef UNIX	      
    dlclose(_libsys);    
#else
    // Unload the DLL and free memory
    FreeLibrary( (HMODULE)_libsys);	
#endif
    _libsys = NULL;
  }else {
    CERR(this,"~MoteurJeuC")<<" _libsys was NULL!!"<<endl;
  }

  CTRACE(this,"~MoteurJeuC")<<" Destruction of "<<_lib<<endl;	    

  DestructorInclude();
  
}//@CODE_931


void MoteurJeuC::cleanUserData(BD* pBD)
{//@CODE_4771

  CTRACE(this,"cleanUserData")<<"entrée"<<endl;
  typedef void (*fonc)(BD*);
  assertC( _libsys );
  assertC2( pBD, NULL);
  
  fonc clean = (fonc) loadFunction("cleanUserData");
  if (clean){
    clean(pBD);
  }

}//@CODE_4771


bool MoteurJeuC::loadXML(const xmlpp::Node* node, LoaderXML* loaderXML)
{//@CODE_1137

  XMLUtil xml(node);
  
  //attention à l'ordre d'évaluation dans une chaine s1+s2+...
  //la variable rep est un passage obligé !!! 
  
  setName(xml.getFirstText("nom"));
  
  setRep(xml.getFirstText("rep"));    

#ifdef UNIX
  //répertoire abs + répertoire relatif + nom librairie
  setLib( loaderXML->getPath( _rep+xml.getFirstText("lib") )); 
#else
  setLib( loaderXML->getPath( _rep+xml.getFirstText("libMingw") )); 
#endif

  return load();
}//@CODE_1137


void MoteurJeuC::postInitInstances(LoaderGeneric* loader)
{//@CODE_1140
  CTRACE(this,"postInitInstances")<<"entrée"<<endl;
  typedef void (*fonc)(LoaderGeneric*);
  assertC( _libsys );

  fonc post = (fonc) loadFunction("postInitInstances");
  if (post){
    post(loader);
  }

}//@CODE_1140


void MoteurJeuC::preInitInstances(LoaderGeneric* loader)
{//@CODE_1141
  CTRACE(this,"preInitInstances")<<"entrée"<<endl;
  typedef void (*fonc)(LoaderGeneric*); 
  assertC( _libsys );

  fonc pre = (fonc) loadFunction("preInitInstances");
  if (pre){
    pre(loader);
  }

}//@CODE_1141


Evaluable* MoteurJeuC::staticCreate(BD* pBD)
{//@CODE_3041
    return new MoteurJeuC(pBD);
}//@CODE_3041


//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_932
Method which must be called first in a constructor.
*/
void MoteurJeuC::ConstructorInclude()
{
}


/*@NOTE_933
Method which must be called first in a destructor.
*/
void MoteurJeuC::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
} //end of namespace cooki3d

//poubelle
/*  //if (!((MJClib = dlopen("/home/bertrand/CookiC++/Metier/MoteurJeu/plugin.so", RTLD_LAZY)))) 
  if (!((MJClib = dlopen("/home/bertrand/CookiC++/Metier/MoteurJeu/plugin.so", RTLD_NOW)))) 
    
    CWARN_MESS<<"MoteurJeuC::preInitInstances"<<" code erreur 3, dlopen ('/home/bertrand/CookiC++/Metier/MoteurJeu/plugin.so') ne peut pas ouvir plugin.so "<<dlerror()<<endl;
  
  // retrieve symbol pointer 
  //cout << "Loading symbol hello...\n";
  
  typedef void (*fonc)();
  fonc pre = (fonc) dlsym(MJClib, "preInitInstancesAllLibs");
  if (!pre) {
    CWARN_MESS<<"MoteurJeuC::preInitInstances Cannot load symbol 'hello': " << dlerror() <<'\n';
    dlclose(MJClib);
  }else {
    pre();
  }
    
  //dlclose(MJClib);*/
