/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          OsgNodeReference.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'OsgNodeReference'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include <osg/PositionAttitudeTransform>
#include <osg/MatrixTransform>
#include <osg/AutoTransform>

#include "Evaluable.h"
#include "OsgNodeReference.h"
#include "XMLUtil.h"
#include "LoaderGeneric.h"

namespace cooki3d{
//@END_USER2




/*@NOTE_1911
Constructor method.
*/
OsgNodeReference::OsgNodeReference(BD* pBD) //@INIT_1911
    : Evaluable(pBD)
    , _osgNode(NULL)
    , _insertTransform(false)
    , _nodeName()
    , _transform(NULL)
    , _transformType("")
{//@CODE_1911
    ConstructorInclude();

    // Put in your own code
}//@CODE_1911


/*@NOTE_1898
Destructor method.
*/
OsgNodeReference::~OsgNodeReference()
{//@CODE_1898
    DestructorInclude();

    // Put in your own code
}//@CODE_1898


/*@NOTE_4330
Check the integritry of the object
*/
bool OsgNodeReference::check()
{//@CODE_4330
    bool resultOk = _osgNode != NULL && 
            ( _transformType == "" || _transform != NULL );

    if (!resultOk){
      CERR(this,"check")<<" \""<<getName()<<"\" : \""<<getNodeName()
			<<"\" is not linked !!"<<endl;
    }
    
    return resultOk;
}//@CODE_4330


OsgNodeReference* OsgNodeReference::getOsgNodeReference(BD* pBD,
                                                        std::string nodeName)
{//@CODE_1912
  BD::EvaluableIterator it( pBD );
  OsgNodeReference * onr;

  while (++it)
  {
    onr=dynamic_cast<OsgNodeReference *>(it.Get());

    if (onr && (onr->getNodeName() == nodeName))
    {
      return onr;
    }
  }
  return NULL;
}//@CODE_1912


bool OsgNodeReference::linkOsgNodeReference(osg::Node* n, osg::Node* parent,
                                            const std::string& pickName,
                                            LoaderGeneric* loader)
{//@CODE_1915

  BD::EvaluableIterator it(loader->getBD());
  //std::string pickName = n->getName();
  OsgNodeReference * onr;
  bool res=false;

  while (++it)
  {
    onr=dynamic_cast<OsgNodeReference *>(it.Get());

    if (onr && (onr->getNodeName() == pickName))
    {
      if ( ! onr->getOsgNode())
      {
          onr->setOsgNode(n);
	 	  
	  CTRACEs("OsgNodeReference::"
		  "linkOsgNodeReference")<<" sur "<<pickName
					 <<" typeid du noeud liée="
					 <<typeid(*n).name()<<endl;
	  
	  CINFOs<<"OsgNodeReference link \""<<onr->getNodeName()
		<<"\"("<<onr->getName()<<") "<<endl;


	  if ( onr->isInsertTransform() ){
	     
	    if ( onr->getTransformType() == "osg::PositionAttitudeTransform"){
	      onr->setTransform( new osg::PositionAttitudeTransform() );
	    }else if (onr->getTransformType() == "osg::MatrixTransform"){
	      onr->setTransform( new osg::MatrixTransform() );
	    }else if (onr->getTransformType() == "osg::AutoTransform"){
	      onr->setTransform( new osg::AutoTransform() );
	    }else {
	      CERRs("OsgNodeReference::linkOsgNodeReference")
		<<" unknow transformType ("
		<<onr->getTransformType()<<")"<<endl;
	      return false;
	    }
	    
	    assertC2( parent , loader->getBD());
	    assertC2( parent->asGroup() , loader->getBD());


	    onr->getTransform()->setName( "osgNodeTeapot" );

	    //faire le add avant le remove car les objets sont référencés!
	    onr->getTransform()->addChild( n );

	    parent->asGroup()
	      ->insertChild( parent->asGroup()->getChildIndex( n ),
			  onr->getTransform() );
	    
	    //TODO Revoir ce point, pourquoi le core enregistré est altéré
	    /*CTRACEs("OsgNodeReference::linkOsgNodeReference")
	      <<"TODO Revoir ce point, pourquoi le fichier "
	      <<"core enregistré est altéré ? "
	      <<"les controles points sont perdues" 
	      <<endl;*/
	    //=> ne n'est pas un bug, c'est le _pathInit mis par une animation
	    parent->asGroup()->removeChild( n );

	    CINFOs<<"OsgNodeReference alter osg tree,"
		  <<" insert \"osgNodeTeapot\""<<endl;
	    
	  }
	
          res=true;
      }
    }
  }
  return res;

}//@CODE_1915


bool OsgNodeReference::loadXML(const xmlpp::Node* node, LoaderXML* loaderXML)
{//@CODE_1917
  CTRACE(this,"loadOsgNodeReference")<<"&&&&&&&&&&&&&&&& entrée"<<endl;


  XMLUtil xml(node);
  
  setName(xml.getFirstText("nom"));
  setNodeName(xml.getFirstText("nodeName"));  
  setInsertTransform ( xml.getFirstText("insertTransform") == "true" );
  if ( isInsertTransform() ){
    setTransformType(  xml.getFirstText("transformType") );
  }
  //OsgReference::loadXML( node, loaderXML );


  return true;
}//@CODE_1917


Evaluable* OsgNodeReference::staticCreate(BD* pBD)
{//@CODE_3045
    return new OsgNodeReference(pBD);
}//@CODE_3045


//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_1899
Method which must be called first in a constructor.
*/
void OsgNodeReference::ConstructorInclude()
{
}


/*@NOTE_1900
Method which must be called first in a destructor.
*/
void OsgNodeReference::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
} //end of namespace cooki3d
