/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          OsgReference.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'OsgReference<OsgType>'
*
* Modifications: @INSERT_MODIFICATIONS(* )
* November 14, 2006 00:37 bg
*     Update comment header
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _OSGREFERENCE_H
#define _OSGREFERENCE_H

//@START_USER1
#include "Evaluable.h"
#include "XMLUtil.h"

namespace cooki3d{
//@END_USER1


/*@NOTE_4464
class...
*/
template<class OsgType>
class OsgReference
    : public Evaluable
{

//@START_USER2
//@END_USER2

//
// Group: ClassBuilder methods
//

private:
    void ConstructorInclude();
    void DestructorInclude();

//
// Non-Grouped Members
//

private:
    osg::Node* _osgNode;

protected:
    std::string _nodeName;
    osg::Transform* _transform;
    bool _insertTransform;
    std::string _transformType;

//
// Non-Grouped Methods
//

public:
    OsgReference(BD* pBD);
    virtual ~OsgReference();
    virtual bool check();
    static bool linkOsgNodeReference(osg::Node* n, osg::Node* parent,
                                     const std::string& pickName,
                                     LoaderGeneric* loader);
    virtual bool loadXML(const xmlpp::Node* node, LoaderXML* loaderXML);
    bool isInsertTransform() const;
    void setInsertTransform(bool insertTransform);
    std::string getNodeName() const;
    void setNodeName(std::string nodeName);
    osg::Transform* getTransform() const;
    void setTransform(osg::Transform* pTransform);
    std::string getTransformType() const;
    void setTransformType(std::string transformType);
};

#endif


#ifdef CB_INLINES
#ifndef _OSGREFERENCE_H_INLINES
#define _OSGREFERENCE_H_INLINES


//
// Group: ClassBuilder methods
//
/*@NOTE_4466
Method which must be called first in a constructor.
*/
template<class OsgType>
void OsgReference<OsgType>::ConstructorInclude()
{
}



/*@NOTE_4467
Method which must be called first in a destructor.
*/
template<class OsgType>
void OsgReference<OsgType>::DestructorInclude()
{
}



// End of Group: ClassBuilder methods

/*@NOTE_4490
Constructor method.
*/
template<class OsgType>
OsgReference<OsgType>::OsgReference(BD* pBD) //@INIT_4490
    : Evaluable(pBD)
    , _nodeName("")
    , _transform(NULL)
    , _insertTransform(false)
    , _transformType("")
    , _osgNode()
{//@CODE_4490
    ConstructorInclude();

    // Put in your own code
}//@CODE_4490



/*@NOTE_4465
Destructor method.
*/
template<class OsgType>
OsgReference<OsgType>::~OsgReference()
{//@CODE_4465
    DestructorInclude();

    // Put in your own code
}//@CODE_4465



/*@NOTE_4491
Check the integritry of the object
*/
template<class OsgType>
bool OsgReference<OsgType>::check()
{//@CODE_4491
    bool resultOk = _osgNode != NULL && 
        ( _transformType == "" || _transform != NULL );

    if (!resultOk){
      CERR(this,"check")<<" \""<<getName()<<"\" : \""<<getNodeName()
			<<"\" is not linked !!"<<endl;
    }
    
    return resultOk;
}//@CODE_4491



template<class OsgType>
bool OsgReference<OsgType>::linkOsgNodeReference(osg::Node* n,
                                                 osg::Node* parent,
                                                 const std::string& pickName,
                                                 LoaderGeneric* loader)
{//@CODE_4633

  BD::EvaluableIterator it(BD::instance());
  //std::string pickName = n->getName();
  OsgNodeReference * onr;
  bool res=false;

  while (++it)
  {
    onr=dynamic_cast<OsgNodeReference *>(it.Get());

    if (onr && (onr->getNodeName() == pickName))
    {
      if ( ! onr->getOsgNode())
      {
          onr->setOsgNode(n);
	 	  
	  CTRACEs("OsgNodeReference::"
		  "linkOsgNodeReference")<<" sur "<<pickName
					 <<" typeid du noeud liÃ©e="
					 <<typeid(*n).name()<<endl;
	  
	  CINFOs<<"OsgNodeReference link \""<<onr->getNodeName()
		<<"\"("<<onr->getName()<<") "<<endl;


	  if ( onr->isInsertTransform() ){
	     
	    if ( onr->getTransformType() == "osg::PositionAttitudeTransform"){
	      onr->setTransform( new osg::PositionAttitudeTransform() );
	    }else if (onr->getTransformType() == "osg::MatrixTransform"){
	      onr->setTransform( new osg::MatrixTransform() );
	    }else if (onr->getTransformType() == "osg::AutoTransform"){
	      onr->setTransform( new osg::AutoTransform() );
	    }else {
	      CERRs("OsgNodeReference::linkOsgNodeReference")
		<<" unknow transformType ("
		<<onr->getTransformType()<<")"<<endl;
	      return false;
	    }
	    
	    assertC( parent );
	    assertC( parent->asGroup() );


	    onr->getTransform()->setName( "osgNodeTeapot" );

	    //faire le add avant le remove car les objets sont rÃ©fÃ©rencÃ©s!
	    onr->getTransform()->addChild( n );

	    parent->asGroup()
	      ->insertChild( parent->asGroup()->getChildIndex( n ),
			  onr->getTransform() );
	    
	    //TODO Revoir ce point, pourquoi le core enregistrÃ© est altÃ©rÃ©
	    /*CTRACEs("OsgNodeReference::linkOsgNodeReference")
	      <<"TODO Revoir ce point, pourquoi le fichier "
	      <<"core enregistrÃ© est altÃ©rÃ© ? "
	      <<"les controles points sont perdues" 
	      <<endl;*/
	    //=> ne n'est pas un bug, c'est le _pathInit mis par une animation
	    parent->asGroup()->removeChild( n );

	    CINFOs<<"OsgNodeReference alter osg tree,"
		  <<" insert \"osgNodeTeapot\""<<endl;
	    
	  }
	
          res=true;
      }
    }
  }
  return res;

}//@CODE_4633



template<class OsgType>
bool OsgReference<OsgType>::loadXML(const xmlpp::Node* node,
                                    LoaderXML* loaderXML)
{//@CODE_4498
  
  XMLUtil xml(node);
  
  setName(xml.getFirstText("nom"));
  setNodeName(xml.getFirstText("nodeName"));  
  setInsertTransform ( xml.getFirstText("insertTransform") == "true" );
  if ( isInsertTransform() ){
    setTransformType(  xml.getFirstText("transformType") );
  }


  return true;
}//@CODE_4498



/*@NOTE_4483
Returns the value of member '_insertTransform'.
*/
template<class OsgType>
inline bool OsgReference<OsgType>::isInsertTransform() const
{//@CODE_4483
    return _insertTransform;
}//@CODE_4483



/*@NOTE_4484
Set the value of member '_insertTransform' to 'insertTransform'.
*/
template<class OsgType>
inline void OsgReference<OsgType>::setInsertTransform(bool insertTransform)
{//@CODE_4484
    _insertTransform = insertTransform;
}//@CODE_4484



/*@NOTE_4471
Returns the value of member '_nodeName'.
*/
template<class OsgType>
inline std::string OsgReference<OsgType>::getNodeName() const
{//@CODE_4471
    return _nodeName;
}//@CODE_4471



/*@NOTE_4472
Set the value of member '_nodeName' to 'nodeName'.
*/
template<class OsgType>
inline void OsgReference<OsgType>::setNodeName(std::string nodeName)
{//@CODE_4472
    _nodeName = nodeName;
}//@CODE_4472



/*@NOTE_4479
Returns the value of member '_transform'.
*/
template<class OsgType>
inline osg::Transform* OsgReference<OsgType>::getTransform() const
{//@CODE_4479
    return _transform;
}//@CODE_4479



/*@NOTE_4480
Set the value of member '_transform' to 'pTransform'.
*/
template<class OsgType>
inline void OsgReference<OsgType>::setTransform(osg::Transform* pTransform)
{//@CODE_4480
    _transform = pTransform;
}//@CODE_4480



/*@NOTE_4487
Returns the value of member '_transformType'.
*/
template<class OsgType>
inline std::string OsgReference<OsgType>::getTransformType() const
{//@CODE_4487
    return _transformType;
}//@CODE_4487



/*@NOTE_4488
Set the value of member '_transformType' to 'transformType'.
*/
template<class OsgType>
inline void OsgReference<OsgType>::setTransformType(std::string transformType)
{//@CODE_4488
    _transformType = transformType;
}//@CODE_4488



//@START_USER3
} //end of namespace cooki3d
//@END_USER3

#endif
#endif
