//-*-c++-*-
//directly extracted and adpated from osg api...
#ifndef  __PickIntersectVisitor__
#define __PickIntersectVisitor__


#include <osg/Group>
#include <osgUtil/UpdateVisitor>
#include <osg/PositionAttitudeTransform> 
#include "CookiMessage.h"


//Picking intersection visitor.
class PickIntersectVisitor : public osgUtil::IntersectVisitor
{
 public:
  PickIntersectVisitor()
    { 
    }
  virtual ~PickIntersectVisitor() {}
  
  HitList& getIntersections(osg::Node *scene, osg::Vec3 nr, osg::Vec3 fr)
    { 
      // option for non-sceneView users: 
      //      you need to get the screen perp line and call getIntersections
      // if you are using Projection nodes you should also call setxy 
      //      to define the xp,yp positions for use with
      //      the ray transformed by Projection
      _lineSegment = new osg::LineSegment;
      _lineSegment->set(nr,fr); // make a line segment
      
      //std::cout<<"near "<<nr<<std::endl;
      //std::cout<<"far "<<fr<<std::endl;
      
      addLineSegment(_lineSegment.get());
      
      scene->accept(*this);
      return getHitList(_lineSegment.get());
    }
 private:
  osg::ref_ptr<osg::LineSegment> _lineSegment;
  friend class osgUtil::IntersectVisitor;
};







// PickVisitor traverses whole scene and checks below all Projection nodes
class PickVisitor : public osg::NodeVisitor
{
 public:
  PickVisitor()
    { 
      init();
    }
  
  void init(){
    xp=yp=0;    
    setTraversalMode(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN);
  }
  
  ~PickVisitor() {}
  
  void setTraversalMask(osg::Node::NodeMask traversalMask)
    {
      NodeVisitor::setTraversalMask(traversalMask);
      _piv.setTraversalMask(traversalMask);   
    }
  
  // Aug 2003 added to pass the nodemaskOverride to the PickIntersectVisitor
  // may be used make the visitor override the nodemask to visit 
  // invisible actions
  inline void setNodeMaskOverride(osg::Node::NodeMask mask) {
    _piv.setNodeMaskOverride(mask);
    _nodeMaskOverride = mask; 
  }
  
  
  virtual void apply(osg::Projection& pr)
    { // stack the intersect rays, transform to new projection, traverse
      // Assumes that the Projection is an absolute projection
      osg::Matrixd mt;
      mt.invert(pr.getMatrix());
      osg::Vec3 npt=osg::Vec3(xp,yp,-1.0f) * mt,
	            farpt=osg::Vec3(xp,yp,1.0f) * mt;
      
      // traversing the nodes children, using the projection direction
      for (unsigned int i=0; i<pr.getNumChildren(); i++) 
        {
	  osg::Node *nodech=pr.getChild(i);
	  osgUtil::IntersectVisitor::HitList &hli=
	    _piv.getIntersections(nodech,npt, farpt);

	  for(osgUtil::IntersectVisitor::HitList::iterator hitr=hli.begin();
	      hitr!=hli.end();
	      ++hitr)
            { 
	      // add the projection hits to the scene hits.
	      // This is why _lineSegment is retained as a member 
	      //of PickIntersectVisitor
	      _PIVsegHitList.push_back(*hitr);
            }
	  traverse(*nodech);
        }
    }
  
  osgUtil::IntersectVisitor::HitList& getHits(osg::Node *node, const osg::Vec3& near_point, const osg::Vec3& far_point)
    {
      CTRACEnext<<"_piv.getNodeMaskOverride()="<<_piv.getNodeMaskOverride()
	<<", _piv.getTraversalMask()="<<_piv.getTraversalMask()
	<<endl;
      
      // High level get intersection with sceneview 
      //using a ray from x,y on the screen
      // sets xp,yp as pixels scaled to a mapping of (-1,1, -1,1); 
      // needed for Projection from x,y pixels      
      // first get the standard hits in un-projected nodes
      _PIVsegHitList=_piv.getIntersections(node,near_point,far_point); // fill hitlist
      
      // then get hits in projection nodes
      traverse(*node); // check for projection nodes
      return _PIVsegHitList;
    }
  
  osgUtil::IntersectVisitor::HitList& getHits(osg::Node *node, const osg::Matrixd &projm, const float x, const float y)
    { 
      // utility for non=sceneview viewers
      // x,y are values returned by 
      osg::Matrixd inverseMVPW;
      inverseMVPW.invert(projm);
      osg::Vec3 near_point = osg::Vec3(x,y,-1.0f)*inverseMVPW;
      osg::Vec3 far_point = osg::Vec3(x,y,1.0f)*inverseMVPW;
      setxy(x,y);    
      getHits(node,near_point,far_point);
      return _PIVsegHitList;
    }
  
  inline void setxy(float xpt, float ypt) { xp=xpt; yp=ypt; }
  inline bool hits() const { return _PIVsegHitList.size()>0;}
  
 private:
  
  PickIntersectVisitor _piv;
  float xp, yp; // start point in viewport fraction coordiantes
  osgUtil::IntersectVisitor::HitList       _PIVsegHitList;
};



class PickVisitorRectangle : public osg::NodeVisitor
{
 public:
  PickVisitorRectangle()
    { 
      init();
    }
  
  void init(){
    xp=yp=0;    
    setTraversalMode(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN);
  }
  
  ~PickVisitorRectangle() {}
  
  void setTraversalMask(osg::Node::NodeMask traversalMask)
    {
      NodeVisitor::setTraversalMask(traversalMask);
      _piv.setTraversalMask(traversalMask);   
    }
  
  // Aug 2003 added to pass the nodemaskOverride to the PickIntersectVisitor
  // may be used make the visitor override the nodemask to visit 
  // invisible actions
  inline void setNodeMaskOverride(osg::Node::NodeMask mask) {
    _piv.setNodeMaskOverride(mask);
    _nodeMaskOverride = mask; }
  
  
  virtual void apply(osg::Projection& pr)
    { // stack the intersect rays, transform to new projection, traverse
      // Assumes that the Projection is an absolute projection
      osg::Matrixd mt;
      mt.invert(pr.getMatrix());
      osg::Vec3 npt=osg::Vec3(xp,yp,-1.0f) * mt;
      osg::Vec3 farpt=osg::Vec3(xp,yp,1.0f) * mt;
      
      // traversing the nodes children, using the projection direction
      for (unsigned int i=0; i<pr.getNumChildren(); i++) 
        {
	  osg::Node *nodech=pr.getChild(i);
	  /*osgUtil::IntersectVisitor::HitList &hli=
	    _piv.getIntersections(nodech,npt, farpt);

	  for(osgUtil::IntersectVisitor::HitList::iterator hitr=hli.begin();
	      hitr!=hli.end();
	      ++hitr)
            { 
	      // add the projection hits to the scene hits.
	      // This is why _lineSegment is retained as a member 
	      //of PickIntersectVisitor
	      _PIVsegHitList.push_back(*hitr);
	      }*/
	  traverse(*nodech);
        }
    }
  
  osgUtil::IntersectVisitor::HitList& 
    getHits(osg::Node *node,
	    const osg::Vec3& near_point1, const osg::Vec3& far_point1,
	    const osg::Vec3& near_point2, const osg::Vec3& far_point2)
    {
      // High level get intersection with sceneview using a ray 
      //     from x,y on the screen
      // sets xp,yp as pixels scaled to a mapping of (-1,1, -1,1); 
      // needed for Projection from x,y pixels      
      // first get the standard hits in un-projected nodes
      
      //_PIVsegHitList=_piv.getIntersections(node,near_point,far_point); // fill hitlist
      
      // then get hits in projection nodes
      traverse(*node); // check for projection nodes
      return _PIVsegHitList;
    }
  
  osgUtil::IntersectVisitor::HitList& getHits(osg::Node *node, const osg::Matrixd &projm, const float x1, const float y1,const float x2, const float y2 )
    { 
      assert(x1<x2 && y1<y2);

      // utility for non=sceneview viewers
      // x,y are values returned by 
      osg::Matrixd inverseMVPW;
      inverseMVPW.invert(projm);

      osg::Vec3 near_point1 = osg::Vec3(x1,y1,-1.0f)*inverseMVPW;
      osg::Vec3 far_point1 = osg::Vec3(x1,y1,1.0f)*inverseMVPW;

      osg::Vec3 near_point2 = osg::Vec3(x1,y1,-1.0f)*inverseMVPW;
      osg::Vec3 far_point2 = osg::Vec3(x1,y1,1.0f)*inverseMVPW;

      setxy(x1,y1);    

      getHits(node,near_point1,far_point1,near_point2,far_point2);


      return _PIVsegHitList;
    }
  
  inline void setxy(float xpt, float ypt) { xp=xpt; yp=ypt; }
  inline bool hits() const { return _PIVsegHitList.size()>0;}
  
 private:
  
  PickIntersectVisitor _piv;
  float xp, yp; // start point in viewport fraction coordiantes


   osgUtil::IntersectVisitor::HitList       _PIVsegHitList;
};



#endif
