/** -*-c++-*- *************************************************************\
* File:          SDLWindows.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'SDLWindows'
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/        

#include "SDLWindows.h"

#include "CookiMetier.h"
#include "CookiMessage.h"

SDLWindow::SDLWindow(){
        g_bDetail = true;
        g_DetailScale = 16;
        g_FrameInterval = 0.0f; 
        framesPerSecond=0.0f;
        g_bRenderMode = true;             
        videoFlags=0;
        surface= 0;
        g_bLighting=true;
        fov=near_=far_=0.0f;           
	_dontMoveCursor = false;
	_undecorated = false;
}

SDLWindow::~SDLWindow(){
  
  //SDL_Quit();
  SDL_QuitSubSystem(SDL_INIT_VIDEO);
}

//////////////////   CREATE MY WINDOW   \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
////////
////////  This function CREATES our WINDOW for drawing the GL stuff
////////
//////////////////   CREATE MY WINDOW   \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

SDL_Surface *  SDLWindow::Create_GL_Window(const char * strWindowName, int w, int h, int d) 
{
    height=h; width=w; depth=d;
    
    // try to initialize SDL video module
    if( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) < 0 ) 
    {
      // report error if it fails
      cerr << "Failed initializing SDL Video : " << SDL_GetError() << endl;  
      // we use this instead of Quit because SDL isn't yet initialized
      assertC2(0, NULL); 
    }
    atexit(SDL_Quit);
    
    SetupPixelFormat();

    //CheckError();
    
    //cout<<"surface = SDL_SetVideoMode("<<width<<","<<height
    //<<","<<depth<<","<<videoFlags<<"); avant"<<endl;
    
    surface = SDL_SetVideoMode(width, height,depth, videoFlags);     

    //cout<<"surface = SDL_SetVideoMode(width, 
    //height,depth, videoFlags); après"<<endl;

    // if window creation failed
    if( surface == NULL )                            
    {
      // report error 
      cerr << "Failed to Create Window : " << SDL_GetError() << endl;         
      Quit(0);
    }
    
    if (!_undecorated){
      // set the window caption (first argument) and icon caption (2nd arg)
      SDL_WM_SetCaption(strWindowName,strWindowName);    
    }

    if (!_dontMoveCursor){
      SDL_WarpMouse(width/2,height/2);  
    }
    // Enable Key Repeat
    if( SDL_EnableKeyRepeat(100,SDL_DEFAULT_REPEAT_INTERVAL) )
    {
        cerr << "Failed enabling key repeat" << endl;
        Quit(1);
    }  
    
    InitGLScreen();
    InitGL_Texturing2D();
    initGLState();
    return surface;
}





////////////////////   SETUP PIXEL FORMAT   \\\\\\\\\\\\\\\\\\\\\\\\\\\\*
///////
///////  Sets the pixel format for openGL and video flags for SDL
///////
////////////////////   SETUP PIXEL FORMAT   \\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void SDLWindow::SetupPixelFormat(void)
{
  //////// SURFACE IS THE DRAWABLE PORTION OF AN SDL WINDOW \\\\\\\\*
  
  /////////////  we set the common flags here
  // it's an openGL window
  videoFlags    = SDL_OPENGL;        
  // exclusive access to hardware colour palette
  videoFlags   |= SDL_HWPALETTE;     

  //videoFlags   |= SDL_FULLSCREEN;

  // the window must be resizeable
  videoFlags   |= SDL_RESIZABLE;     

  if (_undecorated){
    videoFlags |= SDL_NOFRAME;
  }
    
  // query SDL for information about our video hardware
  const SDL_VideoInfo * videoInfo = SDL_GetVideoInfo();     
  
  // if this query fails
  if(videoInfo == NULL)                                     
    {
        CERR(this,"SetupPixelFormat")<< "Failed getting Video Info : " 
				      << SDL_GetError() 
				      << endl;          // report error
        Quit(0);
      }
    
  ///////////// we set the system dependant flags here
  if(videoInfo -> hw_available)  // is it a hardware surface ?
    videoFlags |= SDL_HWSURFACE;
  else {
    videoFlags |= SDL_SWSURFACE;
    CTRACE(this,"SetupPixelFormat")
      <<"la surface n'est pas chargée en mémoire vidéo"<<endl;
  }
  
  // Blitting is fast copying / moving /swapping 
  // of contiguous sections of memory
  // for more about blitting check out : 
  // http://www.csc.liv.ac.uk/~fish/HTML/blitzman/bm_blitter.html
  // is hardware blitting available
  if(videoInfo -> blit_hw) {                      
    videoFlags |= SDL_HWACCEL;
  }
  else {
    CTRACE(this,"SetupPixelFormat")<<" blitting is not available "<<endl;
  }
  
  // tell SDL that the GL drawing is going to be double buffered
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );      
  // size of depth buffer 
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE,   depth);   
  // we aren't going to use the stencil buffer
  SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0);       
  // this and the next three lines set the bits allocated per pixel -
  SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0);     
  // - for the accumulation buffer to 0
  SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0);   
  SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0);
  SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0);
  
  //cout<<"fin SetupPixelFormat"<<endl;
}


/////////////////// RESIZE OPENGL SCREEN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////    This function resizes the viewport for OpenGL.
/////    Initialize The GL Window
/////////////////// RESIZE OPENGL SCREEN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void SDLWindow::InitGLScreen(float Fov,float Nearest,float Farthest)   
{

  //cout<<"InitGLScreen"<<endl;  
  fov=Fov;
  near_=Nearest;
  far_=Farthest;
  
  // Prevent A Divide By Zero error
  if (height==0) {
    // Make the Height Equal One
    height=1;                              
  }

  // Make our viewport the whole window
  glViewport(0,0,width,height);              
  
  // We could make the view smaller inside
  //cout<<"InitGLScreen 2"<<endl;  
  // Our window if we wanted too.
  // The glViewport takes (x, y, width, height)
  // This basically means, what our our drawing boundries
  
  // Select The Projection Matrix
  glMatrixMode(GL_PROJECTION);               
  // Reset The Projection Matrix
  glLoadIdentity();                          

  //cout<<"InitGLScreen 3"<<endl;  
  
  // Calculate The Aspect Ratio Of The Window
  // The parameters are:
  // (view angle, aspect ration of the width to the height, 
  //  The closest distance to the camera before it clips, 
  
  // FOV   // Ratio  //  The farthest distance before it stops drawing)
  gluPerspective(Fov,(GLfloat)width/(GLfloat)height,Nearest ,Farthest);
  
  // Select The Modelview Matrix
  glMatrixMode(GL_MODELVIEW);               
  // Reset The Modelview Matrix
  glLoadIdentity();                         
  
  //cout<<"InitGLScreen 4"<<endl;  
}


/////////////////////// INIT GAME WINDOW \\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////    This function initializes the game window.
/////
/////////////////////// INIT GAME WINDOW \\\\\\\\\\\\\\\\\\\\\\\\\\*

void SDLWindow::InitGL_Texturing2D()
{
  // Enables Depth Testing
  glEnable(GL_DEPTH_TEST);         
  // Enable Texture Mapping 
  glEnable(GL_TEXTURE_2D);       
  // Turn on back face culling
  //glEnable(GL_CULL_FACE);             
       
}


//////////////////// CALCULATE FRAME RATE \\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
/////   This function calculates the frame rate 
/////   and time intervals between frames
/////
//////////////////// CALCULATE FRAME RATE \\\\\\\\\\\\\\\\\\\\\\\\\\\*
  
void SDLWindow::CalculateFrameRate()
{
    
  // This will hold the time from the last frame
  static float lastTime           = 0.0f;     
  // We will store the string here for the window title
  static char strFrameRate[50] = {0};         
  // This stores the last frame's time
  static float frameTime = 0.0f;              

  // Get the current time in seconds
  float currentTime = SDL_GetTicks() * 0.001f;             
  
  // Here we store the elapsed time between the current and last frame,
  // then keep the current frame in our static variable for the next frame.
  g_FrameInterval = currentTime - frameTime;
  frameTime = currentTime;
  
  // Increase the frame counter
  ++framesPerSecond;
  
  // Now we want to subtract the current time by the last time that was stored
  // to see if the time elapsed has been over a second, 
  // which means we found our FPS.
  if( currentTime - lastTime > 1.0f )
    {
      // Here we set the lastTime to the currentTime
      lastTime = currentTime;
      
      // Copy the frames per second into a string to display 
      // in the window title bar
      sprintf(strFrameRate, "SDL Render : Cooki3d engine, fps: %d", 
	      int(framesPerSecond));
      
      // Set the window title bar to our string
      SDL_WM_SetCaption(strFrameRate, strFrameRate);
      
      // Reset the frames per second
      framesPerSecond = 0;
    }
}



void SDLWindow::SetResize(int w,int h,int d){
  if (!d) {
    d=depth;
  }
  glPushAttrib(GL_ALL_ATTRIB_BITS);
  
  //request SDL to resize the window to the size and depth etc. that we specify
  surface=SDL_SetVideoMode( w, h, d,videoFlags );
  //SDL_HWPALETTE|SDL_RESIZABLE|SDL_HWSURFACE|SDL_HWACCEL);
  glPopAttrib();
  height=h;width=w;
  // now resize the OpenGL viewport
  InitGLScreen();
  InitGL_Texturing2D();
  initGLState();
  // if window resize has failed
  if(surface == NULL){                              
    // report error
    cerr << " Failed resizing SDL window : " << SDL_GetError() << endl;   
    Quit(0);
  }
  
}

void SDLWindow::initGLState(){
  //GLfloat light0_null[] = { 0.0f,  0.0f,  0.0f,  1.0f}; 
   GLfloat light0_ambient[] = { 0.6f,  0.6f,  0.6f,  1.0f};
   GLfloat light0_diffuse[] = { 0.9f,  0.9f,  0.9f,  1.0f};
   GLfloat light0_specular[] = { 1.0f,  1.0f,  1.0f,  1.0f};
   GLfloat light0_position[] = {1.0f, 100.0f, 0.0f, 0.0f};      
   
   glLightfv(GL_LIGHT0, GL_AMBIENT,  light0_ambient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE,  light0_diffuse);
   glLightfv(GL_LIGHT0, GL_SPECULAR,  light0_specular);
   glLightfv(GL_LIGHT0, GL_POSITION,  light0_position);


   
   GLfloat material_diffuse []= { 1, 1, 1, 1 };
   GLfloat material_specular [] = { 1, 1, 1, 0 };
   GLfloat material_shininess [] = { 28}; //128 = maxi
   glMaterialfv(GL_FRONT, GL_DIFFUSE, material_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, material_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, material_shininess);
   
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_NORMALIZE);
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glEnable(GL_COLOR_MATERIAL);
   glShadeModel(GL_SMOOTH);
    //glEnable(GL_SMOOTH);
}


void SDLWindow::CheckError(){
/* Check for error conditions. */
  
  GLenum gl_error;
  char* sdl_error;
  gl_error = glGetError( );
  
  if( gl_error != GL_NO_ERROR ) {
    cerr<<"GL_INVALID_ENUM ="<<GL_INVALID_ENUM<<endl;
    cerr<<"GL_INVALID_VALUE ="<< GL_INVALID_VALUE<<endl;
    cerr<<"GL_INVALID_OPERATION ="<< GL_INVALID_OPERATION<<endl;
    cerr<<"GL_STACK_OVERFLOW ="<<GL_STACK_OVERFLOW <<endl;
    cerr<<"GL_STACK_UNDERFLOW  ="<<GL_STACK_UNDERFLOW<<endl;
    cerr<<"GL_OUT_OF_MEMORY  ="<<GL_OUT_OF_MEMORY<<endl;		
    fprintf( stderr, "testgl: OpenGL error: %d\n", gl_error );
    return;
  }
  
  sdl_error = SDL_GetError( );
  
  if( sdl_error[0] != '\0' ) {
    fprintf(stderr, "testgl: SDL error '%s'\n", sdl_error);
    SDL_ClearError();
  }
}




////////////////////// TOGGLE FULL SCREEN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
///////
///////   This function TOGGLES between FULLSCREEN and WINDOWED mode
///////
////////////////////// TOGGLE FULL SCREEN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

void SDLWindow::ToggleFullScreen(void){
  // try to toggle fullscreen mode for window 'main_window' 
  if(SDL_WM_ToggleFullScreen(surface) == 0) {
    // report error in case toggle fails
    cerr << "Failed to Toggle Fullscreen mode : " << SDL_GetError() << endl;   
    Quit(0);
  }
}


////////////////////// IsExtensionSupported \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////
///// This Function checks whether a particular extension is supported or not
/////
/////////////////////// IsExtensionSupported \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

bool SDLWindow::isExtensionSupported(string ext){
  int pos = 0;
  bool supported = false;
  
  int n = ext.length();
  
  cout<<"glGetString(GL_EXTENSIONS) "<<GL_EXTENSIONS<<endl;
  string extensions ((char *)glGetString(GL_EXTENSIONS));
  cout<<extensions<<endl;
  
  while( !supported )  {
    //pour linux: .compare(ext,n,pos) )
    if ( !extensions.compare(pos,n,ext) ) {
      return true;
    }
    pos = extensions.find(' ', pos) + 1;
    if( pos <= 0 ) {
      return false;
    }
  }
  return false;
}

