/** -*-c++-*- *************************************************************\
* File:          SDLWindows.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'SDLWindows'
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _SDLWINDOWS_H
#define _SDLWINDOWS_H




#include <iostream>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <GL/gl.h>           // Header File For The OpenGL32 Library
#include <GL/glu.h>          // Header File For The GLu32 Library
#include <GL/glext.h>
#include <SDL/SDL.h>
//#include <SDL/SDL_image.h>
#include <math.h>
#include <vector>
#include <string>

using namespace std;



class SDLWindow {
  
  //données    
  
  // This controls if we have detail texturing on or off
  bool g_bDetail ;       
  
  //This handles the curr scale for the texture matrix for the detail texture
  int g_DetailScale;     
  
  // Our global float that stores the elapsed time between the current and last frame
  float g_FrameInterval; 
  
  // This will store our fps
  float framesPerSecond;  

  // This tells us if we want lines or fill mode
  bool  g_bRenderMode;   

  // The Pixel Format flags
  int videoFlags;        

  // Our Drawable portion of the window 
  SDL_Surface * surface; 

  // Dimensions of the surface
  int height,width,depth;
  
  bool g_bLighting;
  float fov;
  float near_;
  float far_;  ///perspective
  
  bool _dontMoveCursor;
  bool _undecorated;
  
 public:

  SDLWindow();
  void setDontMoveCursor(bool b){ _dontMoveCursor = b; }
  void setUndecoracted(bool b){  _undecorated=b; }
  
  virtual ~SDLWindow();
  
  //void SetDetailScale(int d){g_DetailScale=d;}
  
  void SetResize(int w,int h,int d=0);
  
  void CheckError();
    
  void initGLState();
  
  void InvertRenderMode(){
    g_bRenderMode= !g_bRenderMode;
    if( g_bRenderMode )
      glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    else
      glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
  }
  
  void CalculateFrameRate();
  
  void ToggleFullScreen(void);
  
  // This allows us to configure our window for OpenGL and backbuffered
  void SetupPixelFormat(void);
  
  // This is our own function that makes creating a window modular and easy
  SDL_Surface * Create_GL_Window(const char *strWindowName, 
				 int w, int h, int d=16);
  
  // This inits our screen translations and projections
  void InitGLScreen(float Fov=90.0f,float Nearest=3.0f,float Farthest=5000.0f);

  // This initializes the texturing 2D
  void InitGL_Texturing2D();
  
  // This shuts down SDL and quits program
  void Quit(int ret_val){
    //TODO revoir exit
    SDL_Quit();         // shuts down SDL stuff
    //SDL_QuitSubSystem(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_JOYSTICK);
    exit(ret_val);      // quit the program
  }
  void setCaption(char * c){ SDL_WM_SetCaption(c,c); }
  
  void setCaption(string c){ SDL_WM_SetCaption(c.c_str(),c.c_str()); }
  
  bool isExtensionSupported(string ext);
  
  SDL_Surface * GetSurface(){ return surface; }
  
};


#endif
