/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          Sequence.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'Sequence'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include <osg/Sequence>

#include "Media.h"
#include "BD.h"
#include "Sequence.h"
#include "XMLUtil.h"
#include "EnvironnementOSG.h"
#include "LoaderGeneric.h"

namespace cooki3d{
//@END_USER2





//
// Group: Media
//
void Sequence::setPosition(double sec)
{//@CODE_1630

  //on calcul le saut à faire de le temps des frames
  double saut = double( int(sec*10000) % int(_totalFrameTime*10000))/10000.0;
  
  unsigned int i=0;
  double totalParcouru = 0;
  float speed;
  int nreps;
  _specializedOsgSequence->getDuration(speed, nreps);

  while ( saut > (totalParcouru+(_specializedOsgSequence->getTime( i )*speed))
	  && 
	  i < _specializedOsgSequence->getNumFrames() ){
    
    totalParcouru = totalParcouru 
      + (_specializedOsgSequence->getTime( i )*speed);
    i++;
  }

  //on set la frame trouvée
  _specializedOsgSequence->setValue ( i );
  

  //vérification
  assertC( (saut - totalParcouru) >= 0);

  double time = getEnv()->getFrameStamp()->getReferenceTime();
  //on précise l'avancement pour la frame choisie
  _specializedOsgSequence->setLast( time - (saut - totalParcouru) );

}//@CODE_1630


bool Sequence::isPlay()
{//@CODE_1632
  return _specializedOsgSequence->getMode()!=osg::Sequence::STOP;

}//@CODE_1632


bool Sequence::isPause()
{//@CODE_1633
  return _specializedOsgSequence->getMode()==osg::Sequence::PAUSE;
}//@CODE_1633


double Sequence::getDebut()
{//@CODE_1638
  return double( (double(_debut)/_fps) );
}//@CODE_1638


double Sequence::getDuration()
{//@CODE_1639
    return getFin()-getDebut();
}//@CODE_1639


double Sequence::getFin()
{//@CODE_1640
  return double( (double(_fin)/_fps) - 0.00001); 
}//@CODE_1640


void Sequence::pause()
{//@CODE_1648
  _specializedOsgSequence->setMode(osg::Sequence::PAUSE);

}//@CODE_1648


void Sequence::play()
{//@CODE_1649
  if (isPause())
    _specializedOsgSequence->setMode(osg::Sequence::RESUME);
  else
    _specializedOsgSequence->setMode(osg::Sequence::START);
  

}//@CODE_1649


double Sequence::getPosition()
{//@CODE_1650


  float speed;
  int nreps;
  _specializedOsgSequence->getDuration(speed, nreps);
 
  float totalParcouru = 0;
  int i=0;
  
  while ( i < _specializedOsgSequence->getValue() ){

    
    totalParcouru = totalParcouru 
      + (_specializedOsgSequence->getTime( i )*speed);
    i++;
  }
  
  double time = getEnv()->getFrameStamp()
    ->getReferenceTime();
  
  return totalParcouru + (time - _specializedOsgSequence->getLast());

}//@CODE_1650


void Sequence::setDuration(double duree)
{//@CODE_1651
   _specializedOsgSequence->setDuration((float)duree);
}//@CODE_1651


void Sequence::stop()
{//@CODE_1655
  _specializedOsgSequence->setMode(osg::Sequence::STOP);
  _specializedOsgSequence->setValue( _specializedOsgSequence->getBegin() );
}//@CODE_1655


// End of Group: Media


//
// Group: Modes
//
osg::Sequence::LoopMode Sequence::traduireLoopModeFromStringToEnum(std::string mode)
{//@CODE_1641
  if (mode == "LOOP") 
    return osg::Sequence::LoopMode(osg::Sequence::LOOP);
  if (mode == "SWING") 
    return osg::Sequence::LoopMode(osg::Sequence::SWING);
  assertC(0);
  return (osg::Sequence::LoopMode)-1; //dummy pour compilat°
  
}//@CODE_1641


void Sequence::setSequenceMode(std::string mode)
{//@CODE_1845
  _specializedOsgSequence->setMode(traduireSequenceModeFromStringToEnum(mode));
}//@CODE_1845


void Sequence::setLoopMode(std::string mode)
{//@CODE_1653
   // _specializedOsgSequence->setInterval(traduireLoopModeFromStringToEnum(mode),int begin, int end)
  assertC(0);
}//@CODE_1653


osg::Sequence::SequenceMode Sequence::traduireSequenceModeFromStringToEnum(std::string mode)
{//@CODE_1848
  CTRACE(this,"traduireSequenceModeFromStringToEnum")
    <<"mode ="<<mode<<"."<<endl;
  
  if (mode == "START") 
    return osg::Sequence::SequenceMode(osg::Sequence::START);
  if (mode == "STOP") 
    return osg::Sequence::SequenceMode(osg::Sequence::STOP);
  if (mode == "PAUSE") 
    return osg::Sequence::SequenceMode(osg::Sequence::PAUSE);
  if (mode == "RESUME") 
    return osg::Sequence::SequenceMode(osg::Sequence::RESUME);
  
 CERR(this,"traduireSequenceModeFromStringToEnum")
   <<"mode.length() ="<<mode.length()<<endl;
 assertC(0);
 return (osg::Sequence::SequenceMode)-1; //dummy pour compilat°
}//@CODE_1848


std::string Sequence::traduireLoopModeFromEnumToString(osg::Sequence::LoopMode mode)
{//@CODE_1864
    switch (mode){    
    case osg::Sequence::LOOP:    
        return "LOOP";
    case osg::Sequence::SWING:
        return "SWING";              
    }
    assertC(0);    
    return "null";
}//@CODE_1864


std::string Sequence::traduireSequenceModeFromEnumToString(osg::Sequence::SequenceMode mode)
{//@CODE_1866
    switch (mode){    
    case osg::Sequence::START:    
        return "START";
    case osg::Sequence::STOP:
        return "STOP";  
    case osg::Sequence::PAUSE:
        return "PAUSE"; 
    case osg::Sequence::RESUME:       
        return "RESUME"; 
    }
    assertC(0);    
    return "null"; //dummy pour compilation
}//@CODE_1866


std::string Sequence::getLoopMode()
{//@CODE_1871
  // return traduireLoopModeFromEnumToString(_specializedOsgSequence->getLoopMode());
  assertC(0);
  return "null"; //dummy
}//@CODE_1871


std::string Sequence::getSequenceMode()
{//@CODE_1872
    return traduireSequenceModeFromEnumToString(_specializedOsgSequence->getMode());
}//@CODE_1872


// End of Group: Modes


//
// Group: load
//
bool Sequence::loadXML(const xmlpp::Node* node, LoaderXML* loaderXML)
{//@CODE_1646
  CTRACE(this,"loadXML(const xmlpp::Node* node)")<<"LoadSequence &&&&&&&&&&&&&&&&&&&&&&&&&"<<endl;


  XMLUtil xml(node);

  setName(xml.getFirstText("nom"));
  setDebut(atoi(xml.getFirstText("debut").c_str()));
  setFin(atoi(xml.getFirstText("fin").c_str()));
  setFps(atoi(xml.getFirstText("fps").c_str()));
  _bufferLoopMode = xml.getFirstText("loopMode");
  _bufferSequenceMode = xml.getFirstText("sequenceMode");
  setOsgNodeName(xml.getFirstText("osgNodeName"));
  
  return true;
}//@CODE_1646


bool Sequence::linkSequence(osg::Node* n, const std::string& pickName,
                            LoaderGeneric* loader)
{//@CODE_1644
  
  //cout<<" passage linkSequence su le Node "<<typeid(*n).name()<<endl;
  assertC2(n, loader->getBD());

  //std::string pickName = n->getName();
  //cout<<" passage linkSequence su le Node "<<pickName<<endl;
  
  
  if (n->asGroup()){
    osg::Sequence *osgSeq=dynamic_cast<osg::Sequence *>(n);
    if (osgSeq) {
      
      
      BD::EvaluableIterator itMed(loader->getBD());
	
      Sequence * seq;
           
      while (++itMed){
	seq=dynamic_cast<Sequence *>(itMed.Get());
	if (seq && (seq->getOsgNodeName() == pickName)) {
	  seq->
	    setOsgSequence(reinterpret_cast<SpecializedOsgSequence*>(osgSeq));
	  
	  CTRACEs("Sequence::linkSequence")<<" setOsgSequence sur "
					   <<pickName<<endl;
	  seq->init();
	  CTRACEs("Sequence::linkSequence")<<" init animation "
					   <<pickName<<" "<<seq->getName()
					   <<endl;
	  return true;
	}
      }
      
    }    
  }
  return false;
  
}//@CODE_1644


void Sequence::init()
{//@CODE_1643
  //setLoopMode(_bufferLoopMode);
  setSequenceMode(_bufferSequenceMode);
  float speed;
  int nreps;
  _specializedOsgSequence->getDuration(speed, nreps);
  
  for (unsigned int i=0; i< _specializedOsgSequence->getNumFrames(); i++){    
    
    _totalFrameTime = _totalFrameTime + (_specializedOsgSequence->getTime( i )*speed);    
  }
  //on se met sur la premiere frame plutot que -1 qui ne fait rien apparaître
  if (_bufferSequenceMode == "STOP"){
    _specializedOsgSequence->setValue( _specializedOsgSequence->getBegin() );
  }

}//@CODE_1643


bool Sequence::check()
{//@CODE_3298
    bool resultOk = _specializedOsgSequence != NULL && _totalFrameTime > 0;

    if (!resultOk){
      CERR(this,"check")<<getName()<<":"<<getOsgNodeName()<<" is not linked !!"<<endl;
    }
    
    return resultOk;
}//@CODE_3298


bool Sequence::checkAllInstances(LoaderGeneric* loader)
{//@CODE_3299

  BD::EvaluableIterator itSeq(loader->getBD());
  
  Sequence * seq;

  bool resultOk = true;
   
  while (++itSeq){
    
    seq=dynamic_cast<Sequence *>(itSeq.Get());
    
    if (seq && seq->getName() != "staticLoader") {
      resultOk = resultOk && seq->check();
    }

  }
   
  return resultOk;
}//@CODE_3299


/*@NOTE_1629
Constructor method.
*/
Sequence::Sequence(BD* pBD) //@INIT_1629
  : MediaEvaluable(pBD)
    , _bufferLoopMode("LOOP")
    , _bufferSequenceMode("STOP")
    , _totalFrameTime(0)
    , _debut(0) , _fin(100), _fps(30)  
    , _osgNodeName("null")  
    , _specializedOsgSequence(NULL)      
{//@CODE_1629
    ConstructorInclude();
    //_loopMode=osg::AnimationPath::NO_LOOPING;
    //_pathInit = new osg::AnimationPath;    
    
    // Put in your own code	
}//@CODE_1629


/*@NOTE_1594
Destructor method.
*/
Sequence::~Sequence()
{//@CODE_1594
    DestructorInclude();

    // Put in your own code
}//@CODE_1594


Evaluable* Sequence::staticCreate(BD* pBD)
{//@CODE_3027
  /*Sequence * s = new Sequence();
    s->setNameWithoutPrefix("staticLoader");
    return s;*/
  return new Sequence(pBD);
}//@CODE_3027


// End of Group: load

//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_1595
Method which must be called first in a constructor.
*/
void Sequence::ConstructorInclude()
{
}


/*@NOTE_1596
Method which must be called first in a destructor.
*/
void Sequence::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
} //end of namespace cooki3d
