/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          Sequence.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'Sequence'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _SEQUENCE_H
#define _SEQUENCE_H

//@START_USER1



#include <osg/Matrixf>
#include <osg/Matrixd>
#include <osg/Quat>
#include <osg/NodeCallback>
#include <osg/AnimationPath>
#include <osg/Sequence>

#include <map>
#include <float.h>

//#include <libxml++/libxml++.h> 



#include "MediaEvaluable.h"
#include "XMLUtil.h"



namespace cooki3d{

//Attention danger, ne pas rajouter d'attribut...
//pour les reinterpret_cast, seule les ajouts de méthodes sont possibles
//car l'objet instancié est un osg::Sequence
  class SpecializedOsgSequence : public osg::Sequence {
    
  public:
    void setLast(float f){
      _last = f;
    }
    float getLast(){
      return _last;
    }
    int getBegin(){
      return _begin;
    }
  };

//@END_USER1


/*@NOTE_1593
classe ...
*/

class AFX_EXT_CLASS Sequence
    : public MediaEvaluable
{

//@START_USER2
  public :
    typedef map<string,Sequence*,less<string > > mapStrAnim;
    
//@END_USER2

//
// Group: ClassBuilder methods
//

private:
    void ConstructorInclude();
    void DestructorInclude();

//
// Group: Command
//

//
// Group: Media
//

public:
    virtual void setPosition(double sec);
    virtual bool isPlay();
    virtual bool isPause();
    double getDebut();
    virtual double getDuration();
    double getFin();
    virtual void pause();
    virtual void play();
    virtual double getPosition();
    virtual void setDuration(double duree);
    virtual void stop();

//
// Group: Modes
//

private:
    std::string _bufferLoopMode;
    std::string _bufferSequenceMode;

public:
    osg::Sequence::LoopMode traduireLoopModeFromStringToEnum(std::string mode);
    void setSequenceMode(std::string mode);
    void setLoopMode(std::string mode);
    osg::Sequence::SequenceMode traduireSequenceModeFromStringToEnum(std::string mode);
    std::string traduireLoopModeFromEnumToString(osg::Sequence::LoopMode mode);
    std::string traduireSequenceModeFromEnumToString(osg::Sequence::SequenceMode mode);
    std::string getLoopMode();
    std::string getSequenceMode();

//
// Group: load
//

public:
    bool loadXML(const xmlpp::Node* node, LoaderXML* loaderXML);
    static bool linkSequence(osg::Node* n, const std::string& pickName,
                             LoaderGeneric* loader);
    void init();
    bool check();
    static bool checkAllInstances(LoaderGeneric* loader);
    Sequence(BD* pBD);
    virtual ~Sequence();
    static Evaluable* staticCreate(BD* pBD);

//
// Group: data
//

protected:
    double _totalFrameTime;
    int _debut;
    int _fin;
    int _fps;
    std::string _osgNodeName;
    SpecializedOsgSequence* _specializedOsgSequence;
    SpecializedOsgSequence* getSpecializedOsgSequence() const;
    void setSpecializedOsgSequence(SpecializedOsgSequence* pSpecializedOsgSequence);

public:
    int getDebut() const;
    void setDebut(int debut);
    int getFin() const;
    void setFin(int fin);
    int getFps() const;
    void setFps(int fps);
    std::string getOsgNodeName() const;
    void setOsgNodeName(std::string osgNodeName);
    void setOsgSequence(osg::Sequence* seq);
    osg::Sequence* getOsgSequence() const;

//
// Non-Grouped Members
//

//
// Non-Grouped Methods
//
};

#endif


#ifdef CB_INLINES
#ifndef _SEQUENCE_H_INLINES
#define _SEQUENCE_H_INLINES


//
// Group: data
//
inline void Sequence::setOsgSequence(osg::Sequence* seq)
{//@CODE_3302
  setSpecializedOsgSequence (reinterpret_cast<SpecializedOsgSequence*>(seq));
}//@CODE_3302



inline osg::Sequence* Sequence::getOsgSequence() const
{//@CODE_3306
  return _specializedOsgSequence;
}//@CODE_3306



/*@NOTE_1606
Returns the value of member '_debut'.
*/
inline int Sequence::getDebut() const
{//@CODE_1606
    return _debut;
}//@CODE_1606



/*@NOTE_1607
Set the value of member '_debut' to 'debut'.
*/
inline void Sequence::setDebut(int debut)
{//@CODE_1607
    _debut = debut;
}//@CODE_1607



/*@NOTE_1610
Returns the value of member '_fin'.
*/
inline int Sequence::getFin() const
{//@CODE_1610
    return _fin;
}//@CODE_1610



/*@NOTE_1611
Set the value of member '_fin' to 'fin'.
*/
inline void Sequence::setFin(int fin)
{//@CODE_1611
    _fin = fin;
}//@CODE_1611



/*@NOTE_1624
Returns the value of member '_fps'.
*/
inline int Sequence::getFps() const
{//@CODE_1624
    return _fps;
}//@CODE_1624



/*@NOTE_1625
Set the value of member '_fps' to 'fps'.
*/
inline void Sequence::setFps(int fps)
{//@CODE_1625
    _fps = fps;
}//@CODE_1625



/*@NOTE_1853
Returns the value of member '_osgNodeName'.
*/
inline std::string Sequence::getOsgNodeName() const
{//@CODE_1853
    return _osgNodeName;
}//@CODE_1853



/*@NOTE_1854
Set the value of member '_osgNodeName' to 'osgNodeName'.
*/
inline void Sequence::setOsgNodeName(std::string osgNodeName)
{//@CODE_1854
    _osgNodeName = osgNodeName;
}//@CODE_1854



/*@NOTE_1850
Returns the value of member '_osgSequence'.
*/
inline SpecializedOsgSequence* Sequence::getSpecializedOsgSequence() const
{//@CODE_1850
    return _specializedOsgSequence;
}//@CODE_1850



/*@NOTE_1851
Set the value of member '_osgSequence' to 'pOsgSequence'.
*/
inline void Sequence::setSpecializedOsgSequence(SpecializedOsgSequence* pSpecializedOsgSequence)
{//@CODE_1851
    _specializedOsgSequence = pSpecializedOsgSequence;
}//@CODE_1851



// End of Group: data

//@START_USER3
} //end of namespace cooki3d
//@END_USER3

#endif
#endif
