/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          SolCollisions.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'SolCollisions'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include "SolCollisions.h"
//#include "Action.h"
//#include "Event.h"
//#include "EvtMouse.h"
//#include "MediaInterface.h"
//#include "Media.h"
//#include "Command.h"
#include "Evaluable.h"
#include "EnvironnementOSG.h"
#include "LoaderXML.h"
#include <osg/PositionAttitudeTransform>
#include "HeightMap.h"
#include <osgDB/ReadFile>
//#include "HeightMap/HeightMapDynamic.hpp"
namespace cooki3d{
//@END_USER2


// Static members
MapUserRegister<std::vector<HeightMapAbstract*> > SolCollisions::_maps2;
MapUserRegister<std::vector<double> > SolCollisions::_altitude2;
MapUserRegister<std::vector<double> > SolCollisions::_scale2;


/*@NOTE_2250
Constructor method.
*/
SolCollisions::SolCollisions(BD* pBD) //@INIT_2250
    :SolGravitation(pBD)
{//@CODE_2250
    ConstructorInclude();

    // Put in your own code
}//@CODE_2250


/*@NOTE_2233
Destructor method.
*/
SolCollisions::~SolCollisions()
{//@CODE_2233
    DestructorInclude();

    // Put in your own code
}//@CODE_2233


float SolCollisions::getDeltaCase(BD* pBD)
{//@CODE_2305

  std::vector<HeightMapAbstract*> & regMaps2 = _maps2.getRegister(pBD);
    
  if (!regMaps2.size()) {
    CERRs("getDeltaCase()")<<"erreur pas de solCollision disponible!"<<endl;
    assert(0);
  }
    
    //@TODO à voir pour généraliser avec plus d'une maps
    return regMaps2[0]->getDeltaCase();
    
    
}//@CODE_2305


float SolCollisions::getHeightAbs(float x, float y, BD* pBD)
{//@CODE_2251
  
  float value=0.0f;
  
  std::vector<HeightMapAbstract*> & regMaps2 = _maps2.getRegister(pBD);
  std::vector<double> & regAltitude2 = _altitude2.getRegister(pBD);
  std::vector<double> & regScale2 = _scale2.getRegister(pBD);
  
  if (!regMaps2.size()) {
    CERRs("getHeightAbs(float x, float y)")
      <<"erreur pas de solCollision disponible!"<<endl;
    return value;
  }

  assertC2( regMaps2.size() == regAltitude2.size() == regScale2.size(), pBD);

  std::vector<HeightMapAbstract*>::iterator itMap = regMaps2.begin();
  std::vector<double>::iterator itAltitude = regAltitude2.begin();  
  std::vector<double>::iterator itScale =  regScale2.begin();
  
  while (itMap != regMaps2.end()){
    
    if ( (*itMap)->contains( x/(*itScale), y/(*itScale) ) ){
      
      value= ( (*itMap)->getHeightAbsNoException( x/(*itScale), y/(*itScale) )
	       + (*itAltitude) ) * (*itScale);
      return value;
    }
    
    itMap++;
    itAltitude++;
    itScale++;
    
  }
 
  CERRs("SolCollision::getHeightAbs(float x, float y)")
    <<"No find x="<<x<<" y="<<y<<endl;
    
  return value;
}//@CODE_2251


bool SolCollisions::loadXML(const xmlpp::Node* node, LoaderXML* loaderXML)
{//@CODE_2258
  CTRACE(this,"loadXML")<<"LoadSolCollisions &&&&"<<endl;

  return SolGravitation::loadXML(node,loaderXML);
}//@CODE_2258


void SolCollisions::pushAltitude(double altitude)
{//@CODE_2282
  
  std::vector<double> & regAltitude2 = _altitude2.getRegister(getBD());
  regAltitude2.push_back(altitude);
}//@CODE_2282


void SolCollisions::pushMap(HeightMapAbstract* pHeightMapAbstract)
{//@CODE_2284
  std::vector<HeightMapAbstract*> & regMaps2 = _maps2.getRegister(getBD());
  regMaps2.push_back(pHeightMapAbstract);    
}//@CODE_2284


void SolCollisions::pushScale(double scale)
{//@CODE_2286

  std::vector<double> & regScale2 = _scale2.getRegister(getBD());
  regScale2.push_back( scale );
}//@CODE_2286


Evaluable* SolCollisions::staticCreate(BD* pBD)
{//@CODE_3043
    return new SolCollisions(pBD);
}//@CODE_3043


//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_2234
Method which must be called first in a constructor.
*/
void SolCollisions::ConstructorInclude()
{
}


/*@NOTE_2235
Method which must be called first in a destructor.
*/
void SolCollisions::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
} //end of namespace cooki3d
