/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          SolGravitation.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'SolGravitation'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include "SolGravitation.h"
//#include "Action.h"
//#include "Event.h"
//#include "EvtMouse.h"
//#include "MediaInterface.h"
//#include "Media.h"
//#include "Command.h"
#include "Evaluable.h"
#include "EnvironnementOSG.h"
#include "LoaderXML.h"
#include <osg/PositionAttitudeTransform>
#include "HeightMap.h"
#include <osgDB/ReadFile>
//#include "HeightMap/HeightMapDynamic.hpp"
#include "ScanSetGraph.h"
#include "Methode_2args.h"


namespace cooki3d{
//@END_USER2


// Static members
MapUserRegister<std::vector<HeightMapAbstract*> > SolGravitation::_maps;
MapUserRegister<std::vector<double> > SolGravitation::_altitude;
MapUserRegister<std::vector<double> > SolGravitation::_scale;
const int SolGravitation::_MODE_HEIGHTMAP = 0;
const int SolGravitation::_MODE_DYNAMIQUE = 1;


/*@NOTE_419
Constructor method.
*/
SolGravitation::SolGravitation(BD* pBD) //@INIT_419
    : Evaluable(pBD)
    , _mode(_MODE_HEIGHTMAP)
    , _terrainVisible(true)
    , _heightMapVisible()
{//@CODE_419
    ConstructorInclude();

    // Put in your own code
}//@CODE_419


/*@NOTE_415
Destructor method.
*/
SolGravitation::~SolGravitation()
{//@CODE_415
    DestructorInclude();

    // Put in your own code
}//@CODE_415


float SolGravitation::getHeightAbs(float x, float y, BD* pBD)
{//@CODE_424
  
  float value=100.0f;
  
  std::vector<HeightMapAbstract*> & regMaps = _maps.getRegister(pBD);
  std::vector<double> & regAltitude = _altitude.getRegister(pBD);
  std::vector<double> & regScale = _scale.getRegister(pBD);
  
  if (!regMaps.size()) {
    CERRs("getHeightAbs(float x, float y)")
      <<"erreur pas de solGravitation disponible!"<<endl;
    return value;
  }
  assertC2( regMaps.size() == regAltitude.size() == regScale.size(), pBD );

  std::vector<HeightMapAbstract*>::iterator itMap = regMaps.begin();
  std::vector<double>::iterator itAltitude = regAltitude.begin();  
  std::vector<double>::iterator itScale =  regScale.begin();
  
  while ( itMap != regMaps.end() ){
    
    if ( (*itMap)->contains( x/(*itScale), y/(*itScale) ) ){
      
      value= ( (*itMap)->getHeightAbsNoException( x/(*itScale), y/(*itScale) )
	       + (*itAltitude) ) * (*itScale);
      return value;
    }
    
    itMap++;
    itAltitude++;
    itScale++;
    
  }
 
  CERRs("SolGravitation::getHeightAbs(float x, float y)")
    <<"No find x="<<x<<" y="<<y<<endl;
    
  return value;
}//@CODE_424


bool SolGravitation::loadXML(const xmlpp::Node* node, LoaderXML* loaderXML)
{//@CODE_420
  
  XMLUtil xml(node);
  
  setName(xml.getFirstText("nom"));
  
  string hMapGeo=xml.getFirstText("heigthMapGeo");

  setTerrainVisible(xml.getFirstText("terrainVisible")=="true");  

  string mode =xml.getFirstText("altitudeDetection");

  string hMapRaw=xml.getFirstText("heigthMapRaw");

  setHeightMapVisible(xml.getFirstText("heightMapVisible")=="true");  
  
  double x= atof(xml.getFirstText("x").c_str());
  double y= atof(xml.getFirstText("y").c_str());
  
  double altitude = atof(xml.getFirstText("altitude").c_str());
  pushAltitude(altitude);
    
  double scale = atof(xml.getFirstText("scale").c_str());
  pushScale(scale);


  osg::ref_ptr<osg::Node> ter=
    osgDB::readNodeFile(loaderXML->getPath(hMapGeo));
  
  
  //@TODO ne pas charger si pas de regénération
  //càd vérifier que le fichier.raw existe...

  //chargement du fichier 
  osg::PositionAttitudeTransform * pat = new osg::PositionAttitudeTransform();
  pat->setName(getName());
  pat->setPosition(osg::Vec3(x,y,altitude));
  pat->setScale(osg::Vec3(scale,scale,scale));
  osg::StateSet *state= new osg::StateSet();
  state->setMode(GL_RESCALE_NORMAL, osg::StateAttribute::ON );
  pat->setStateSet(state);
  pat->addChild(ter.get());  

      
  
  if (mode=="MODE_HEIGHTMAP"){
    CTRACE(this,"loadXML")<<"mode HEIGHTMAP"<<endl;    

    //on sauvegarde la scene avant de ne laisser que le terrain
    osg::ref_ptr<osg::Group>  sauvRootNode= loaderXML->getRootNode();
    //* .get();
    loaderXML->setRootNode(pat);
    
    //important de ne passer que le terrain à cette classe
    //pour des probleme de détermination d'echelle et autres...
    
    HeightMap * hmap= new HeightMap(ter.get(), loaderXML->getEnv());
    hMapRaw = loaderXML->getPath(hMapRaw);
    
    if ( !  hmap->loadRawFile(hMapRaw.c_str()) ){
      return false;
    }
    
    pushMap(hmap);


    //on restitue la scene originelle
    loaderXML->setRootNode(sauvRootNode);
            
    if (_terrainVisible || _heightMapVisible){
 
      loaderXML->getRootNode()->addChild(pat);   
 
      if (_heightMapVisible){
	
	pat->addChild(hmap->getGeode());       
	
      }
      //attention à l'ordre le desallocate peut etre violent...
      if (!_terrainVisible){	
	
	ter->ref();
		
	pat->removeChild(ter.get());	
      }
      
      //TODO à revoir
      //parcours de l'arbre et adaptation des noms de noeuds
      /*Rename*/
      /*ScanSetGraph mesh;  
      Methode_2args<SolGravitation,osg::Node*,osg::Node*> 
	com(&SolGravitation::scanCommand,this);   
      com.setName("parcoursGraphSolGravitation_"+getName());
      mesh.scanEtCommand2args(pat,NULL,&com);*/
      
    }else {
      //@TODO s'occuper d'effacer pat completement?
      CTRACE(this,"loadXML")<<"TODO";
      //delete pat;          
    }
  }
  else if (mode=="MODE_DYNAMIQUE"){
    
    CTRACE(this,"loadXML")<<"mode DYNAMIQUE"<<endl;
    HeightMapDynamic * dyna= new HeightMapDynamic(ter.get(),
						  loaderXML->getEnv());
    pushMap(dyna);    
  }
  else {
    CERR(this,"loadXML")<<"mode "<<mode<<" inconnu!!"<<endl;
    return false;
  }

  return true;
}//@CODE_420


void SolGravitation::pushAltitude(double altitude)
{//@CODE_2278
  std::vector<double> & regAltitude = _altitude.getRegister(getBD());  
  regAltitude.push_back(altitude);
}//@CODE_2278


void SolGravitation::pushMap(HeightMapAbstract* pHeightMapAbstract)
{//@CODE_2276
  std::vector<HeightMapAbstract*> & regMaps = _maps.getRegister(getBD());
  regMaps.push_back(pHeightMapAbstract);    
}//@CODE_2276


void SolGravitation::pushScale(double scale)
{//@CODE_2280
  std::vector<double> & regScale = _scale.getRegister(getBD());
  regScale.push_back(scale);
}//@CODE_2280


void SolGravitation::scanCommand(osg::Node* n, osg::Node* parent)
{//@CODE_2270
  
  
  if (n->getName() == "") {
    if (parent){
      n->setName(parent->getName());
    }
    else {
      n->setName(getName()+"_ValeurParDefaut");
    }    
  }
  else n->setName(getName()+"_"+n->getName());

  
  
}//@CODE_2270


Evaluable* SolGravitation::staticCreate(BD* pBD)
{//@CODE_3042
    return new SolGravitation(pBD);
}//@CODE_3042


//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_416
Method which must be called first in a constructor.
*/
void SolGravitation::ConstructorInclude()
{
}


/*@NOTE_417
Method which must be called first in a destructor.
*/
void SolGravitation::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
} //end of namespace cooki3d
