/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          SolGravitation.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'SolGravitation'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _SOLGRAVITATION_H
#define _SOLGRAVITATION_H

//@START_USER1

#include <vector>
using namespace std;

#include "XMLUtil.h"
#include "HeightMap.h"
#include "MapUserRegister.h"

namespace cooki3d{
//@END_USER1


/*@NOTE_414
class ...
*/

class AFX_EXT_CLASS SolGravitation
    : public Evaluable
{

//@START_USER2
public :
//@END_USER2

//
// Group: ClassBuilder methods
//

private:
    void ConstructorInclude();
    void DestructorInclude();

//
// Non-Grouped Members
//

private:
    static MapUserRegister<std::vector<HeightMapAbstract*> > _maps;
    static MapUserRegister<std::vector<double> > _altitude;
    static MapUserRegister<std::vector<double> > _scale;

protected:
    int _mode;
    bool _terrainVisible;
    bool _heightMapVisible;

public:
    static const int _MODE_HEIGHTMAP;
    static const int _MODE_DYNAMIQUE;

//
// Non-Grouped Methods
//

public:
    SolGravitation(BD* pBD);
    virtual ~SolGravitation();
    static float getHeightAbs(float x, float y, BD* pBD);
    virtual bool loadXML(const xmlpp::Node* node, LoaderXML* loaderXML);
    virtual void pushAltitude(double altitude);
    virtual void pushMap(HeightMapAbstract* pHeightMapAbstract);
    virtual void pushScale(double scale);
    void scanCommand(osg::Node* n, osg::Node* parent);
    static Evaluable* staticCreate(BD* pBD);
    bool getHeightMapVisible() const;
    void setHeightMapVisible(bool heightMapVisible);
    int getMode() const;
    void setMode(int mode);
    bool getTerrainVisible() const;
    void setTerrainVisible(bool terrainVisible);
};

#endif


#ifdef CB_INLINES
#ifndef _SOLGRAVITATION_H_INLINES
#define _SOLGRAVITATION_H_INLINES

/*@NOTE_2267
Returns the value of member '_heightMapVisible'.
*/
inline bool SolGravitation::getHeightMapVisible() const
{//@CODE_2267
    return _heightMapVisible;
}//@CODE_2267



/*@NOTE_2268
Set the value of member '_heightMapVisible' to 'heightMapVisible'.
*/
inline void SolGravitation::setHeightMapVisible(bool heightMapVisible)
{//@CODE_2268
    _heightMapVisible = heightMapVisible;
}//@CODE_2268



/*@NOTE_1384
Returns the value of member '_mode'.
*/
inline int SolGravitation::getMode() const
{//@CODE_1384
    return _mode;
}//@CODE_1384



/*@NOTE_1385
Set the value of member '_mode' to 'mode'.
*/
inline void SolGravitation::setMode(int mode)
{//@CODE_1385
    _mode = mode;
}//@CODE_1385



/*@NOTE_2261
Returns the value of member '_terrainVisible'.
*/
inline bool SolGravitation::getTerrainVisible() const
{//@CODE_2261
    return _terrainVisible;
}//@CODE_2261



/*@NOTE_2262
Set the value of member '_terrainVisible' to 'terrainVisible'.
*/
inline void SolGravitation::setTerrainVisible(bool terrainVisible)
{//@CODE_2262
    _terrainVisible = terrainVisible;
}//@CODE_2262



//@START_USER3
} //end of namespace cooki3d
//@END_USER3

#endif
#endif
