/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          TimeBar.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'TimeBar'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
//@START_USER1
//@END_USER1


// Master include file
#include "CookiMetier.h"


//@START_USER2
#include <SDL/SDL.h>
#include <SDL/SDL_mutex.h>
#include <SDL/SDL_thread.h>

#include "TimeBar.h"
#include "LoaderXML.h"

namespace cooki3d{
//@END_USER2





//
// Group: Media
//
double TimeBar::eval(std::string methode, double args[10], int nbParam)
{//@CODE_1205
    return Media::eval(methode,args,nbParam);
}//@CODE_1205


double TimeBar::getDuration()
{//@CODE_494
    return _timeSize;
}//@CODE_494


double TimeBar::getPosition()
{//@CODE_497
  if (_stop) return 0.0;
  
  return osg::Timer::instance()->
    delta_s(_tickDebut,osg::Timer::instance()->tick())+_tpsSauv;
  
}//@CODE_497


bool TimeBar::isPause()
{//@CODE_493
    return _pause;
}//@CODE_493


bool TimeBar::isPlay()
{//@CODE_492
    return !_stop;
}//@CODE_492


void TimeBar::pause()
{//@CODE_495
  _pause = true;
  _tpsSauv = osg::Timer::instance()->delta_s(_tickDebut,
					     osg::Timer::instance()->tick()); 
    
}//@CODE_495


void TimeBar::play()
{//@CODE_496
  //static bool _started=false;//-->MAUVAIS
  //if (!_started) {
    assertC(_clefs.size());
    _clefIT=_clefs.begin();
    //_started = true;
    //start();
    //}
  
  _tickDebut = osg::Timer::instance()->tick();

  if (_stop) {//nécessairement après?
    _stop = false;
  }
  else if (_pause){
    _pause = false;
  }
  
  
  
}//@CODE_496


void TimeBar::setDuration(double duree)
{//@CODE_1886

  assertCMessage(0,"Not implemented");
  
}//@CODE_1886


void TimeBar::setPosition(double sec)
{//@CODE_490
  _tpsSauv = sec;
  assertC(_clefs.size());
  MapClef::iterator it=_clefs.begin();
  while (  (it != _clefs.end()) && (it->first <sec)  ) it++;
  
  SDL_LockMutex(_clefITmutex); 
  if (it == _clefs.end()) _clefIT = _clefs.begin();
  else _clefIT = it;
  SDL_UnlockMutex(_clefITmutex);  
  
  _tickDebut = osg::Timer::instance()->tick();
}//@CODE_490


void TimeBar::stop()
{//@CODE_498
  _stop =true;
  _pause = false;

  SDL_LockMutex(_clefITmutex); 
  _clefIT = _clefs.begin();
  SDL_UnlockMutex(_clefITmutex); 
  
  _tpsSauv = 0.0;
}//@CODE_498


// End of Group: Media


//
// Group: load
//
bool TimeBar::loadXML(const xmlpp::Node* node, LoaderXML* loaderXML)
{//@CODE_3038

  _loader = loaderXML;

  XMLUtil xml(node);
  
  setName( xml.getFirstText("nom") );
  _timeSize = atof( xml.getFirstText("timeSize").c_str() );
  _loop = xml.getFirstText("loop") == "true";
  _stop = xml.getFirstText("playOnStart") == "false";

 
  vector<const xmlpp::Element*> commands= xml.getAllElement("command");
  Command * com = NULL;
  string code;
  double time;

  for (unsigned int i=0; i< commands.size();i++) {
    
    xml.set( commands[i] ); //on un objet xml sur le noeud command
    
    time = atof( xml.getFirstText("time").c_str() );
    code = xml.getFirstText("code");
    
    Evaluable * codeTmp = loaderXML->getBD()->FindEvaluable( code );
    //BUG_C++_JVM sur dynamic_cast quand load fonction interne depuis Editeur
    //_codeExec =dynamic_cast< Command * >(evTmp);
    com  = static_cast< Command * >(codeTmp);    
  
    if (!com){
      CERR(this,"loadXML")<<"la commande "<<code
			  <<" n'existe pas ->destruction"
			  <<endl;
      this->~TimeBar();
      return false;
    }
    addCommand ( time , com );
    
  }  

  if (! _stop ){
    play();
  }

  return true;
}//@CODE_3038


/*@NOTE_468
Destructor method.
*/
TimeBar::~TimeBar()
{//@CODE_468
    DestructorInclude();
    
    SDL_DestroyMutex(_clefITmutex);

    // Put in your own code
    CTRACE(this,"~TimeBar")<<"destruction thread TimeBar : "
			   <<this<<" OK"<<endl;
    
}//@CODE_468


/*@NOTE_473
Constructor method.
*/
TimeBar::TimeBar(BD* pBD, double longu, LoaderGeneric* loader) //@INIT_473
    : ThreadSdl(pBD)
      , _clefIT()
      , _clefITmutex(NULL)
      , _clefs()
      , _timeSize(longu)
      , _loop(false)
      , _pause(false)
      , _stop(true)
      , _tickDebut()
      , _tpsSauv(0.0)
      , _loader( loader)  
{//@CODE_473
    ConstructorInclude();
    
    CTRACE(this,"TimeBar")<<"new TimeBar "<<this<<endl;
    //assertC(_clefs.size());
    //_clefIT=_clefs.begin();

    ////start();
    
    _clefITmutex = SDL_CreateMutex();
    start();

}//@CODE_473


Evaluable* TimeBar::staticCreate(BD* pBD)
{//@CODE_3037
    return new TimeBar(pBD);
}//@CODE_3037


// End of Group: load

void TimeBar::addCommand(double t, Command* c)
{//@CODE_481
  
  CTRACE(this, "addCommand")<<"time ="<<t<<" command="<<c->getName()<<endl;

  if (t > _timeSize){
    CWARN(this,"addCommand")<<" t > _timeSize !! _timeSize="
			    <<_timeSize<<" ("<<getName()<<")"<<endl;
  }

  
  MapClef::iterator it=_clefs.find(t);
  if (it != _clefs.end()) {
    it->second.push_back(c);
  }
  else {
    Clef cl;
    cl.push_back(c);
    _clefs[t]=cl;
  }
  
  //_clefs.sort(); en espérant que les clefs sont automatiq classées
  
}//@CODE_481


void TimeBar::run()
{//@CODE_475
    
      
  osg::Timer_t tickFin;
  double tpsRelatif;
  

  CTRACE(this,"run2")<<"TimeBar "<<_timeSize<<"s thread en attende"<<endl;

  if ( _loader != NULL ){
    while (!_loader->isLoadFinished() 
            && _vivant) {
        SDL_Delay(100);//YieldCurrentThread();
        CTRACE(this,"run2")<<"TimeBar thread is standing "<<endl;    
    }
  }
  
  CTRACE(this,"run2")<<"TimeBar "<<_timeSize<<"s thread lancée"<<endl;

  while (_vivant){
    if (!_stop && !_pause){
      //_tickDebut = osg::Timer::instance()->tick();
      tpsRelatif = 0;
      //cout<<"TimeBar pass1"<<endl;
 
      while (tpsRelatif < _timeSize && _vivant &&!_stop && !_pause){ 
	
	//cout<<"TimeBar pass2"<<endl;
	tickFin = osg::Timer::instance()->tick();
	tpsRelatif = osg::Timer::instance()->delta_s(_tickDebut,tickFin)+_tpsSauv;
	//bool b=_clefIT != _clefs.end();
	//b=tpsRelatif >= _clefIT->first;
	//cout<<"TimeBar pass3"<<endl;

	if (_clefIT != _clefs.end() && tpsRelatif >= _clefIT->first){
	  //cout<<"TimeBar pass4"<<endl;
	  for(unsigned int i=0;
              i<_clefIT->second.size() && (!_stop && !_pause);
              i++) {
	    SDL_LockMutex(_clefITmutex); 
	    if (!_stop && !_pause){
                _clefIT->second[i]->exec();
            }
	    //cout<<"TimeBar pass4"<<endl;
	    SDL_UnlockMutex(_clefITmutex); 
	  }
	  //cout<<"TimeBar pass5"<<endl;
	  //CTRACE(this,"run2")<<"av _clefIT"<<endl;
	  _clefIT++;
	}
	else {
	  //cout<<"TimeBar pass6"<<endl;
	  SDL_Delay(33);  ////YieldCurrentThread();     
	}
      }
      
      if (!_stop && !_pause && _loop) { stop(); play(); }
      
    }
    
 
    ////YieldCurrentThread();  
    SDL_Delay(33);   
  }
  CTRACE(this,"run2")<<"TimeBar "<<_timeSize<<"s thread arreté"<<endl;
  
}//@CODE_475


void TimeBar::vider()
{//@CODE_656
  if (!_stop)
    stop();
  
  SDL_LockMutex(_clefITmutex); 
    
  MapClef::iterator it=_clefs.begin(); //iterateur sur map
  Clef::iterator itc;                  //iterateur sur veceur de clefs 
  while (it != _clefs.end()){
    itc = it->second.begin();
    while (itc != it->second.end()) {
      delete (*itc); //appel du destructeur de Command
      itc++;
    }
    it++;
  }
  _clefs.erase(_clefs.begin(),_clefs.end()); //on vide la map

  SDL_UnlockMutex(_clefITmutex); 
}//@CODE_656


//{{AFX DO NOT EDIT CODE BELOW THIS LINE !!!

/*@NOTE_469
Method which must be called first in a constructor.
*/
void TimeBar::ConstructorInclude()
{
}


/*@NOTE_470
Method which must be called first in a destructor.
*/
void TimeBar::DestructorInclude()
{
}


// Methods for the relation(s) of the class

//}}AFX DO NOT EDIT CODE ABOVE THIS LINE !!!

//@START_USER3
//attention CB fait parfois sauter ce crochet fermant...
} //end of namespace cooki3d

