/** -*-c++-*- *************************************************************\
* (Remise  zro le 24/10/04, 23/06/06, 14/11/06)
* File:          TimeBar.h
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of class 'TimeBar'
*
* Modifications: @INSERT_MODIFICATIONS(* )
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/
#ifndef _TIMEBAR_H
#define _TIMEBAR_H

//@START_USER1
#include <map>
#include <vector>
#include <osg/Timer>
#include <SDL/SDL_mutex.h>

#include "CookiMetier.h"
#include "Media.h"
#include "Threadd.h"
#include "Command.h"
#include "ThreadSdl.h"
#include "LoaderGeneric.h"

namespace cooki3d{
//@END_USER1


/*@NOTE_467
class ...
*/

class AFX_EXT_CLASS TimeBar
    : public Media
    , public ThreadSdl
{

//@START_USER2
public:

  typedef  vector<Command *> Clef;

  typedef map<double,Clef,less<double > > MapClef;
  
    
//@END_USER2

//
// Group: ClassBuilder methods
//

private:
    void ConstructorInclude();
    void DestructorInclude();

//
// Group: Media
//

public:
    virtual double eval(std::string methode, double args[10], int nbParam);
    virtual double getDuration();
    virtual double getPosition();
    virtual bool isPause();
    virtual bool isPlay();
    virtual void pause();
    virtual void play();
    virtual void setDuration(double duree);
    virtual void setPosition(double sec);
    virtual void stop();

//
// Group: load
//

public:
    virtual bool loadXML(const xmlpp::Node* node, LoaderXML* loaderXML);
    virtual ~TimeBar();
    TimeBar(BD* pBD, double longu = 10.0, LoaderGeneric* loader = NULL);
    static Evaluable* staticCreate(BD* pBD);

//
// Group: data
//

private:
    MapClef::iterator _clefIT;
    SDL_mutex* _clefITmutex;
    MapClef _clefs;
    double _timeSize;
    bool _loop;
    bool _pause;
    bool _stop;
    osg::Timer_t _tickDebut;
    double _tpsSauv;
    LoaderGeneric* _loader;

public:
    double getTimeSize() const;
    void setTimeSize(double timeSize);
    bool getLoop() const;
    void setLoop(bool loop);

//
// Non-Grouped Members
//

//
// Non-Grouped Methods
//

public:
    void addCommand(double t, Command* c);
    virtual void run();
    void vider();
};

#endif


#ifdef CB_INLINES
#ifndef _TIMEBAR_H_INLINES
#define _TIMEBAR_H_INLINES


//
// Group: data
//
/*@NOTE_963
Returns the value of member '_timeSize'.
*/
inline double TimeBar::getTimeSize() const
{//@CODE_963
    return _timeSize;
}//@CODE_963



/*@NOTE_964
Set the value of member '_timeSize' to 'timeSize'.
*/
inline void TimeBar::setTimeSize(double timeSize)
{//@CODE_964
    _timeSize = timeSize;
}//@CODE_964



/*@NOTE_960
Returns the value of member '_loop'.
*/
inline bool TimeBar::getLoop() const
{//@CODE_960
    return _loop;
}//@CODE_960



/*@NOTE_961
Set the value of member '_loop' to 'loop'.
*/
inline void TimeBar::setLoop(bool loop)
{//@CODE_961
    _loop = loop;
}//@CODE_961



// End of Group: data

//@START_USER3
} //end of namespace cooki3d
//@END_USER3

#endif
#endif
