/** -*-c++-*- *************************************************************\
* File:          sigseg.cpp
* Creation date: November 14, 2006 00:37
* Author:        Bertrand Greslier
*                Entreprise Individuelle
* Purpose:       Declaration of function 'signal_segv, setup_sigsegv'
*
* Copyright (C) 2006, Bertrand Greslier
* This file is part of Cooki3d API
*
* This file may be used under the terms of the GNU General Public
* License version 2.0 as published by the Free Software Foundation
* and appearing in the file LICENSE.GPL included in the packaging of
* this file.  
*
* If you are unsure which license is appropriate for your use, please
* review the following information:
* www.cooki3d.org => licence or 
* contact the author at bertrand.greslier@gmail.com.
*
* This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
* WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*
\**************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include <signal.h>

#ifdef UNIX

#include <ucontext.h>
#include <dlfcn.h>
#include <execinfo.h>

#if defined(REG_RIP)
# define SIGSEGV_STACK_IA64
# define REGFORMAT "%016lx"
#elif defined(REG_EIP)
# define SIGSEGV_STACK_X86
# define REGFORMAT "%08x"
#else
# define SIGSEGV_STACK_GENERIC
# define REGFORMAT "%x"
#endif

#include <string>
#include <iostream>
extern "C" void printStackTrace( std::string & os);

extern "C"
 static void signal_segv(int signum, siginfo_t* info, void*ptr) {
    static const char *si_codes[3] = {"", "SEGV_MAPERR", "SEGV_ACCERR"};

    size_t i;
    ucontext_t *ucontext = (ucontext_t*)ptr;

#if defined(SIGSEGV_STACK_X86) || defined(SIGSEGV_STACK_IA64)
    int f = 0;
    Dl_info dlinfo;
    void **bp = 0;
    void *ip = 0;
#else
    void *bt[20];
    char **strings;
    size_t sz;
#endif

    std::cout<<"Segmentation Fault!\n"<<std::flush;
        
    std::string s;
    printStackTrace(s);
    std::cout<< s << std::endl;
    
    
    std::cout<< "info.si_signo = "<< signum<<std::endl;
    std::cout<< "info.si_errno = "<< info->si_errno<<std::endl;
    std::cout<< "info.si_code  = "<<info->si_code
	     <<", "<<si_codes[info->si_code]<<std::endl;
    std::cout<< "info.si_addr  = (%pTODO) "<< info->si_addr<<std::endl;
      

#if defined(SIGSEGV_STACK_X86) || defined(SIGSEGV_STACK_IA64)

# if defined(SIGSEGV_STACK_IA64)
    ip = (void*)ucontext->uc_mcontext.gregs[REG_RIP];
    bp = (void**)ucontext->uc_mcontext.gregs[REG_RBP];
# elif defined(SIGSEGV_STACK_X86)
    ip = (void*)ucontext->uc_mcontext.gregs[REG_EIP];
    bp = (void**)ucontext->uc_mcontext.gregs[REG_EBP];
# endif
   

    std::cout<< "------------\n"<<std::endl;
    //TODO :don't use stdout to redirect logs correctly (see MainApp:498)
    fprintf(stdout, "Stack trace complex:\n");

    while(bp && ip) {
      
      if(!dladdr(ip, &dlinfo))
	break;
      
      fprintf(stdout, "% 2d: %p <%s+%u> (%s)\n",
		++f,
	      ip,
	      dlinfo.dli_sname, /* function name */
	      (unsigned int)(int(ip) - int(dlinfo.dli_saddr)),
	      dlinfo.dli_fname);
    
      if(dlinfo.dli_sname && !strcmp(dlinfo.dli_sname, "main"))
	break; 
      
      ip = bp[1];
      bp = (void**)bp[0];
      
    }
#else
    fprintf(stdout, "Stack trace (non-dedicated):\n");
    sz = backtrace(bt, 20);
    strings = backtrace_symbols(bt, sz);
    
    for(i = 0; i < sz; ++i)
        fprintf(stdout, "%s\n", strings[i]);
#endif
    fprintf(stdout, "End of stack trace\n");

    //REGISTERS
    for(i = 0; i < NGREG; i++) {
        fprintf(stdout, "reg[%02d]       = 0x" REGFORMAT "\n", 
		i, ucontext->uc_mcontext.gregs[i]);
    }

    exit (-1);
}

extern "C" 
int setup_sigsegv() {
    struct sigaction action;
    memset(&action, 0, sizeof(action));
    action.sa_sigaction = signal_segv;
    action.sa_flags = SA_SIGINFO;
    if(sigaction(SIGSEGV, &action, NULL) < 0) {
        perror("sigaction");
        return 0;
    }
    return 1;
}

/* POSE PROBLEM AVEC JAVA !!! */
// #ifndef SIGSEGV_NO_AUTO_INIT 
//extern "C"
//static void __attribute((constructor)) init(void) { 
//  fprintf(stdout,"???????????????????????"); 
//  setup_sigsegv(); 
//} 
//#endif 

#else //UNIX

extern "C" 
int setup_sigsegv() {

  //#error "STACK TRACE ON SIGSEGV IS NOT IMPLEMENTED FOR WINDOWS!"
  printf("(WARNING) STACK TRACE ON SIGSEGV IS NOT IMPLEMENTED FOR WINDOWS!\n");
  return -1;
}


#endif //UNIX

